#! /bin/bash
########################################################################
# this script ATTEMPTS to convert the XML-based savefile from 
# Gnofin 0.8.4 (and above?) to QHacc 2.2 savefiles. The syntax is 
#
# convert_gnofin084_to_qhacc22.sh <gnofin savefile> [paired]
#
# if paired is given, the script will attempt to convert Gnofin's
# single-entry transactions into QHacc's double-entry system
#
# This script is not perfect. It (currently) has the following pitfalls:
#  o the double-entry stuff assumes you don't have any transfers. (Strip
#    them out before running this script.)
#  o Account notes are not maintained.
#  o Nested Gnofin categories are not converted to nested QHacc accounts.
#    (They are converted to regular accounts, which must then be re-nested.)
#  o Transaction types are completely ignored.
#  o Accounts with names identical to Gnofin categories will be duplicated
#    and generally hork up the resulting files. Make sure your Gnofin account
#    names are unique from category names.
# 
# Please feel free to make this script better, and also to send me an
# updated copy if you do.
########################################################################

if [ $# -lt 1 ]; then
  echo "Syntax: $0 <Gnofin 8.4 save file>"
  exit 1
fi

typeset -i CACCT=0
typeset -i CTRANS=0
typeset -i PAIR=0

if [ $# -gt 1 ]; then
  PAIR=1
fi

#rm -f accounts transactions preferences

cat $1| while read line
do
  typeset ACCT=$(echo $line| grep 'fin:account name=')
  if [ x"$ACCT" != "x" ]; then
    CACCT=$CACCT+1
    typeset ANAME=$(echo $line| cut -d \" -f2)
    echo "Account $CACCT: $ANAME"
    echo "$CACCT||$ANAME||||||||||||||||||||||||||||||||||||||||||N||Y||0||0" >> accounts
  fi

  ACCT=$(echo $line| grep 'fin:category name=')
  if [ x"$ACCT" != "x" ]; then
    CACCT=$CACCT+1
    typeset ANAME=$(echo $line| cut -d \" -f2)
    echo "Account $CACCT: $ANAME"
    echo "$CACCT||$ANAME||||||||||||||||||||||||||||||||||||||||||Y||Y||0||0" >> accounts
  fi

# first pass at converting transactions
  typeset REC=$(echo $line| grep 'fin:record ')
  if [ x"$REC" != "x" ]; then
    CTRANS=$CTRANS+1
#    echo "new trans: $CTRANS $line in account $CACCT"
    typeset TRANS=$(echo $line|awk -F\" '{
      num=$10
      if( num==0 ) num=""
	    amt=$18/100
	    reco=$20
	    if( reco==1 ) reco=2
	    printf "%s||%s||%s||%s||%s/%s/%s||%s\n",num,$14,$16,amt,$4,$2,$6,reco }'
		)
		
		echo "$CTRANS||$TRANS||$ANAME" >> tr.tmp
		if [ $PAIR -gt 0 ]; then
			CTRANS=$CTRANS+1
			typeset PTRANS=$(echo $TRANS|awk -F\| '{
        amt=0-$7
        printf "%s||%s||%s||%s||%s||%s",$1,$3,$5,amt,$9,$11 }'
			)
		  echo "$CTRANS||$PTRANS||"$( echo $line|cut -d\" -f12) >> tr.tmp
		fi
  fi
done

# now we have all the transactions in tr.tmp, but we
# need to coallate the account names to account ids
#cat tr.tmp
cat tr.tmp|while read line;
do
  typeset ACCTN=$(echo $line|cut -d\| -f15)
	typeset AID=$(grep "|$ACCTN|" accounts|cut -d\| -f1)
	typeset TRANS=$(echo $line|awk -v AID=$AID -F\| '{
    printf "%s||%s||%s||%s||%s||%s||%s||%s",$1,$3,$5,amt,$9,$11,$13,AID }'
	)
	
	if [ $PAIR -gt 0 ]; then
		echo "$TRANS||$PAIR||1" >> transactions
		typeset -i TID=$(echo $line|cut -d\| -f1)
		typeset -i TEVEN=$TID%2
		if [ $TEVEN -eq 0 ]; then
			PAIR=$PAIR+1
		fi
  else
		echo "$TRANS||0||1" >> transactions
	fi
done

if [ $PAIR -gt 0 ]; then
  echo "DOUBLEENTRY||Y" >> preferences
fi

rm -f tr.tmp
