#! /bin/bash
##################################################################
# qifimport_to_23.sh
#   this script ATTEMPTS to import a Quicken Interchange Formatted
#   file into QHacc v2.3. It uses the format in
#   http://bbcash.sourceforge.net/_QIF-format.html, but the format
#   is available all over the net. By Intuit's own admission, IT
#   IS NOT A GOOD FORMAT for online transactions, and many people
#   have discovered that there are discrepancies between the
#   published standard and what actually works. Also, I don't
#   actually have Quicken, so this script is based on the examples
#   I have found in various places on the web.
#   
#   YOUR RESULTS WILL VARY
#   
#   QHacc doesn't support all the features of Quicken, so this
#   script will only import enough data for QHacc to use.
#   
#   
#   
##################################################################

if [ $# -gt 3 -o $# -lt 2 ]; then
  echo "Syntax: $0 <QIF file> <Account Identifier> [QHACC_HOME]"
	exit 1
fi

FILE=$1
ACCT=$2
HOME=$3
if [ x"$HOME" = "x" ]; then
  HOME=$QHACC_HOME
fi
if [ x"$HOME" = "x" ]; then
  echo "QHACC_HOME must be set, or given on the command line"
  echo "Syntax: $0 <QIF file> <Account Identifier> [QHACC_HOME]"
	exit 1	
fi

RM="rm -f"
SORT1="sort -n -r -t |"
SORT2="sort -n -r"
HEAD="head -1"
GREP=grep
CAT=cat
CUT=cut
SED=sed
AWK=awk
TMP=tmpqif.$$
TMPACCTS=$TMP.1
SPLITS=$TMP.2
QTRANS=$TMP.3

$RM $SPLITS $QTRANS transactions

typeset -i TOPID=$($SORT1 $HOME/transactions|$HEAD|$CUT -d\| -f1)
typeset -i TOPSG=$($SORT1 -k 17 $HOME/transactions|$HEAD|$CUT -d\| -f17)

# increment the topid
TOPID=$TOPID+1
TOPSG=$TOPSG+1

# this is all the data we'll use
typeset DATE=""
typeset MEMO=""
typeset RECO=""
typeset PAYEE=""
typeset NUM=""
typeset AMT=""
typeset CSPLIT=""
typeset CSPLITD=""

typeset -i LINE=0
$CAT $FILE | while read line; do
  LINE=$LINE+1
		
	# skip the header line of the file
  if [ $LINE -ne 1 ]; then
		# parse the data line
		typeset F=$(echo $line|cut -c1)
		typeset TEXT=$(echo $line|cut -c2-)

		case $F in
		  'D') DATE=$TEXT;;

			'T') AMT=$TEXT;;

			'C')
				 if [ x"$TEXT" = "xR" ]; then
				   RECO="2"
				 else
				   RECO="0"
				 fi
				 ;;

			'N') NUM=$TEXT;;

			'P') PAYEE=$TEXT;;

			'M') MEMO=$TEXT;;

			'S') TOPID=$TOPID+1
					 CSPLIT=$TEXT
				 ;;

			'L')
					 TOPID=$TOPID+1
					 CSPLIT=$TEXT
				 ;;

			'$') echo "$TOPID $TOPSG $TEXT $CSPLIT" >> $SPLITS;;

			'^')
				 # this is the end of the transaction, so write what we have so far
				 TOPID=$TOPID+1
				 echo "$TOPID $TOPSG $AMT $ACCT" >> $SPLITS
				 echo "$TOPSG $NUM||$PAYEE||$MEMO||XXXAMTXXX||$DATE||$RECO" >> $QTRANS
				 TOPSG=$TOPSG+1

				 # reset all our transaction information for the next transaction
				 DATE=""
				 MEMO=""
				 RECO=""
				 PAYEE=""
				 NUM=""
				 AMT=""
				 CSPLIT=""
				 CSPLITD=""

				 ;;
		esac
	fi
done

# at this point, we should have QTRANS containing most of the transaction
# information, and SPLITS containing the split information. All that's left
# is to figure out the IDs from the account names, and put the data into a
# QHacc-readable format

# first, create an account lookup
$AWK -F\| '{print $1,$3 }' $QHACC_HOME/accounts > $TMP
$SORT2 $TMP > $TMPACCTS
typeset -i TOPACCT=$($HEAD $TMPACCTS|$CUT -d\  -f1)

# Quicken likes to put some stuff in [], but grep will treat these
# like regular expressions, so get rid of them
$SED -e "s/\[//g" -e "s/\]//g" $SPLITS > $TMP
mv $TMP $SPLITS

# we're ready to make the transactions
cat $SPLITS|while read TID TSG TAMT ACCT; do
  $GREP ^$TSG $QTRANS > $TMP
  typeset DATA=$($CAT $TMP|$CUT -d\  -f2-)

	typeset TACCT=$($GREP $ACCT$ $TMPACCTS|cut -d\  -f1)
	
	# we didn't find a valid account, so make one
	if [ x"$TACCT" = "x" ]; then
		TOPACCT=$TOPACCT+1
		echo "$TOPACCT||$ACCT||||||||||||||||||||||||||||||||||||||||||Y||Y||3||0" >> accounts
		TACCT=$TOPACCT
	fi
	
	echo "$TID||$DATA||$TACCT||$TSG||1"| $SED "s/XXXAMTXXX/$TAMT/" >> transactions
done


$RM $TMP $TMPACCTS $QTRANS $SPLITS

echo "To finish the import, run the following commands:"
echo
echo "  $CAT transactions >> $QHACC_HOME/transactions"
echo "  $CAT accounts >> $QHACC_HOME/accounts"
echo
echo "These commands will copy the newly-created transactions to your"
echo "QHacc datafiles. You should backup these files first."
