/************************** * * * * * * * * * * * * ***************************
    Copyright (c) 1999-2003 Ryan Bobko
                       ryan@ostrich-emulators.com

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.     
************************** * * * * * * * * * * * * ***************************/

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include "qhacc.h"
#include "qhaccutils.h"
#include "qhaccwidget.h"

#include <stdlib.h>
#include <getopt.h>
#include <locale.h>

#include <qapplication.h>
#include <qtextstream.h>
#include <qtextcodec.h>

void showIntro( const char * progname ){
  cout<<"QHacc version "<<VERSION;
  cout<<", Copyright (C) 2002 Ryan Bobko <ryan@ostrich-emulators.com>"<<endl
      <<"QHacc comes with ABSOLUTELY NO WARRANTY; for details type `"
      <<progname<<" -w'."<<endl
      <<"This is free software, and you are welcome to redistribute it"
			<<" under"<<endl
      <<"certain conditions; Please see the GNU Public License "
      <<"for details."<<endl;
}

void helpQuit( const char * progname ){
  cout<<"usage: "<<progname<<endl;
  cout<<" [-f             <QHACC_HOME>]          set QHACC_HOME for loading"<<endl
      <<" [-w]                                   show warranty information"<<endl
			<<" [--debug        <Level>]               set debugging output"<<endl
      <<" [--cron         <Transaction File>]    comand-line insert"<<endl
      <<" [--archive      <Date String|Account>] archive transactions"<<endl
      <<" [--restore      <Archived File>]       restore transactions"<<endl
      <<" [--preport      <Account>]             \"Totals By Payee\" report"<<endl
      <<" [--breport      <Account>]             \"Monthly Balances\" report"<<endl
      <<" [--dreport      <Account>]             \"Monthly Deltas\" report"<<endl
      <<" [--areport      <Account>]             \"Averages\" report"<<endl
			<<" [--budget       <Account>]             print out of budget accounts"<<endl
      <<" [--transactions <Account>]             list transactions for account"<<endl
			<<" [--journal      <Ledger>]              print journal of ledger"<<endl
      <<" [--accounts]                           list accounts and balances"<<endl
      <<" [--version]                            show program information and exit"<<endl
      <<" [--help]                               show this screen"<<endl<<endl;
  exit ( EXIT_SUCCESS );
}

char * opt( int argc, char ** argv, int& i ){
	i++;
	if( i<argc ) return argv[i];
	else helpQuit( argv[0] );
	return 0;
}

bool cmp( const char * model, const char * test ){
	return ( strcmp( model, test )==0 );
}

int main( int argc, char **argv ){
	const char * home=getenv( "QHACC_HOME" );
	const char * langdir=getenv( "QHACC_LANGDIR" );
	bool intro=true;
	bool visual=true;

	/*
	char * optarg=0;
	int result=0, oidx=0;
	static struct option longOpts[]={
		{ "cron",         1, 0, 1 },
		{ "preport",      1, 0, 2 },
		{ "areport",      1, 0, 3 },
		{ "beport",       1, 0, 4 },
		{ "dreport",      1, 0, 5 },
		{ "preport",      1, 0, 6 },
		{ "budget",       1, 0, 7 },
		{ "journal",      0, 0, 8 },
		{ "archive",      1, 0, 9 },
		{ "restore",      1, 0, 10 },
		{ "accounts",     1, 0, 11 },
		{ "transactions", 1, 0, 12 },
		{ "debug",        1, 0, 13 },
		{ 0, 0, 0, 0 }
	};

	result=getopt_long( argc, argv, "f:w", longOpts, &oidx );
	switch( result ){
	case '?':
		helpQuit( argv[0] );
		break;
	default:
		cout<<"getopt returned: "<<result<<"..."<<longOpts[oidx].name<<"  ";
		if( optarg ) cout<<optarg;
		cout<<endl;
		
		break;
	};
	*/

	// first, see if we should display the header
	for( int i=0; i<argc; i++ ){
		if( cmp( argv[i], "-w" ) ||
				cmp( argv[i], "--cron" ) ||
				cmp( argv[i], "--preport" ) ||
				cmp( argv[i], "--breport" ) ||
				cmp( argv[i], "--dreport" ) ||
				cmp( argv[i], "--areport" ) ||
				cmp( argv[i], "--budget" )  ||
				cmp( argv[i], "--journal" ) ||
				cmp( argv[i], "--archive" ) ||
				cmp( argv[i], "--restore" ) ||
				cmp( argv[i], "--accounts" ) ||
				cmp( argv[i], "--transactions" ) ){
			
			intro=false;
			break;
		}
	}

	if( intro ) showIntro( argv[0] );

	QHacc qhacc;
	// now that that's out of the way, actually parse the command line
	for( int i=1; i<argc; i++ ){
		if( cmp( argv[i], "--debug" ) ){
			Utils::setDebug( QString( opt( argc, argv, i ) ).toInt() );
		}
		else if( cmp( argv[i], "-f" ) ){
			home=opt( argc, argv, i );
		}
		else if( cmp( argv[i], "-l" ) ){
			langdir=opt( argc, argv, i );
		}
		else if( cmp( argv[i], "-w" ) ){
      cout<<"                                NO WARRANTY\n    BECAUSE THE PROGRAM IS LICENSED FREE OF CHARGE, THERE IS NO WARRANTY\nFOR THE PROGRAM, TO THE EXTENT PERMITTED BY APPLICABLE LAW.  EXCEPT WHEN\nOTHERWISE STATED IN WRITING THE COPYRIGHT HOLDERS AND/OR OTHER PARTIES\nPROVIDE THE PROGRAM \"AS IS\" WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED\nOR IMPLIED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF\nMERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.  THE ENTIRE RISK AS\nTO THE QUALITY AND PERFORMANCE OF THE PROGRAM IS WITH YOU.  SHOULD THE\nPROGRAM PROVE DEFECTIVE, YOU ASSUME THE COST OF ALL NECESSARY SERVICING,\nREPAIR OR CORRECTION.\n    IN NO EVENT UNLESS REQUIRED BY APPLICABLE LAW OR AGREED TO IN WRITING\nWILL ANY COPYRIGHT HOLDER, OR ANY OTHER PARTY WHO MAY MODIFY AND/OR\nREDISTRIBUTE THE PROGRAM AS PERMITTED ABOVE, BE LIABLE TO YOU FOR DAMAGES,\nINCLUDING ANY GENERAL, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES ARISING\nOUT OF THE USE OR INABILITY TO USE THE PROGRAM (INCLUDING BUT NOT LIMITED\nTO LOSS OF DATA OR DATA BEING RENDERED INACCURATE OR LOSSES SUSTAINED BY\nYOU OR THIRD PARTIES OR A FAILURE OF THE PROGRAM TO OPERATE WITH ANY OTHER\nPROGRAMS), EVEN IF SUCH HOLDER OR OTHER PARTY HAS BEEN ADVISED OF THE\nPOSSIBILITY OF SUCH DAMAGES."<<endl<<endl;
		}
		else if( cmp( argv[i], "--help" ) ) helpQuit( argv[0] );
		else if( cmp( argv[i], "--preport" ) ){
			qhacc.report( home, QHacc::TOTALS, opt( argc, argv, i ) );
		}
		else if( cmp( argv[i], "--dreport" ) ){
			qhacc.report( home, QHacc::MONTHLYD, opt( argc, argv, i ) );
		}
		else if( cmp( argv[i], "--breport" ) ){
			qhacc.report( home, QHacc::MONTHLYB, opt( argc, argv, i ) );
		}
		else if( cmp( argv[i], "--areport" ) ){
			qhacc.report( home, QHacc::AVES, opt( argc, argv, i ) );
		}
		else if( cmp( argv[i], "--budget" ) ){
			qhacc.report( home, QHacc::BUDGET, opt( argc, argv, i ) );
		}
		else if( cmp( argv[i], "--journal" ) ){
			qhacc.report( home, QHacc::JOURNAL, opt( argc, argv, i ) );
		}
		else if( cmp( argv[i], "--archive" ) ){
			qhacc.archive( home, true, opt( argc, argv, i ) );
		}
		else if( cmp( argv[i], "--restore" ) ){
			qhacc.archive( home, false, opt( argc, argv, i ) );
		}
		else if( cmp( argv[i], "--transactions" ) ){
			qhacc.showTrans( home, opt( argc, argv, i ) );
		}
		else if( cmp( argv[i], "--accounts" ) ){
			qhacc.showAccounts( home );
		}
		else if( cmp( argv[i], "--cron" ) ){
			qhacc.cron( home, opt( argc, argv, i ) );
		}
		else if( cmp( argv[i], "--version" ) ){
			visual=false;
      cout<<"\ncompile-time options:"<<endl;
      cout<<"  + Graphing and Reporting"<<endl;
			cout<<"  + Debugability"<<endl;
#ifdef QHACCMYSQL
			cout<<"  + MySQL Connectivity"<<endl;
#endif
      cout<<endl;
		}
		else helpQuit( argv[0] );
	}

	if( intro && visual ){
		if( !qhacc.setHome( home ) ) return EXIT_FAILURE;
		QApplication a( argc, argv );

		QTranslator translator( 0 );
		
		if( !langdir ) langdir=".";

		std::ostream * str=0;
		if( Utils::debug( 4, str ) ) 
			*str<<"locale is "<<QTextCodec::locale()<<"; QHACC_LANDGIR is "
					<<langdir<<endl;

		setlocale( LC_ALL, "" );
		translator.load( QString( "qhacc_" )+QTextCodec::locale(), langdir );
		a.installTranslator( &translator );

		QHaccWidget * q = new QHaccWidget( &qhacc );
		q->show();
		a.connect( &a, SIGNAL( lastWindowClosed() ), &a, SLOT( quit() ) );
		return a.exec();
	}
	return 0;
}
