/************************* * * * * * * * * * * * * ***************************
    Copyright (c) 1999-2003 Ryan Bobko
                       ryan@ostrich-emulators.com

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.     
************************** * * * * * * * * * * * * **************************/

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#ifdef QHACCMYSQL
#include <sqlplus.hh>
#endif

#include "qhacc.h"
#include "qhaccutils.h"
#include "qhacclineedits.h"
#include "qhaccsegmenter.h"
#include "qhaccreportwriter.h"
#include "qhacc.moc"

#include <math.h>
#include <stdlib.h>
#include <sys/stat.h> 

#include <qfile.h>

const int QHacc::TID=           0;
const int QHacc::TNUM=          1;
const int QHacc::TPAYEE=        2;
const int QHacc::TMEMO=         3;
const int QHacc::TSUM=          4;
const int QHacc::TDATE=         5;
const int QHacc::TRECO=         6;
const int QHacc::TACCTID=       7;
const int QHacc::TSPLITGROUP=   8;
const int QHacc::TLID=          9;
const int QHacc::TCOLS=         10;
const char * QHacc::TCOLNAMES[]={ "ID", "NUM", "PAYEE", "MEMO", "SUM", "DATE",
																	"RECO", "ACCTID", "SPLITGROUP", "LEDGERID" };
const ColType QHacc::TCOLTYPES[]={ CTUINT, CTSTRING, CTSTRING, CTSTRING, CTSTRING, CTDATE, CTUINT, CTUINT, CTUINT, CTUINT };

const int QHacc::AID=           0;
const int QHacc::ANAME=         1;
const int QHacc::AOBAL=         2;
const int QHacc::ACBAL=         3;
const int QHacc::ARBAL=         4;
const int QHacc::ABUDGET=       5;
const int QHacc::ATRANSNUMS=    6;
const int QHacc::ADEFAULTNUM=   7;
const int QHacc::ANUM=          8;
const int QHacc::AINSTNAME=     9;
const int QHacc::AINSTADDR1=    10;
const int QHacc::AINSTADDR2=    11;
const int QHacc::AINSTCITY=     12;
const int QHacc::AINSTSTATE=    13;
const int QHacc::AINSTZIP=      14;
const int QHacc::AINSTPHONE=    15;
const int QHacc::AINSTFAX=      16;
const int QHacc::AINSTEMAIL=    17;
const int QHacc::AINSTCONTACT=  18;
const int QHacc::AINSTNOTES=    19;
const int QHacc::ACATEGORY=     20;
const int QHacc::ATAXED=        21;
const int QHacc::ATYPE=         22;
const int QHacc::APID=          23;
const int QHacc::ACOLS=         24;
const char * QHacc::ACOLNAMES[]={ "ID", "NAME", "OBAL", "CBAL", "RBAL",
																	"BUDGET", "TRANSNUMS", "DEFAULTNUM",
																	"NUM", "INSTNAME",	"INSTADDR1",
																	"INSTADDR2", "INSTCITY", "INSTSTATE",
																	"INSTZIP", "INSTPHONE", "INSTFAX",
																	"INSTEMAIL", "INSTCONTACT", "INSTNOTES",
																	"CATEGORY", "TAXED", "TYPE", "PID" };
const ColType QHacc::ACOLTYPES[]={ CTUINT, CTSTRING, CTSTRING, CTSTRING,
																	 CTSTRING, CTSTRING, CTSTRING, CTSTRING,
																	 CTSTRING, CTSTRING, CTSTRING, CTSTRING,
																	 CTSTRING, CTSTRING, CTSTRING, CTSTRING,
																	 CTSTRING, CTSTRING, CTSTRING, CTSTRING,
																	 CTBOOL, CTBOOL, CTUINT, CTUINT };

const QString QHacc::ASEP="::";


const int QHacc::PPREF=  0;
const int QHacc::PVALUE= 1;
const int QHacc::PCOLS=  2;
const char * QHacc::PCOLNAMES[]={ "PREF", "VALUE" };
const ColType QHacc::PCOLTYPES[]={ CTSTRING, CTSTRING };


const int QHacc::YID=  0;
const int QHacc::YNAME=1;
const int QHacc::YLP=  2;
const int QHacc::YCOLS=3;
const char * QHacc::YCOLNAMES[]={ "ID", "NAME", "LEFTPLUS" };
const ColType QHacc::YCOLTYPES[]={ CTUINT, CTSTRING, CTBOOL };

const int QHacc::LID=  0;
const int QHacc::LNAME=1;
const int QHacc::LDESC=2;
const int QHacc::LCOLS=3;
const char * QHacc::LCOLNAMES[]={ "ID", "NAME", "DESCRIPTION" };
const ColType QHacc::LCOLTYPES[]={ CTUINT, CTSTRING, CTSTRING };

// these values shouldn't change much in future accounting...
//const uint QHacc::ASSET=    0;
//const uint QHacc::LIABILITY=1;
//const uint QHacc::EQUITY=   2;
//const uint QHacc::EXPENSE=  3;
//const uint QHacc::REVENUE=  4;
// left plus accounts...


const uint QHacc::NREC=0;
const uint QHacc::MREC=1;
const uint QHacc::YREC=2;

const int QHacc::DECIMALS=2;
const int QHacc::ROLLOVER=( int )std::pow( ( double )10, QHacc::DECIMALS );

const int QHacc::NEVER=0;
const int QHacc::NEW=  1;
const int QHacc::ALL=  2;

const int QHacc::AMERICAN= 0;
const int QHacc::EUROPEAN= 1;
const int QHacc::YEARFIRST=2;

const int QHacc::TOTALS=  0;
const int QHacc::MONTHLYB=1;
const int QHacc::MONTHLYD=2;
const int QHacc::AVES    =3;
const int QHacc::JOURNAL =4;
const int QHacc::BUDGET  =5;

const int QHacc::LINE=   0;
const int QHacc::BAR=    1;
const int QHacc::TWOLINE=2;
const int QHacc::TWOBAR= 3;
const int QHacc::PIE=    4;

const QDate QHacc::XDATE=QDate( 8000, 1, 1 );

QHacc::QHacc() : QObject() {}

QHacc::~QHacc(){}

/******
 * Account table maintenace
 ******/

void QHacc::addA( const Account& a ) {
	Account newA=a;
	newA.set( AID, TableCol( db.maxID( QHaccDB::ACCOUNTS ).getu()+1 ) );
	newA.set( ACBAL, TableCol( newA.gets( AOBAL ) ) );
	if( db.add( QHaccDB::ACCOUNTS, newA )>=0 ){
		emit addedA( newA );
		if( !db.autosave() ) emit needSave( true );
	}
}	

void QHacc::removeA( const Account& a ) {
	// remove this account and all its transactions
	// reparent any accounts whose parent is this account
	TableCol aid=a.get( AID );

	std::ostream * str=0;
	if( Utils::debug( 6, str ) ) 
		*str<<"removing account "<<a.gets( ANAME )<<endl;

	// delete the account from the accounts table
	db.deleteWhere( QHaccDB::ACCOUNTS, AID, aid );

	// get the transactions affected by the deletion...
	QHaccTable * deleters=0;
	getTForArchive( a, deleters );
	// ...and remove them
	db.deleteWhere( QHaccDB::TRANSACTIONS, deleters );
	
	
	// find this account's children and reset their parent information
	db.updateWhere( QHaccDB::ACCOUNTS, APID, aid, APID, TableCol( ( uint )0 ) );
	
	// recalc all balances
	for( uint i=0; i<db.rows( QHaccDB::ACCOUNTS ); i++ ){
		Account ac=db.at( QHaccDB::ACCOUNTS, i );
		emit updateA( ac, calcBalOfA( ac ) );
	}
	
	emit removedA( a );
	if( !db.autosave() ) emit needSave( true );
}

void QHacc::getTForArchive( const Account& a, QHaccTable *& ret ){
	// we need to return all the transactions that are owned by
	// this account, but also all transactions paired to trans
	// in this account--using splitgroups, we can get both
	
	// atrans hold this account's transactions
	ret=new QHaccTable( TCOLTYPES, TCOLNAMES, TCOLS );

	QHaccTable * atrans=0;
	getTForA( a, atrans );
	
	uint rows=atrans->rows();

	bool de=getBP( "DOUBLEENTRY" );
	for( uint i=0; i<rows; i++ ){
		// we'll archive the transactions in this account
		Transaction tr=atrans->at( i );
		ret->add( tr );
		
		if( de ){
			// we're keeping double-entry books, so delete
			// this transaction's splits, as well
			QHaccTable * temp=0;
			getTSplits( tr, temp );
			ret->load( *temp );
			delete temp;
		}
	}
	
	delete atrans;
}

void QHacc::updateA( const Account& older , const Account& newer ){
	Account mynew=newer;
	//mynew.set( ACBAL, TableCol( newer.getf( ACBAL )+1 ) );
	if( db.updateWhere( QHaccDB::ACCOUNTS, AID, older.get( AID ), newer )>=0 ){
		emit updatedA( older, calcBalOfA( newer ) );
		if( !db.autosave() ) emit needSave( true );
	}
}

Account QHacc::getA( uint id ) {
	Account ret;
	QHaccTable * temp=0;
	if( db.getWhere( QHaccDB::ACCOUNTS, TableSelect( TID, TableCol( id ) ), 
									 temp ) )
		ret=temp->at( 0 );
	delete temp;
	return ret;
}

Account QHacc::getA( const QString& model ) {
	uint rows=db.rows( QHaccDB::ACCOUNTS );

	// search full names
	for( uint i=0; i<rows; i++ ){
		if( getFNameOfA( db.at( QHaccDB::ACCOUNTS, i ) )==model )
			return db.at( QHaccDB::ACCOUNTS, i );
	}	

	// search for short names
	for( uint i=0; i<rows; i++ ){
		if( db.at( QHaccDB::ACCOUNTS, i ).gets( QHacc::ANAME )==model )
			return db.at( QHaccDB::ACCOUNTS, i );
	}	

	// search IDs, just in case
	for( uint i=0; i<rows; i++ ){
		QString name;
		name.setNum( db.at( QHaccDB::ACCOUNTS, i ).getu( AID ) );
		if( name==model ) return db.at( QHaccDB::ACCOUNTS, i );
	}	

	// not found
	return Account();
}

void QHacc::getAs( bool noCats, QHaccTable *& table ){
 	if( noCats ) db.getWhere( QHaccDB::ACCOUNTS, TableSelect( ACATEGORY,
																														TableCol( true ),
																														TableSelect::NE ),
														table );
	else db.getWhere( QHaccDB::ACCOUNTS, TableSelect(), table );
}

uint QHacc::aDepth( uint id ) {
	uint pid=getA( id ).getu( APID );	
	if( pid==0 ) return 0;
	return aDepth( pid )+1;
}

QString QHacc::getFNameOfA( const Account& account ) {
	if( account.isNull() ) return QString();

	uint pid=account.getu( APID );	
	QString name=account.gets( ANAME );

	if( pid==0 ) return name;

	name=getFNameOfA( getA( pid ) )+ASEP+name;
	return name;
}

Account QHacc::getBlankA() const {
	TableCol cols[ACOLS];
	// mostly, we have strings...
	for( int i=0; i<ACOLS; i++ ) cols[i]=TableCol( QString() );
	// plus a couple others...
	uint zip=0;
	cols[AID]=TableCol( zip );
	cols[APID]=TableCol( zip );
	cols[AOBAL]=TableCol( "0.00" );
	cols[ATRANSNUMS]=TableCol( getSP( "TRANSTYPES" ) );
	cols[ATAXED]=TableCol( true );
	cols[ACATEGORY]=TableCol( true );
	cols[ATYPE]=TableCol( ( uint )getIP( "DEFAULTACCOUNTTYPE" ) );

	return TableRow( cols, ACOLS );
}

QString QHacc::getTypeNameOfA( const Account& acct ){
	QHaccTable * temp=0;
	if( db.getWhere( QHaccDB::ACCOUNTTYPES, TableSelect( YID, acct.get( ATYPE ) ), temp ) )
		return temp->at( 0 ).gets( YNAME );
	else return QString();
}

int QHacc::getARBalTo( const Account& acct, const QDate& date ) {
	// return the balance of all reconciled transactions up to the given date
	TableSelect criteria[]={
		TableSelect( TACCTID, acct.get( AID ) ),
		TableSelect( TDATE, TableCol( date ), TableSelect::LE ),
		TableSelect( TRECO, TableCol( YREC ) )
	};
	QHaccTable * trans=0;
	getTWhere( criteria, 3, trans );

	int ret=convMoney( acct.gets( AOBAL ) );
	for( uint i=0; i<trans->rows(); i++ )
		ret+=convMoney( trans->at( i ).gets( TSUM ) );

	delete trans;
	return ret;	
}

int QHacc::getABalOn( const Account& acct, const QDate& date ) {
	// return the balance of the account at the beginning of the specified
	// date( i.e., before any transactions for that day are added)

	// get every transaction after date, subtract
	// their sum from the current balance
	TableSelect criteria[]={
		TableSelect( TACCTID, acct.get( AID ) ),
		TableSelect( TDATE, TableCol( date ), TableSelect::LT )
	};
	QHaccTable * trans=0;
	getTWhere( criteria, 2, trans );

	int ret=convMoney( acct.gets( AOBAL ) );
	for( uint i=0; i<trans->rows(); i++ )
		ret+=convMoney( trans->at( i ).gets( TSUM ) );

	delete trans;
	return ret;
}

void QHacc::makeAT( QHaccTable *& ret, int pkcol ) const {
	ret=new QHaccTable( ACOLTYPES, ACOLNAMES, ACOLS, pkcol );
}

bool QHacc::isLPA( const Account& acct ) {
	QHaccTable * temp=0;
	db.getWhere( QHaccDB::ACCOUNTTYPES, TableSelect( YID, acct.get( ATYPE ) ),
							 temp );
	TableRow row=temp->at( 0 );
	delete temp;
	return row.getb( YLP );
}


/******
 * Transaction table maintenace
 ******/

uint QHacc::addTSG( QHaccTable * ts ){
	// add a whole split group of transactions
	uint ret=0;
	if( condenseSG( ts ) ){
		uint tid=db.maxID( QHaccDB::TRANSACTIONS ).getu();  // new highest tid
		QHaccTableIndex * tsidx=0;

		db.indexOn( QHaccDB::TRANSACTIONS, TSPLITGROUP, tsidx );
		TableCol sg( tsidx->maxID().getu()+1 ); // new highest splitgroup
		
		uint tsrows=ts->rows();
		for( uint i=0; i<tsrows; i++ ){
			Transaction t=ts->at( i );
			t.set( TID, TableCol( ++tid ) );
			t.set( TSPLITGROUP, sg );

			//cout<<"  addtsg: "<<t.toString()<<endl;
			
			db.add( QHaccDB::TRANSACTIONS, t );
			emit addedT( t );

			Account ac=getA( t.getu( TACCTID ) );
			emit updatedA( ac, calcBalOfA( ac ) );
		}
		
		if( !db.autosave() ) emit needSave( true );
		
		if( getBP( "DOUBLEENTRY" ) ) ret=sg.getu();
		emit addedT();
	}

	return ret;
}

void QHacc::updateTSG( const TableCol& sg, QHaccTable * splits ){
	// update a whole split group of transactions...
	// we can keep the old split group, but we need to
	// see which splits must be updated, which must be
	// added, and which should be removed, if any
	
	// we want to correlate old splits with new splits,
	// so we'll check based on tacctids (which are unique,
	// thanks to condenseSG).
	QHaccTable newsplits( *splits );
	if( !condenseSG( &newsplits ) ) return;
	
	if( !getBP( "DOUBLEENTRY" ) ){
		Transaction t=splits->at( 0 );
		db.updateWhere( QHaccDB::TRANSACTIONS, TID, t.get( TID ), t );
		emit updatedT( t );
		Account ac=getA( t.getu( TACCTID ) );
		emit updatedA( ac, calcBalOfA( ac ) );
	}
	else{
		QHaccTableIndex * aidx=0;
		newsplits.addIndexOn( TACCTID, aidx );
		
		QHaccTable * temp=0;
		db.getWhere( QHaccDB::TRANSACTIONS, TableSelect( TSPLITGROUP, sg ), temp );
		QHaccTable oldsplits( *temp );
		delete temp;
		
		// make sure everyone's part of the same splitgroup
		newsplits.updateWhere( TableSelect(), TSPLITGROUP, sg );
		
		// keep track of which signals we should emit and emit them 
		// only afer everything is done (this function is really atomic)
		for( uint i=0; i<oldsplits.rows(); i++ ){
			Transaction old=oldsplits.at( i );
			
			//cout<<"looking for "<<old->toString()<<endl;
			//uint aid=old.getu( TACCTID );
			
			uint idx=0;
			if( aidx->contains( old, idx ) ){
				// we have a transaction from the old splits in the new splits
				// so use the tid from the old trans for the new one
				Transaction newy=aidx->at( idx );
				newy.set( TID, old.get( TID ) );
				db.updateWhere( QHaccDB::TRANSACTIONS, TID, old.get( TID ), newy );
				emit updatedT( newy );
				
				// don't process again
				newsplits.deleteWhere( TACCTID, newy.get( TACCTID ) );
			}
			else{
				// old split isn't in the new splits, so we'll remove it
				//cout<<"  not found, so removing "<<old->toString()<<endl;
				db.deleteWhere( QHaccDB::TRANSACTIONS, TID, old.get( TID ) );
				emit removedT( old );
			}
			
			Account ac=getA( old.getu( TACCTID ) );
			emit updatedA( ac, calcBalOfA( ac ) );
		}

		// at this point, anything left in the newsplits table is a new split
		for( uint i=0; i<newsplits.rows(); i++ ){
			uint topid=db.maxID( QHaccDB::TRANSACTIONS ).getu()+1;
			Transaction t=newsplits.at( i );
			t.set( TID, TableCol( topid ) );
			db.add( QHaccDB::TRANSACTIONS, t );
			
			emit addedT( t );

			Account ac=getA( t.getu( TACCTID ) );
			emit updatedA( ac, calcBalOfA( ac ) );
		}
	}

	if( !db.autosave() ) emit needSave( true );
	emit updatedT();
}

bool QHacc::condenseSG( QHaccTable * splits ) const {
	// make sure that each split has a unique account
	// otherwise combine non-unique transactions
	// the easiest way is to add an index on tacctid
	if( splits->isEmpty() ) return false;
	if( !getBP( "DOUBLEENTRY" ) ) return true;
	
	QHaccTableIndex * idx=0;
	splits->addIndexOn( TACCTID, idx );

	uint * pos=0, sz=0;
	QHaccSegmenter::segment( this, idx, pos, sz );
	for( uint i=0; i<sz-1; i++ ){
		if( pos[i+1]-pos[i]>1 ){ 
			// we have duplicate TACCTIDs, so get all the
			// info we need, and then combine the trans
			TableRow change=idx->at( pos[i] );
			int sum=0;
			for( uint j=pos[i]; j<pos[i+1]; j++ ){
				sum+=convMoney( idx->at( j ).gets( TSUM ) );
			}
			
			// we now know the sum, so delete the offending
			// transactions and add the one we just made
			change.set( TSUM, TableCol( convMoney( sum ) ) );
			
			splits->deleteWhere( TACCTID, change.get( TACCTID ) );
			splits->add( change );
		}
	}
	delete [] pos;

	if( splits->rows()<2 ){
		std::ostream * str=0;
		if( Utils::error( 4, str ) ) *str<<"split transaction does not resolve to enough accounts"<<endl;
		return false;
	}
	return true;
			
}

void QHacc::removeTSG( const TableCol& sg ){
	// remove all splits of a splitgroup
	QHaccTable * temp=0;
	db.getWhere( QHaccDB::TRANSACTIONS, TableSelect( TSPLITGROUP, sg ), temp );
	QHaccTable splits( *temp );
	delete temp;

	db.deleteWhere( QHaccDB::TRANSACTIONS, TSPLITGROUP, sg );
	
	for( uint i=0; i<splits.rows(); i++ ){
		Transaction t=splits.at( i );
		emit removedT( t );

		Account ac=getA( t.getu( TACCTID ) );
		emit updatedA( ac, calcBalOfA( ac ) );
	}

	if( !db.autosave() ) emit needSave( true );
	emit removedT();
}

Transaction QHacc::getT( uint id ) {
	Transaction ret;
	QHaccTable * temp=0;
	if( db.getWhere( QHaccDB::TRANSACTIONS, TableSelect( TID, TableCol( id ) ),
									 temp ) )
		ret=temp->at( 0 );
	delete temp;
	return ret;
}

void QHacc::getTWhere( TableSelect * sel, uint size, QHaccTable *& r ) {
	db.getWhere( QHaccDB::TRANSACTIONS, sel, size, r );
}

void QHacc::setTRec( const Transaction& t, uint rt ){
	if( db.updateWhere( QHaccDB::TRANSACTIONS, TID, t.get( TID ),
											TRECO, TableCol( rt ) )>=0 ){
		Transaction tr( t );
		tr.set( TRECO, TableCol( rt ) );
		
		Account ac=getA( tr.getu( TACCTID ) );
		Account rac=calcBalOfA( ac );
		if( rac.gets( ARBAL )!=ac.gets( ARBAL ) ) emit updatedA( ac, rac );	

		emit updatedT( tr );
		if( !db.autosave() ) emit needSave( true );
	}
}

void QHacc::getTSplits( uint tid, QHaccTable *& table ){
	Transaction t=getT( tid );
	if( !t.isNull() )	getTSplits( t, table );
	else makeTT( table );
}

void QHacc::getTSplits( const Transaction& t, QHaccTable *& table ) {
	// return the splits, MINUS the transaction that got sent to us
	TableSelect criteria[]={
		TableSelect( TSPLITGROUP, t.get( TSPLITGROUP ) ),
		TableSelect( TID, t.get( TID ), TableSelect::NE )
	};
	QHaccTable * tmp=0;
	db.getWhere( QHaccDB::TRANSACTIONS, criteria, 2, tmp );

	makeTT( table );
	table->load( *tmp );
}

void QHacc::getTForA( const Account& a, QHaccTable *& ret ){
	db.getWhere( QHaccDB::TRANSACTIONS, TableSelect( TACCTID, a.get( AID ) ),
							 ret );
}

void QHacc::makeTT( QHaccTable *& ret, int pkcol ) const {
	ret=new QHaccTable( TCOLTYPES, TCOLNAMES, TCOLS, pkcol );
}

Transaction QHacc::getBlankTForAL( const Account& a, const Ledger& l ) const {
	TableCol cols[TCOLS];
	for( uint i=1; i<6; i++ ) cols[i]=TableCol( QString() );
	uint zip=0;
	cols[TID]=TableCol( zip );
	cols[TDATE]=TableCol( QDate() );
	cols[TNUM]=TableCol( a.gets( ADEFAULTNUM ) );
	cols[TRECO]=TableCol( zip );
	cols[TSPLITGROUP]=TableCol( zip );
	cols[TACCTID]=a.get( AID );
	cols[TLID]=l.get( LID );

	return Transaction( cols, TCOLS );
}

void QHacc::getMSubsetForA( const Account& acct, QHaccTable *& ret ){
	TableSelect criteria[]={
		TableSelect( TACCTID, acct.get( AID ) ),
		TableSelect( TID, TableCol( ( uint )0 ), TableSelect::NE )
	};
	db.getWhere( QHaccDB::MEMTRANS, criteria, 2, ret );
}

void QHacc::memorize( uint tid, bool add ){
	Transaction t=getT( tid );
	
	//cout<<"in memorize, tid is "<<tid<<" and add is "<<add<<endl;

	if( add ){ // add new memorized trans
		QHaccTable * splits=0;
		getTSplits( t, splits );

		db.startLoad( QHaccDB::MEMTRANS, splits->rows()+1 );
		db.add( QHaccDB::MEMTRANS, t );
		for( uint i=0; i<splits->rows(); i++ ){
			Transaction spl=splits->at( i );

			// when retrieving memorized trans, we will 
			// only grab them if their TID isn't 0 (otherwise,
			// memorizing one transaction would automatically
			// memorize all its splits, too
			spl.set( TID, TableCol( ( uint )0 ) );
			db.add( QHaccDB::MEMTRANS, spl );
		}
		db.stopLoad( QHaccDB::MEMTRANS );

		delete splits;
	}
	else{ // remove a previously-memorized trans
		db.deleteWhere( QHaccDB::MEMTRANS, TSPLITGROUP, t.get( TSPLITGROUP ) );
	}
}

void QHacc::addMtoL( uint tid, QDate date, const Ledger& l ){
	// insert a previously-memorized transaction into the transactions table
	QHaccTable * temp=0;
	db.getWhere( QHaccDB::MEMTRANS, TableSelect( TID, TableCol( tid ) ), temp );

	QHaccTable * table=0;
	db.getWhere( QHaccDB::MEMTRANS, TableSelect( TSPLITGROUP, temp->at( 0 ).get( TSPLITGROUP ) ),	table );
	delete temp;
	
	TableCol dater( date.isValid() ? date : QDate::currentDate() );
	table->updateWhere( TableSelect(), TDATE, dater );
	table->updateWhere( TableSelect(), TLID, l.get( LID ) );
	table->updateWhere( TableSelect(), TRECO, TableCol( NREC ) );

	addTSG( table );
	delete table;
}

/******
 * Ledger table maintenace
 ******/

void QHacc::getLs( QHaccTable *& ret ){ 
	db.getWhere( QHaccDB::LEDGERS, TableSelect(), ret );
}

Ledger QHacc::getL( const QString& lname ){
	Ledger ret;
	QHaccTable * temp=0;
	if( db.getWhere( QHaccDB::LEDGERS, TableSelect( LNAME, TableCol( lname ) ), 
									 temp ) )
		ret=temp->at( 0 );
	delete temp;
	return ret;
}

Ledger QHacc::getL( uint lid ){
	Ledger ret;
	QHaccTable * temp=0;
	if( db.getWhere( QHaccDB::LEDGERS, TableSelect( LID, TableCol( lid ) ),
									 temp ) )
		ret=temp->at( 0 );
	delete temp;
	return ret;
}

void QHacc::addL( const Ledger& l ){
	QHaccTableIndex * idx=0;
	db.indexOn( QHaccDB::LEDGERS, LID, idx );
	TableCol maxlid( idx->maxID().getu()+1 ); // new highest ledger id
	
	Ledger newy( l );
	newy.set( LID, maxlid );
	if( db.add( QHaccDB::LEDGERS, newy )>=0 ){
		emit addedL( newy );
		if( !db.autosave() ) emit needSave( true );
	}
}
void QHacc::removeL( const Ledger& l ) {
	if( db.rows( QHaccDB::LEDGERS )>1 ){
		db.deleteWhere( QHaccDB::LEDGERS, TableSelect( LID, l.get( LID ) ) );

		// delete all the transactions from this
		// ledger, then recalc all account balances
		db.deleteWhere( QHaccDB::TRANSACTIONS, TLID, l.get( LID ) );
		for( uint i=0; i<db.rows( QHaccDB::ACCOUNTS ); i++ ){
			Account ac=db.at( QHaccDB::ACCOUNTS, i );
			emit updateA( ac, calcBalOfA( ac ) );
		}
		
		emit removedL( l );
		if( !db.autosave() ) emit needSave( true );
	}
}

void QHacc::updateL( const Ledger& older , const Ledger& newer ){
	Ledger mynew=newer;
	if( db.updateWhere( QHaccDB::LEDGERS, LID, older.get( LID ), newer )>=0 ){
		emit updatedL( older, newer );
		if( !db.autosave() ) emit needSave( true );
	}
}

Ledger QHacc::getBlankL(){
	TableCol tcs[]={ TableCol( ( uint )0 ), TableCol( "" ), TableCol( "" ) };
	return Ledger( tcs, 3 );
}

/******
 * Preference table maintenace
 ******/
QString QHacc::getSP( const QString& pref ) const {
	uint idx=0;
	QString ret;

	if( db.contains( QHaccDB::PREFERENCES, TableCol( pref ), idx ) )
		ret=db.at( QHaccDB::PREFERENCES, idx ).gets( PVALUE );

	return ret;
}

QColor QHacc::getCP( const QString& pref ) const {
	
	QString val=getSP( pref );
	if( val.isEmpty() ){
		if( pref=="MAINCOLOR" ) return QColor( 255, 255, 255 );
		return QColor( 173, 216, 230 );
	}
	
	// colors are saved as space-delimited ints
	QString ints[3];
	Utils::parser( val, " ", 0, ints, 3 );
	return QColor( ints[0].toInt(), ints[1].toInt(), ints[2].toInt() );
}

QDate QHacc::getDP( const QString& pref ) const {
	QString val=getSP( pref );
	QString sep=getSP( "DATESEPARATOR" );
	uint num=3;
	QString strs[num];
	if( val.isEmpty() ) return QDate::currentDate();
	Utils::parser( val, sep, 0, strs, num );
	return QDate( strs[0].toInt(), strs[1].toInt(), strs[2].toInt() );
}

QFont QHacc::getWP( const QString& pref ) const {
	QString val=getSP( pref );
	if( val.isEmpty() ) return QFont( "SansSerif" );
	QFont f;
	f.fromString( val );
	return f;
}

bool QHacc::getBP( const QString& pref ) const {
	QString val=getSP( pref );
	return ( val=="Y" ? true : false );
}

int QHacc::getIP( const QString& pref ) const {
	QString val=getSP( pref );
	return ( val.isEmpty() ? 0 : val.toInt() );
}

float QHacc::getFP( const QString& pref ) const {
	QString val=getSP( pref );
	return ( val.isEmpty() ? 0 : val.toFloat() );
}


bool QHacc::isetP( const QString& p, QString v ){
	bool changed=false;
	uint idx=0;

	TableCol pref( p ), val( v );

	if( db.contains( QHaccDB::PREFERENCES, pref, idx ) ){
		if( db.at( QHaccDB::PREFERENCES, idx ).compareTo( pref, PVALUE )!=0 ){
			db.updateWhere( QHaccDB::PREFERENCES, PPREF, pref, PVALUE, val );
			changed=true;
		}
	}
	else {
		TableCol tcs[]={ pref, val };
		db.add( QHaccDB::PREFERENCES, TableRow( tcs, 2 ) );
		changed=true;
	}
	
	if( changed && !db.autosave() ) emit needSave( true );
	return changed;
}

void QHacc::setSP( const QString& pref, QString val ){
	if( isetP( pref, val ) ) emit changedP( pref, val );
}
void QHacc::setCP( const QString& pref, QColor val ){
	if( isetP(  pref, QString::number( val.red() )+" "
							 +QString::number( val.green() )+" "
							 +QString::number( val.blue() ) ) )
		emit changedP( pref, val );
}
void QHacc::setDP( const QString& pref, QDate val ){
	QString sep=getSP( "DATESEPARATOR" );
	QString date;
	date=QString::number( val.year() )+sep+QString::number( val.month() )
		+sep+QString::number( val.day() );
	if( isetP( pref, date ) ) emit changedP( pref, val );
}

void QHacc::setWP( const QString& pref, QFont val ){
	if( isetP( pref, val.toString() ) ) emit changedP( pref, val );
}
void QHacc::setFP( const QString& pref, float val ){
	if( isetP( pref, QString::number( val ) ) ) emit changedP( pref, val );
}
void QHacc::setBP( const QString& pref, bool val ){
	if( isetP( pref, val ? "Y" : "N" ) ) emit changedP( pref, val );	
}
void QHacc::setIP( const QString& pref, int val ){
	if( isetP( pref, QString::number( val ) ) ) emit changedP( pref, val );
}

void QHacc::getATs( QHaccTable *& ret ){
	db.getWhere( QHaccDB::ACCOUNTTYPES, TableSelect(), ret );
}

bool QHacc::homeIsLocal() const { return ( !db.autosave() ); }
QString QHacc::getHome() const { return qhacchome; }
bool QHacc::setHome( const QString& home ) {
	if( home.isEmpty() ){
		std::ostream * str=0;
		if( Utils::error( 0, str ) )
			*str<<"\nQHACC_HOME is not set. Please set it before running "
					<<"QHacc, or use the -f option.\n"<<endl;
		return false;
	}

	qhacchome=QString( home );

	// get rid of a trailing /, if necessary
	if( qhacchome.endsWith( "/" ) )	qhacchome.remove( qhacchome.length()-1, 1 );
	blockSignals( true );
	db.load( qhacchome );

	// go through accounts and sum up the transactions to get current balances
	uint naccts=db.rows( QHaccDB::ACCOUNTS );
	for( uint i=0; i<naccts; i++ ) calcBalOfA( db.at( QHaccDB::ACCOUNTS, i ) );

	// make sure we have at least the bare-bones preferences needed for operation
	const char * prefs[]={ "CSYMBOL", "DATESEPARATOR", "DOUBLEENTRY", 
												 "MAINCOLOR", "ALTCOLOR", "DEFAULTACCOUNTTYPE" };
	const char * vals[]={ "$", "/", "N", "255 255 255", "46 192 255", "3" };

	for( int i=0; i<6; i++ ){
		if( getSP( prefs[i] ).isNull() ){
			TableCol tcs[]={ TableCol( prefs[i] ), TableCol( vals[i] ) };
			db.add( QHaccDB::PREFERENCES, TableRow( tcs, PCOLS ) );
		}
	}
	QHaccDateEdit::setDateFormat( ( QHaccDateEdit::DateFormat ) getIP( "DATEFORMAT" ) );
	QHaccDateEdit::setDateSeparator( getSP( "DATESEPARATOR" ) );
	
	if( db.isEmpty( QHaccDB::ACCOUNTTYPES ) ){
		// also, make sure we have the default accounttypes
		// these names must correspond to the associated static vars 
		// (we only need to load once, since the values should be
		// consistent across multiple QHACC_HOMEs)
		const char * NMS[]={ "Asset", "Liability", "Equity", 
												 "Expense", "Revenue" };
		const bool YVALS[]={ true, false, false, true, false };
		const uint count=5;
		db.startLoad( QHaccDB::ACCOUNTTYPES, count );
		for( uint i=0; i<count; i++ ){
			TableCol tcs[]={ TableCol( i ), TableCol( NMS[i] ),
											 TableCol( YVALS[i] ) };
			db.add( QHaccDB::ACCOUNTTYPES, TableRow( tcs, YCOLS ) );
		}
		db.stopLoad( QHaccDB::ACCOUNTTYPES );
	}

	if( db.isEmpty( QHaccDB::LEDGERS ) ){
		TableCol tcs[]={ TableCol( ( uint )1 ), TableCol( "General" ),
										 TableCol( "" ) };
		db.add( QHaccDB::LEDGERS, TableRow( tcs, LCOLS ) );
	}

	blockSignals( false );
	return true;
}

Account QHacc::calcBalOfA( const Account& acct ){
	QHaccTable * trans=0;
	getTForA( acct, trans );
	uint rows=trans->rows();
	Account ret( acct );


	std::ostream * str=0;
	if( Utils::debug( 6, str ) ) 
		*str<<"CalcBalOfA "<<acct.gets( ANAME )<<" will count "
				<<rows<<" transactions"<<endl;

	int ocbal=convMoney( acct.gets( ACBAL ) );
	int orbal=convMoney( acct.gets( ARBAL ) );
	int sum=convMoney( acct.gets( AOBAL ) );
	int rsum=sum;

	for( uint i=0; i<rows; i++ ){	
		Transaction t=trans->at( i );
		int tsum=convMoney( t.gets( TSUM ) );

		sum+=tsum;
		if( t.getu( TRECO )==QHacc::YREC ) rsum+=tsum;
	}

	if( !( sum==ocbal && rsum==orbal ) ){
		TableCol col( convMoney( sum ) );
		db.updateWhere( QHaccDB::ACCOUNTS, AID, acct.get( AID ), ACBAL, col );
		ret.set( ACBAL, col );
		
		col=TableCol( convMoney( rsum ) );
		db.updateWhere( QHaccDB::ACCOUNTS, AID, acct.get( AID ), ARBAL, col );
		ret.set( ARBAL, col );
	}
	
	// check budgeting
	//float budget=acct.getf( ABUDGET );
	int budget=convMoney( acct.gets( ABUDGET ) );
	if( budget!=0 ){
		QDate start=QDate::currentDate();
		start.setYMD( start.year(), start.month(), 1 );
		QDate end=start.addMonths( 1 );
		// we only want the last month's worth of transactions
		// we do this here instead of above because the chances
		// aren't so good that a budget has been set, so why
		// waste the time if it's not needed?

		QHaccTableIndex * idx=0;
		trans->addIndexOn( TDATE, idx );

		//cout<<TableCol( today.addMonths( -1 ) ).toString()<<"-"<<TableCol( today ).toString()<<endl;
		TableSelect criteria[]={
			TableSelect( TDATE, TableCol( start ), TableSelect::GE ),
			TableSelect( TDATE, TableCol( end ), TableSelect::LT )
		};

		QHaccTable * trans2=0;
		trans->getWhere( criteria, 2, trans2 );

		sum=0;		
		for( uint i=0; i<trans2->rows(); i++ )
			sum+=convMoney( trans2->at( i ).gets( TSUM ) );
		delete trans2;
		
		if( ( budget<0 ) && ( sum<budget ) || ( budget>0 && sum>budget ) )
			emit overBudget( acct, sum-budget );
	}

	delete trans;
	return ret;
}

bool QHacc::save( QString& error ) {
	// save everything
	if( db.save( qhacchome, error ) ){
		if( !db.autosave() ) emit needSave( false );
		return true;
	}
	return false;
}

void QHacc::archive( const char * home, bool remover, const char * extra ){
	setHome( home );
	std::ostream * str=0;

#ifdef QHACCMYSQL
	if( qhacchome.upper().startsWith( "MYSQL" ) ){
		if( Utils::error( 0, str ) )
			*str<<"Archive and Restore is unsupported to MySQL databases."<<endl;
		return;
	}
#endif
	
	if( Utils::debug( 0, str ) ){
		uint rows=0;
		QString temp;

		if( remover ){ // archive old data
			// we can either archive by date or by account
			Account acct=getA( extra );
			QHaccTable * trans=0;

			// date the transaction file
			QString daylbl;
			int df=getIP( "DATEFORMAT" );
			if( df==AMERICAN ) daylbl="MMddyyyy";
			else if( df==EUROPEAN ) daylbl="ddMMyyyy";
			else daylbl="yyyyddMM";
			
			// archiving by date
			if( acct.isNull() ){
				QDate date=QHaccDateEdit::getDateFromString( extra );
				db.getWhere( QHaccDB::TRANSACTIONS, TableSelect( TDATE,
																												 TableCol( date ),
																												 TableSelect::LT ),
										 trans );
			}
			else getTForArchive( acct, trans );

			rows=trans->rows();
			QFile file( qhacchome+"/transactions."
									+QDate::currentDate().toString( daylbl ) );
			if( trans->save( file, temp ) )	db.deleteWhere( QHaccDB::TRANSACTIONS, trans );			

			if( !getBP( "KEEPFILEPERMS" ) ) chmod( file.name(), S_IRUSR | S_IWUSR );
			delete trans;

			if( !acct.isNull() ){
				// we archived the whole account, so put the account in the
				// archived account file and remove it from the accounts table
				QHaccTable archived( ACOLTYPES, ACOLNAMES, ACOLS, AID,
														 "archived", 0, 3, 5 );
				QFile afile( qhacchome+"/accounts.archived" );
				if( afile.exists() ) archived.load( afile, temp );

				archived.add( acct );
				if( archived.save( afile, temp ) ){ 
					if( !getBP( "KEEPFILEPERMS" ) ) 
						chmod( afile.name(), S_IRUSR | S_IWUSR );
					db.deleteWhere( QHaccDB::ACCOUNTS, AID, acct.get( AID ) );
				}
			}
		}
		else{ // restore previously-archived data
			QHaccTable trans( TCOLTYPES, TCOLNAMES, TCOLS );

			QFile file( extra );
			trans.load( file, temp );
			db.load( QHaccDB::TRANSACTIONS, trans );

			// now that the transactions are restored, see if any of them
			// reference an archived account. If one does, restore the account
			QHaccTable archived( ACOLTYPES, ACOLNAMES, ACOLS, AID,
													 "archived", 0, 3, 5 );
			QFile afile( qhacchome+"/accounts.archived" );
			if( afile.exists() ){
				archived.load( afile, temp );
				
				uint junk=0;
				for( uint i=0; i<archived.rows(); i++ ){
					Account arc=archived.at( i );
					TableCol aaid=arc.get( AID );
					if( db.contains( QHaccDB::TRANSACTIONS, aaid, junk ) ){
						// restore the given account, remove it from the archives
						db.add( QHaccDB::ACCOUNTS, arc );
						archived.deleteWhere( AID, aaid );
					}
				}
				
				// save what's left of the archived table
				archived.save( afile, temp );
				if( !getBP( "KEEPFILEPERMS" ) ) 
					chmod( afile.name(), S_IRUSR | S_IWUSR );
			}
		}
		
		// now save everything
		rows=db.rows( QHaccDB::ACCOUNTS );
		for( uint i=0; i<rows; i++ ){
			Account acct=db.at( QHaccDB::ACCOUNTS, i );
			int cbal=convMoney( acct.gets( ACBAL ) );
			int obal=convMoney( acct.gets( AOBAL ) );
			Account acct2=calcBalOfA( acct );
			
			// we need to refresh our account...
			//acct=db.at( QHaccDB::ACCOUNTS, i );

			// pruned balance
			//float pbal=cbal-acct.getf( ACBAL );
			int pbal=cbal-convMoney( acct2.gets( ACBAL ) );

			db.updateWhere( QHaccDB::ACCOUNTS, AID, acct.get( AID ), AOBAL,
											convMoney( obal+pbal ) );
		}
		
		if( !save( temp ) && Utils::error( 0, str ) ) *str<<temp<<endl;
	}
}

void QHacc::report( const char * home, int type, const char * acctn ){
	setHome( home );
	std::ostream * str=0;
	if( Utils::debug( 0, str ) ){
		QDate end=QDate::currentDate();
		//QDate start=end.addMonths( 0-getIP( "GRAPHTIME" ) );
		QHaccTableIndex * tdidx=0;
		db.indexOn( QHaccDB::TRANSACTIONS, TDATE, tdidx );
		QDate start=tdidx->minID().getd();

		if( type==JOURNAL ){
			QString ledn( acctn );
			Ledger l=getL( ledn );
			if( l.isNull() ){
				uint i=ledn.toUInt();
				if( i>0 && i<db.rows( QHaccDB::LEDGERS )+1 )
					l=db.at( QHaccDB::LEDGERS, i-1 );
			}
			if( !l.isNull() )
				*str<<QHaccReportWriter::journal( l.getu( LID ), this, 
																					start, end )<<endl;
			else if( Utils::error( 2, str ) ) *str<<"No Ledger: "<<acctn<<endl;
		}
		else{
			Account acct=getA( acctn );
			if( !acct.isNull() ){
				QHaccTable accts( ACOLTYPES, ACOLNAMES, ACOLS, AID, 0, 1, 1, 1 );
				accts.add( acct );
				*str<<QHaccReportWriter::report( type, this, getIP( "LEDGERINDEX" ), &accts, start, end )
						<<endl;
			}
			else if( Utils::error( 2, str ) ) *str<<"No Account: "<<acctn<<endl;
		}
	}
}

void QHacc::showAccounts( const char * homer ){
	setHome( homer );

	std::ostream * str=0;
	if( Utils::debug( 0, str ) ){
		uint arows=db.rows( QHaccDB::ACCOUNTS );
		uint trows=db.rows( QHaccDB::TRANSACTIONS );
		*str<<"-- "<<trows<<" Transaction"<<( trows==1 ? "" : "s" )<<" in "
				<<arows<<" Account"<<( arows==1 ? "" : "s" )<<endl;

		QHaccTableIndex * aidx=0;
		db.indexOn( QHaccDB::ACCOUNTS, AID, aidx );
		for( uint i=0; i<arows; i++ ){
			Account acct=aidx->at( i );
			QHaccTable * trans=0;
			getTForA( acct, trans );

			*str<<acct.getu( AID )<<":"
					<<getFNameOfA( acct )<<": "<<acct.gets( ACBAL )<<" in "
					<<trans->rows()<<" Transaction"
					<<( trans->rows()==1 ? "" : "s" )
					<<endl;

			delete trans;
		}
	}
}

void QHacc::showTrans( const char * homer, const char * namer ){
	setHome( homer );
	Account acct=getA( namer );

	std::ostream * str;
	
	if( acct.isNull() ){
		if( Utils::error( 0, str ) ) *str<<"No Account: "<<namer<<endl;
	}
	else{
		QHaccTable * trans=0;
		getTForA( acct, trans );
		
		QHaccTableIndex * index=0;
		trans->addIndexOn( TDATE, index );
		
		if( Utils::debug( 0, str ) ){
			int fields[]={ TID, TLID, TNUM, TDATE, TPAYEE, TMEMO, TSUM };
			int nfields=7;
			
			for( uint i=0; i<trans->rows(); i++ ){
				Transaction row=index->at( i );
				
				*str<<row.get( fields[0] ).toString();
				for( int i=1; i<nfields; i++ )
					*str<<":"<<row.get( fields[i] ).toString();
				*str<<endl;
			}
		}
		
		delete trans;
	}
}

bool QHacc::writeTransToFile( uint tid, const QString& fname,
															QString& error ){
	// this function is the exact opposite of cron...it takes a
	// transaction and writes it and its pairs to a file so it
	// can later be read by the cron option
	Transaction t=getT( tid );

	QHaccTable * temp=0;
	getTSplits( t, temp );

	QHaccTable splits( *temp );
	delete temp;
	splits.add( t );

	splits.updateWhere( TableSelect(), TDATE, TableCol( XDATE ) );
	splits.updateWhere( TableSelect(), TID, TableCol( ( uint )0 ) );
	splits.updateWhere( TableSelect(), TRECO, TableCol( ( uint )0 ) );

	QFile file( fname );
	bool ret=splits.save( file, error );

	return ret;
}

void QHacc::cron( const char * home, const char * filename ){
	// allow a user to insert a transaction from the command line

	setHome( home );
	std::ostream * str=0;
	
	QHaccTable inserts( TCOLTYPES, TCOLNAMES, TCOLS );
	QFile file( filename );

	QString err;
	if( !inserts.load( file, err ) ){
		if( Utils::error( 0, str ) ) *str<<err<<endl;
		return;
	}

	// resolve any dollar amounts we need
	if( !crondollars( &inserts ) ) return;
	if( !condenseSG( &inserts ) ) return;

	// see if we should use the current date for insertion time
	Transaction trans=inserts.at( 0 );

	TableCol d=trans.get( TDATE );
	if( d.getd()==XDATE ) d=TableCol( QDate::currentDate() );

	inserts.updateWhere( TableSelect(), TDATE, d );

	addTSG( &inserts );

	if( !save( err ) && Utils::error( 0, str ) ) *str<<err<<endl;
}

bool QHacc::crondollars( QHaccTable * trans ) {
	QString tsum;

	// first, figure out any percentage amounts in the transactions
	for( uint i=0; i<trans->rows(); i++ ){
		Transaction t=trans->at( i );
		tsum=t.gets( TSUM );

		if( tsum.endsWith( "%" ) ){
			tsum.remove( tsum.length()-1, 1 );
			float perc=tsum.toFloat()/100;
			
			Account acct=getA( t.getu( TACCTID ) );
			int abal=( int )( convMoney( acct.gets( ACBAL ) )*perc );

			trans->updateWhere( TACCTID, t.get( TACCTID ),
													TSUM, TableCol( convMoney( abal ) ) );
		}
	}

	// next, figure out any positional amounts
	for( uint i=0; i<trans->rows(); i++ ){
		Transaction t=trans->at( i );
		tsum=t.gets( TSUM );

		// see if we have positional amounts
		int pos=-1;
		bool neg=false;
		if( tsum.startsWith( "@" ) ) pos=tsum.mid( 1 ).toInt();
		else if( tsum.startsWith( "-@" ) ){
			pos=tsum.mid( 2 ).toInt();
			neg=true;			
		}
		
		if( pos>=0 ){ // we have positional amounts
			std::ostream * str=0;
			if( ( uint )pos>=trans->rows() || ( uint )pos==i ){
				if( Utils::error( 0, str ) )
					*str<<"positional amount "<<tsum<<" is invalid"<<endl;
				return false;
			}

			
			Transaction tr=trans->at( pos );
			QString t2sum=tr.gets( TSUM );
			
			if( t2sum.contains( "@" ) && Utils::error( 0, str ) ){
				*str<<"positional amount "<<tsum
						<<" points to unresolved amount "<<t2sum<<endl;
				return false;
			}

			int sum=convMoney( t2sum );
			if( neg ) sum=0-sum;
			
			// assign the value
			trans->updateWhere( TACCTID, t.get( TACCTID ),
													TSUM, TableCol( convMoney( sum ) ) );
		}
	}

	return true;
}

QString QHacc::convMoney( int cents ){
	// convert the cents to a string of dollars and cents
	
	// we want only positive values for our comparisons; we'll add the neg later
	bool neg=false;
	if( cents<0 ){
		neg=true;
		cents=0-cents;
	}

	// ROLLOVER will not be a double, GUARANTEED!
	int dollars=cents/ROLLOVER;
	int ncents=cents%ROLLOVER;

	QString ret=QString::number( dollars )+".";
	if( ncents<10 ) ret.append( "0" );
	ret.append( QString::number( ncents ) );

	// now that we have the string, put a minus sign if necessary
	if( neg ) ret.prepend( "-" );

	//cout<<"converting "<<cents<<" to string: "<<ret<<endl;
	return ret;
}

int QHacc::convMoney( const QString& st ){
	// convert the string to dollars and cents

	// we only want to deal with positive values here; we'll add the minus later
	bool neg=false;
	QString s;
	if( st.startsWith( "-" ) ){
		neg=true;
		s=st.mid( 1 );
	}
	else s=st;
			
	int i=s.find( "." );
	int ret=s.left( i ).toInt()*ROLLOVER;

	if( i!=-1 ){ // we have a decimal point to worry about
		// only look at the most significant DECIMALS places
		QString cents=s.mid( ++i, DECIMALS ); 
		int l=cents.length();

		for( i=1; i<=l; i++ )
			ret+=cents.mid( i-1, 1 ).toInt()*( int )std::pow( ( double )10, 
																												DECIMALS-i );
	}
	
	if( neg ) ret=0-ret;

	//cout<<"converting "<<s<<" to "<<ret<<" cents"<<endl;
	return ret;
}

/***********************************************************************/
/* QHACCDB                                                             */
/*   a table-holder. Note that when accessing MySQL, the tables are    */
/*   loaded initially and saved manually. This class DOES NOT keep     */
/*   the MySQL tables synchronized with QHacc's tables. While QHacc    */
/*   is running, it uses its own table structures, not MySQL's.        */
/*                                                                     */
/*                                                                     */
/***********************************************************************/
const int QHaccDB::NUMTABLES=6;
const char * QHaccDB::DBNS[]={ "transactions", "accounts", "preferences",
															 "accounttypes", "memorized", "ledgers" };

QHaccDB::QHaccDB(){
	transactions=new QHaccTable( QHacc::TCOLTYPES, QHacc::TCOLNAMES,
															 QHacc::TCOLS, QHacc::TID, DBNS[0] );
	accounts=    new QHaccTable( QHacc::ACOLTYPES, QHacc::ACOLNAMES,
															 QHacc::ACOLS, QHacc::AID, "accounts", 0, 3, 5 );
	preferences= new QHaccTable( QHacc::PCOLTYPES, QHacc::PCOLNAMES,
															 QHacc::PCOLS, QHacc::PPREF, "preferences" );
	accounttypes=new QHaccTable( QHacc::YCOLTYPES, QHacc::YCOLNAMES,
															 QHacc::YCOLS, QHacc::YID, "accounttypes" );
	memtrans    =new QHaccTable( QHacc::TCOLTYPES, QHacc::TCOLNAMES,
															 QHacc::TCOLS, -1, "memtrans" );
	ledgers     =new QHaccTable( QHacc::LCOLTYPES, QHacc::LCOLNAMES,
															 QHacc::LCOLS, QHacc::LID, "ledgers" );

	QHaccTableIndex * junk=0;
	transactions->addIndexOn( QHacc::TACCTID, junk );
	transactions->addIndexOn( QHacc::TSPLITGROUP, junk );
	transactions->addIndexOn( QHacc::TDATE, QHacc::TID, junk );
	transactions->addIndexOn( QHacc::TPAYEE, QHacc::TDATE, junk );

	memtrans->addIndexOn( QHacc::TID, junk );
	memtrans->addIndexOn( QHacc::TSPLITGROUP, junk );

	dbs=new QHaccTable * [NUMTABLES];
	dbs[0]=transactions;
	dbs[1]=accounts;
	dbs[2]=preferences;
	dbs[3]=accounttypes;
	dbs[4]=memtrans;
	dbs[5]=ledgers;

	mysql=0;	
}

QHaccDB::~QHaccDB(){
	delete preferences;
	delete accounts;
	delete transactions;
	delete accounttypes;
	delete memtrans;
	delete ledgers;
	delete [] dbs;
#ifdef QHACCMYSQL
	if( mysql ) delete mysql;
#endif
}

QString QHaccDB::iconv( Table t, const TableRow& r ) const {
	// convert a TableRow into a string suitable for insertions into MySQL

	QString ret("(" );
	const int lkps[]={ QHacc::TCOLS, QHacc::ACOLS, QHacc::PCOLS, QHacc::YCOLS,
										 QHacc::TCOLS, QHacc::LCOLS };

	ret.append( r.get( 0 ).sqlValue() );
	for( int i=1; i<lkps[( int )t]; i++ ){
		ret.append( "," );
		ret.append( r.get( i ).sqlValue() );
	}
	return ret.append( ")" );
}

bool QHaccDB::load( const QString& home ){
	bool ret=true;
	if( home.upper().startsWith( "MYSQL:" ) ){
		std::ostream * str=0;
		if( Utils::debug( 4, str ) ) *str<<"loading tables from MySQL"<<endl;
#ifdef QHACCMYSQL

		try{
			QString strs[5];
			Utils::parser( home, ":", 0, strs, 5 );

			// -f MYSQL:DB:HOST:USER:PASSWD to connect			
			mysql=new Connection( strs[1], strs[2], strs[3], strs[4] );
			Query query=mysql->query();
			
			const uint FIELDS[]={ QHacc::TCOLS, QHacc::ACOLS, QHacc::PCOLS,
														QHacc::YCOLS, QHacc::TCOLS, QHacc::LCOLS };
			
			for( int i=0; i<NUMTABLES; i++ ) {
				QString junk;
				dbs[i]->clear();
				
				if( i==0 ){
					query<<"select id,num,payee,memo,sum,date_format(date,'%m/%d/%Y'),reco,acctid,splitgroup,ledgerid from transactions";
				}
				else query<<"select * from "<<DBNS[i];
				Result result=query.store();
				
				dbs[i]->startLoad( result.size() );
				
				Row row;
				Result::iterator j;
				const QString sep="||";
				// The Result class has a read-only Random Access Iterator
				//cout<<"loading table "<<DBNS[i]<<" from MYSQL"<<endl;
				for ( j=result.begin(); j!=result.end(); j++ ) {
					row=*j;
					QString ins=row[0].c_str();
					for( uint k=1; k<FIELDS[i]; k++ ) ins.append( sep+row[k].c_str() );
					dbs[i]->loadRow( ins );
				} 
				dbs[i]->stopLoad();
				
				//cout<<DBNS[i]<<endl;
				//for( uint k=0; k<dbs[i]->rows(); k++ )
				//cout<<" "<<dbs[i]->at( k ).toString()<<endl;
				//cout<<"end"<<endl;
			}
		}
		catch( BadQuery &er ){
			ret=false;			
			if( Utils::error( 0, str ) ) *str<<er.error<<endl;
		}
		catch( exception &er ) {
			ret=false;
			if( Utils::error( 0, str ) ) *str<<er.what()<<endl;
		}
#else
		ret=false;
		if( Utils::error( 0, str ) )
			*str<<"MySQL is not supported. Please recompile with --enable-mysql"<<endl;
#endif
	}
	else{
#ifdef QHACCMYSQL
		if( mysql ){
			delete mysql;
			mysql=0;
		}
#endif
		for( int i=0; i<NUMTABLES; i++ ) {
			QString junk;
			dbs[i]->clear();
			QFile f( home+"/"+DBNS[i] );
			dbs[i]->load( f, junk );
		}
	}
	
	
	return ret;
}

bool QHaccDB::save( const QString& home, QString& error ) {
	// save everything
	bool ret=true;
#ifdef QHACCMYSQL
	if( mysql ){
		std::ostream * str=0;
		if( Utils::debug( 4, str ) ) *str<<"data automatically saved to MySQL"<<endl;
		/*
			try{
			// delete all the info from the old tables
			Query q=mysql->query();
			for( int i=0; i<NUMTABLES; i++ ){
			QString s( "delete from " );
			s.append( DBNS[i] );
			q.exec( s.latin1() );
			}
			
			// now load the new data
			const QString st( "insert into " );
			const QString sv( " values " );
			for( int i=0; i<NUMTABLES; i++ ){
			const QString sb( DBNS[i] );

				uint rows=dbs[i]->rows();
				for( uint j=0; j<rows; j++ ){
					QString ins( st );
					ins.append( sb ).append( sv );
					ins.append( iconv( ( Table )i, dbs[i]->at( j ) ) );
					q.exec( ins.latin1() );
				}
			}
		}
		catch( BadQuery &er ){
			ret=false;			
			if( Utils::error( 0, str ) ) *str<<er.error<<endl;
		}
		catch( exception &er ) {
			ret=false;
			if( Utils::error( 0, str ) ) *str<<er.what()<<endl;
		}
		*/
		return ret;
	}
#endif
	bool chmodIT=true;
	uint idx=0;
	if( preferences->contains( TableCol( "KEEPFILEPERMS" ), idx ) &&
			preferences->at( idx ).gets( QHacc::PVALUE )=="N" ) chmodIT=false;
	
	QString mesg;
	for( int i=0; i<NUMTABLES; i++ ) {
		QFile f( home+"/"+DBNS[i] );
		if( !dbs[i]->save( f, mesg ) ){
			error=mesg;
			return false;
		}
		
		if( chmodIT ) chmod( f.name(), S_IRUSR | S_IWUSR );
	}
	return ret;
}

QHaccTable * QHaccDB::table( Table t ) const { return dbs[( int )t]; }

TableRow QHaccDB::at( Table t, uint idx ) const { 
	return table( t )->at( idx );
}

bool QHaccDB::load( Table t, QHaccTable& b ){
#ifdef QHACCMYSQL
	bool ret=true;
	if( mysql ){
		for( uint i=0; i<b.rows(); i++ ) ret=( ret && add( t, b.at( i ) ) );
		return ret;
	}
#endif
	return table( t )->load( b );
}

int QHaccDB::add( Table t, const TableRow& r ){
	bool ret=true;
#ifdef QHACCMYSQL
	if( mysql ){
		
		QString ins( "insert into " );
		ins.append( DBNS[( int )t] );
		ins.append( " values " );
		ins.append( iconv( t, r ) );
		
		try{
			Query q=mysql->query();
			q.exec( ins.latin1() );
		}
		catch( BadQuery &er ){
			ret=false;			
			std::ostream * str=0;
			if( Utils::error( 0, str ) ) *str<<er.error<<endl;
		}
		catch( exception &er ) {
			ret=false;
			std::ostream * str=0;
			if( Utils::error( 0, str ) ) *str<<er.what()<<endl;
		}
	}
#endif
	return ( ret && table( t )->add( r ) );
}

int QHaccDB::updateWhere( Table t, const TableSelect& ts, int i,
													const TableCol& replace ){
	return table( t )->updateWhere( ts, i, replace );
}

int QHaccDB::updateWhere( Table t, int position, const TableCol& equals, 
													const TableRow& replace ){
	bool ret=true;
#ifdef QHACCMYSQL
	if( mysql ){
		QHaccTable * tbl=table( t );
		const uint FIELDS[]={ QHacc::TCOLS, QHacc::ACOLS, QHacc::PCOLS,
													QHacc::YCOLS, QHacc::TCOLS, QHacc::LCOLS };
		
		QString stmt( "update " );
		stmt.append( DBNS[( int )t] );
		stmt.append( " set " );
		stmt.append( tbl->colname( 0 )+"="+replace.get( 0 ).sqlValue() );
		for( uint i=1; i<FIELDS[( int )t]; i++ )
			stmt.append( ","+tbl->colname( i )+"="+replace.get( i ).sqlValue() );
		stmt.append( " where "+tbl->colname( position )+"="+equals.sqlValue() );
		
		try{
			Query q=mysql->query();
			q.exec( stmt.latin1() );
		}
		catch( BadQuery &er ){
			ret=false;			
			std::ostream * str=0;
			if( Utils::error( 0, str ) ) *str<<er.error<<endl;
		}
		catch( exception &er ) {
			ret=false;
			std::ostream * str=0;
			if( Utils::error( 0, str ) ) *str<<er.what()<<endl;
		}
	}
#endif
	return ( ret && table( t )->updateWhere( position, equals, replace ) );
}
int QHaccDB::updateWhere( Table t, int checkpos, const TableCol& equals, 
													int changepos, const TableCol& replace ){
	bool ret=true;
#ifdef QHACCMYSQL
	if( mysql ){
		QString upd( "update " );
		upd.append( DBNS[( int )t] );
		upd.append( " set "+table( t )->colname( changepos ) );
		upd.append( "="+replace.sqlValue() );
		upd.append( " where "+table( t )->colname( checkpos ) );
		upd.append( "="+equals.sqlValue() );
		
		try{
			Query q=mysql->query();
			q.exec( upd.latin1() );
		}
		catch( BadQuery &er ){
			ret=false;			
			std::ostream * str=0;
			if( Utils::error( 0, str ) ) *str<<er.error<<endl;
		}
		catch( exception &er ) {
			ret=false;
			std::ostream * str=0;
			if( Utils::error( 0, str ) ) *str<<er.what()<<endl;
		}
	}
#endif
	return ( ret &&
					 table( t )->updateWhere( checkpos, equals, changepos, replace ) );
}

bool QHaccDB::getWhere( Table t, TableSelect * ts, uint size,
												QHaccTable *& results ){
	return table( t )->getWhere( ts, size, results );
}
bool QHaccDB::getWhere( Table t, const TableSelect& s, QHaccTable *& results ){
	return table( t )->getWhere( s, results );
}

uint QHaccDB::deleteWhere( Table t, const TableSelect& ts ){
	bool ret=true;
#ifdef QHACCMYSQL
	if( mysql ){
		int position=0, checktype=0;
		TableCol col;
		ts.getAll( position, col, checktype );

		
		QString del( "delete from " );
		del.append( DBNS[( int )t] );
		del.append( " where "+table( t )->colname( position ) );

		try{
			Query q=mysql->query();
			q.exec( del.latin1() );
		}
		catch( BadQuery &er ){
			ret=false;			
			std::ostream * str=0;
			if( Utils::error( 0, str ) ) *str<<er.error<<endl;
		}
		catch( exception &er ) {
			ret=false;
			std::ostream * str=0;
			if( Utils::error( 0, str ) ) *str<<er.what()<<endl;
		}
	}
#endif
	return ( ret && table( t )->deleteWhere( ts ) );
}

uint QHaccDB::deleteWhere( Table t, int position, const TableCol& equals ){
	bool ret=true;
#ifdef QHACCMYSQL
	if( mysql ){

		QString del( "delete from " );
		del.append( DBNS[( int )t] );
		del.append( " where "+table( t )->colname( position ) );
		del.append( "="+equals.sqlValue() );
		
		try{
			Query q=mysql->query();
			q.exec( del.latin1() );
		}
		catch( BadQuery &er ){
			ret=false;			
			std::ostream * str=0;
			if( Utils::error( 0, str ) ) *str<<er.error<<endl;
		}
		catch( exception &er ) {
			ret=false;
			std::ostream * str=0;
			if( Utils::error( 0, str ) ) *str<<er.what()<<endl;
		}
	}
#endif
	return ( ret && table( t )->deleteWhere( position, equals ) );
}
uint QHaccDB::deleteWhere( Table t, QHaccTable * b ){
	bool ret=true;
#ifdef QHACCMYSQL
	if( mysql ){
		int idcol=table( t )->idcol();
		if( idcol>=0 ){
			for( uint i=0; i<b->rows(); i++ ){
				const TableSelect ts( idcol, b->at( i ).get( idcol ) );
				ret=( ret && deleteWhere( t, ts ) );
			}
		}
		else{
			QHaccTable * tbl=table( t );
			const uint FIELDS[]={ QHacc::TCOLS, QHacc::ACOLS, QHacc::PCOLS,
														QHacc::YCOLS, QHacc::TCOLS, QHacc::LCOLS };

			try{
				Query q=mysql->query();

				for( uint i=0; i<b->rows(); i++ ){
					TableRow r=b->at( i );

					QString stmt( "delete from " );
					stmt.append( DBNS[( int )t] );
					stmt.append( " where " );
					stmt.append( tbl->colname( 0 )+"="+r.get( 0 ).sqlValue() );
					for( uint i=1; i<FIELDS[( int )t]; i++ )
						stmt.append( " and "+tbl->colname( i )+"="+r.get( i ).sqlValue() );
					
					q.exec( stmt.latin1() );
				}
			}
			catch( BadQuery &er ){
				ret=false;			
				std::ostream * str=0;
				if( Utils::error( 0, str ) ) *str<<er.error<<endl;
			}
			catch( exception &er ) {
				ret=false;
				std::ostream * str=0;
				if( Utils::error( 0, str ) ) *str<<er.what()<<endl;
			}
		}
	}
#endif
	return ( ret && table( t )->deleteWhere( b ) );
}

bool QHaccDB::contains( Table t, const TableCol& tc, uint& i ) const{
	return table( t )->contains( tc, i );
}
bool QHaccDB::contains( Table t, const TableRow& tr, uint& i ) const{
	return table( t )->contains( tr, i );
}

uint QHaccDB::rows( Table t ) const { return table( t )->rows(); }
TableCol QHaccDB::maxID( Table t ) const { return table( t )->maxID(); }
TableCol QHaccDB::minID( Table t ) const { return table( t )->minID(); }
bool QHaccDB::isEmpty( Table t ) const { return table( t )->isEmpty(); }

void QHaccDB::startLoad( Table t, uint i ){ table( t )->startLoad( i ); }
void QHaccDB::stopLoad( Table t ){ table( t )->stopLoad(); }

void QHaccDB::indexOn( Table t, int col, QHaccTableIndex *& ret ){
	table( t )->addIndexOn( col, ret );
}

bool QHaccDB::autosave() const { return ( mysql!=0 ); }
