/************************* * * * * * * * * * * * * ***************************
    Copyright (c) 1999-2003 Ryan Bobko
                       ryan@ostrich-emulators.com

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.     
************************** * * * * * * * * * * * * **************************/

#include "qhacc.h"
#include "qhaccutils.h"
#include "qhaccdialogs.h"
#include "qhaccacctchsr.h"
#include "qhaccacctchsr.moc"

#include <qheader.h>
#include <qpixmap.h>
#include <qpainter.h>
#include <qstrlist.h>
#include <qpopupmenu.h>
#include <qmessagebox.h>


const int QHaccAccountChooser::FNAME=0;
const int QHaccAccountChooser::BAL=  1;
const int QHaccAccountChooser::NBAL= 2;
const int QHaccAccountChooser::ID=   3;
const int QHaccAccountChooser::DEPTH=4;
const int QHaccAccountChooser::SNAME=5;
const int QHaccAccountChooser::COLS= 6;

const char * QHaccAccountChooser::COLNAMES[]={ "NAME", "BAL", "NBAL", "ID",
																							 "DEPTH", "SHORTNAME" };
const ColType QHaccAccountChooser::COLTYPES[]={ CTSTRING, CTSTRING, CTINT,
																								CTUINT,	CTUINT, CTSTRING };


QHaccAccountChooser::QHaccAccountChooser( QHacc * eng, QWidget * p,
																					const char * c )
	: QHaccScrollView( eng, p, c ){
	
  setVScrollBarMode( AlwaysOn );
  QHeader * head=header();
  head->addLabel( tr( "Account" ) );
  head->addLabel( tr( "Balance" ) );
	head->setMovingEnabled( false );
  
	connect( engine, SIGNAL( addedA( const Account& ) ), SLOT( makeData() ) );
	connect( engine, SIGNAL( updatedA( const Account&, const Account& ) ),
					 this, SLOT( updData( const Account&, const Account& ) ) );
	connect( engine, SIGNAL( removedA( const Account& ) ), SLOT( makeData() ) );

  QPopupMenu * popup=new QPopupMenu( this );
  popup->insertItem( tr( "reconcile" ), this, SLOT( recoAcct() ) );
  popup->insertItem( tr( "graph" ), this, SLOT( chart() ) );
  popup->insertItem( tr( "report" ), this, SLOT( report() ) );

  popup->insertItem( tr( "edit" ), this, SLOT( editAcct() ) );
  popup->insertItem( tr( "new" ), this, SLOT( newAcct() ) );
  popup->insertItem( tr( "delete" ), this, SLOT( remAcct() ) );  
	
  popup->insertSeparator();
  popup->insertItem( tr( "toggle &view" ), this,
										 SLOT( toggleView() ), CTRL+Key_V );
  popup->insertItem( tr( "view su&bset" ), this,
										 SLOT( viewSubset() ) );

	setPopup( popup );
}

QHaccAccountChooser::~QHaccAccountChooser(){}

void QHaccAccountChooser::isave() const {
	engine->setIP( "SHOWINDEX", getChosen() );
	engine->setIP( "CHSRSORTCOL", sortCol() );
	engine->setBP( "CHSRSORTASC", sortAsc() );

	QString num, str;
  QHeader * head=header();	
	for( int i=0; i<2; i++ )
		str.append( num.setNum( head->sectionSize( i ) )+" " );
	engine->setSP( "CHSRHEADINGSIZES", str );
}

void QHaccAccountChooser::ireadPrefs( bool initial ) {
	if( initial ){
		connect( engine, SIGNAL( changedP( const QString&, bool ) ),
						 this, SLOT( changeP( const QString&, bool ) ) );
	}

	hideCats=engine->getBP( "HIDECATEGORIES" );
	int indexer=engine->getIP( "CHSRSORTCOL" );
	bool asc=engine->getBP( "CHSRSORTASC" );
	setSorter( indexer, asc );

  QHeader * head=header();	
	QString ss=engine->getSP( "CHSRHEADINGSIZES" );
	if( !ss.isEmpty() ){
		QString vals[2];
		Utils::parser( ss, " ", 0, vals, 2 );
		for( int i=0; i<2; i++ )	head->resizeSection( i, vals[i].toInt() );
	}

	makeData();

	setChosen( engine->getIP( "SHOWINDEX" ) );
}

void QHaccAccountChooser::doSetChosen( int ){
	emit changedAccount( getSelected() );
}

void QHaccAccountChooser::viewportMouseDoubleClickEvent( QMouseEvent * ){
	editAcct();
}

void QHaccAccountChooser::grabData( QHaccTable *& ret ){
	std::ostream * str=0;
	if( Utils::debug( 6, str ) ) *str<<"acctchsr grabbing data!"<<endl;
	// we really only care about the account's name and it's balance...
	// just make a small table to hold the important info

	QHaccTable * table=0;
	engine->getAs( hideCats, table );

	ret=new QHaccTable( COLTYPES, COLNAMES, COLS, ID, 
											"acctchsr table", table->rows() );

	int sum=0;
	ret->startLoad();
	for( uint i=0; i<table->rows(); i++ ){
		TableRow row=conv( table->at( i ) );
		ret->add( row );
		sum+=row.geti( NBAL );
	}
	ret->stopLoad();
	delete table;

	// totrow is the row that contains the "TOTAL" line
	uint zip=0;
	TableCol tcs[]={ TableCol( tr( "Total" ) ),
									 TableCol( engine->convMoney( sum ) ), TableCol( sum ),
									 TableCol( zip ), TableCol( zip ),
									 TableCol( tr( "Total" ) ) };
	totrow=TableRow( tcs, COLS );
}

void QHaccAccountChooser::updData( const Account& acct, const Account& newy ){
	//we only show three bits of account information here: name, fullname,
	// and balance, so id those things haven't changed, don't do anything

	Account row=conv( newy );

	if( ( hideCats &&
			acct.getb( QHacc::ACATEGORY!=newy.getb( QHacc::ACATEGORY ) ) ) ||
			( acct.getu( QHacc::APID )!=newy.getu( QHacc::APID ) ) ){
		// if the category flag changes, remake the datatable
		makeData();
	}
	else if( !( ( engine->convMoney( acct.gets( QHacc::ACBAL ) )==
								engine->convMoney( newy.gets( QHacc::ACBAL ) ) ) &&
					 acct.gets( QHacc::ANAME )==newy.gets( QHacc::ANAME ) ) ){
		// if the balance or name changed, remake the row only

		updateRow( row, row );
	}
}


void QHaccAccountChooser::makeIndexFor( int col, 
																				QHaccTableIndex *& idx ) const {
	if( col==0 ) data->addIndexOn( FNAME, ID, idx );
	else data->addIndexOn( NBAL, ID, idx );
}

void QHaccAccountChooser::drawRow( QPainter * p, int y, int h, Account& acct ){
	if( acct.isNull() )	acct=totrow;

	int space=0;
	QHeader * heads=header();
	int x=heads->sectionPos( 0 )+1;
	int w=heads->sectionSize( 0 )-2;

	QString name;
	if( sortCol()==BAL ) name=acct.gets( FNAME );
	else{
		int ww=fontMetrics().width( "w" );
		name=acct.gets( SNAME );
		for( uint i=0; i<acct.getu( DEPTH ); i++ )	space+=ww;
	}
	
	x+=space;
	w-=space;

	p->drawText( x, y, w, h, AlignLeft, name );

	x=heads->sectionPos( 1 );
	w=heads->sectionSize( 1 );
	p->drawText( x, y, w, h, AlignRight,
							 engine->convMoney( engine->convMoney( acct.gets( BAL ) ) ) );
}

TableRow QHaccAccountChooser::uconv( const TableRow& row ) const {
	return engine->getA( row.getu( ID ) );
}

TableRow QHaccAccountChooser::conv( const Account& acct ) const {
	TableCol balc=acct.get( QHacc::ACBAL );
	TableCol tcs[]={ TableCol( engine->getFNameOfA( acct ) ),
									 balc,
									 TableCol( engine->convMoney( balc.gets() ) ),
									 acct.get( QHacc::AID ),
									 TableCol( engine->aDepth( acct.getu( QHacc::AID ) ) ),
									 acct.get( QHacc::ANAME ) };
	return TableRow( tcs, COLS );
}

void QHaccAccountChooser::editAcct(){
	Account acct=getMoused();
  if ( acct.isNull() ) newAcct();
  else ( new AccountDlg( engine, acct, this ) )->show();
}
void QHaccAccountChooser::newAcct(){
	Account acct;
	( new AccountDlg( engine, acct, this ) )->show();
}
void QHaccAccountChooser::remAcct(){ deleteRow( getIUM() ); }

void QHaccAccountChooser::deleteRow( int i ){
	AccountDlg::qremove( engine, uconv( getData( i ) ), this );
}

void QHaccAccountChooser::toggleView(){
	engine->setBP( "HIDECATEGORIES", !hideCats );
}

void QHaccAccountChooser::changeP( const QString& pref, bool val ){
	if( pref=="HIDECATEGORIES" ){
		hideCats=val;
		makeData();
	}
}

void QHaccAccountChooser::recoAcct(){ emit needRecWinOf( getMoused() ); }
void QHaccAccountChooser::viewSubset(){	emit needSubsetOf( getMoused() ); }
void QHaccAccountChooser::chart(){ emit needGraphOf( getMoused() ); }
void QHaccAccountChooser::report(){ emit needReportOf( getMoused() ); }
