/************************* * * * * * * * * * * * * ***************************
    Copyright (c) 1999-2003 Ryan Bobko
                       ryan@ostrich-emulators.com

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.     
************************** * * * * * * * * * * * * **************************/

#include "config.h"
#include "qhaccdialogs.h"
#include "qhacclineedits.h"
#include "qhaccdialogs.moc"

#include <qlabel.h>
#include <qlayout.h>
#include <qcheckbox.h>
#include <qcombobox.h>
#include <qfontdialog.h>
#include <qpushbutton.h>
#include <qmessagebox.h>
#include <qradiobutton.h>
#include <qbuttongroup.h>
#include <qcolordialog.h> 
#include <qmultilineedit.h>

const char * AccountDlg::INCRSTR="<incr number>";

void AccountDlg::qremove( QHacc * eng, const Account& a, QWidget * p ){
	if( !a.isNull() ){
		if ( QMessageBox::warning( p, tr( "Delete " )+a.gets( QHacc::ANAME ),
															 tr( "Are you sure?" ), tr( "Delete Account" ),
															 tr( "Reconsider" ) ) == 0 ) eng->removeA( a );
	}
}

AccountDlg::AccountDlg( QHacc * eng, const Account& acct, QWidget * p,
												const char * n, bool modal ) 
	: QTabDialog( p, n, modal ){

	engine=eng;
	setFont( eng->getWP( "FONT" ) );
	
	if( acct.isNull() ){
		edit=false;
		account=engine->getBlankA();
		setCaption( tr( "Create New Account" ) );
	}
	else{
		edit=true;
		account=acct;
		setCaption( tr( "Edit" )+" "+account.gets( QHacc::ANAME ) );
	}

	tab1=new AccountTab1( eng, account, this );
	tab2=new AccountTab2( eng, account, this );
	
  addTab( tab1, tr( "Account" ) );
  addTab( tab2, tr( "Institution" ) );
  
  setCancelButton( tr( "Cancel" ) );

}

AccountDlg::~AccountDlg(){}
Account AccountDlg::getAccount() const { return account; }
void AccountDlg::close(){ QTabDialog::close( true ); }
void AccountDlg::accept(){
	Account newy( account );
	tab1->modify( newy );
	tab2->modify( newy );

	if( edit ) engine->updateA( account, newy ); 
	else engine->addA( newy );

	account=newy;
	QTabDialog::accept();
}

/* * * * * * * * * * * * * * * * */
/* * * * TAB 1 INFO* * * * * * * */
/* * * * * * * * * * * * * * * * */

AccountTab1::AccountTab1 ( QHacc * eng, const Account& a, 
													 QWidget * w, const char * n ) : QWidget( w, n ){
			
	engine=eng;

	QGridLayout * layout=new QGridLayout( this, 12, 5, 5 );
	name=new QLineEdit( this );
	number=new QLineEdit( this );
	bal=new QLineEdit( this );
	taxed=new QCheckBox( this );
	category=new QCheckBox( this );
  typeBox=new QButtonGroup();	
	budget=new QLineEdit( this );

	QHaccTable * accttypes=0;
	engine->getATs( accttypes );
	uint ats=accttypes->rows();
	int selid=-1;
	int sel=engine->getIP( "DEFAULTACCOUNTTYPE" );
	for( uint i=0; i<ats; i++ ){
		QRadioButton * qrb=0;
		TableRow row=accttypes->at( i );
		qrb=new QRadioButton( row.gets( QHacc::YNAME ), this );
		int j=typeBox->insert( qrb, row.getu( QHacc::YID ) );
		if( ( int )i==sel ) selid=j;

	}
	typeBox->setButton( selid );

	parent=new QHaccChoiceEdit( false, this );
	parent->ignoreCase( engine->getBP( "AUTOCOMPNOCASE" ) );
	transtypes=new QLineEdit( this );
	dttype=new QHaccChoiceEdit( false, this );
	dttype->ignoreCase( engine->getBP( "AUTOCOMPNOCASE" ) );


	QString stypes;
	QStringList ttypes, parents;
	
	ttypes=QStringList::split( " ", a.gets( QHacc::ATRANSNUMS ), false );
	QStringList::iterator it;
	bool notfirst=false;
	for( it=ttypes.begin(); it!=ttypes.end(); ++it ){
		if( notfirst ) stypes.append( " " );
		else notfirst=true;

		stypes.append( *it );
	}


	// go through strings and remove this account's name, and
	// names of any children of this account--we don't want
	// circular parentage    
	QString aname=engine->getFNameOfA( a );
	int length=aname.length();


	// get a list of possible parent accounts to add to parent choice box
	QHaccTable * accts=0;
	engine->getAs( false, accts );
	uint naccts=accts->rows();
	for( uint i=0; i<naccts; i++ ){
		Account acct=accts->at( i );
		QString str=engine->getFNameOfA( acct );
		if( str.left( length )!=aname ) parents.append( str );
	}

	/* set all the fields */
	name->setText( a.gets( QHacc::ANAME ) );
	number->setText( a.gets( QHacc::ANUM ) );
	bal->setText( a.gets( QHacc::AOBAL ) );
	transtypes->setText( stypes );
	budget->setText( QString().setNum( a.getf( QHacc::ABUDGET ), 'f', 2 ) );

	parent->insertStringList( parents );
	parent->setEmptyText( tr( "<top level>" ) );

  dttype->insertItem( tr( AccountDlg::INCRSTR ) );
	dttype->insertStringList( ttypes );
	dttype->setEmptyText( tr( "<none>" ) );

	taxed->setChecked( a.getb( QHacc::ATAXED ) );
	category->setChecked( a.getb( QHacc::ACATEGORY ) );

	typeBox->setButton( a.getu( QHacc::ATYPE ) );

	QString ttext=a.gets( QHacc::ADEFAULTNUM );
	dttype->setText( ttext );

	uint pid=a.getu( QHacc::APID );
	if ( pid ) parent->setText( engine->getFNameOfA( engine->getA( pid ) ) );

	// add everything to the layout
  int row=0;
	const char * names[]={ QT_TR_NOOP( "Account Name" ),
												 QT_TR_NOOP( "Account Number" ),
												 QT_TR_NOOP( "Opening Balance" ),
												 QT_TR_NOOP( "Account is Taxable" ),
												 QT_TR_NOOP( "Account is a Category" ),
												 QT_TR_NOOP( "Account Type" ),
												 QT_TR_NOOP( "Child Of" ),
												 QT_TR_NOOP( "Transaction Types" ),
												 QT_TR_NOOP( "Default Transaction Type" ),
												 QT_TR_NOOP( "Monthly Budget Limit" ) };
	
	QWidget * ws[]={ name, number, bal, taxed, category, typeBox, parent,
									 transtypes, dttype, budget };
	QWidget * last=0;
  for ( int i=0; i<10; i++ ){
    layout->addMultiCellWidget( new QLabel( tr( names[i] ), this ),
																row, row, 0, 3 );
		
		// add all the different account types
    if ( i==5 ){
      for ( uint j=0; j<accttypes->rows(); j++ ){
				TableRow trow=accttypes->at( j );
				uint id=trow.getu( QHacc::YID );
				layout->addMultiCellWidget( typeBox->find( id ), row, row, 4, 9 );
				row++;
      }
    }
    else layout->addMultiCellWidget( ws[i], row, row, 4, 9 );
    row++;

		if( last ) setTabOrder( last, ws[i] );
		last=ws[i];
  }


	connect( transtypes, SIGNAL( textChanged( const QString& ) ),
					 this, SLOT( makeTTChoices() ) );


	delete accts;
	delete accttypes;
}

AccountTab1::~AccountTab1(){ delete typeBox; }

void AccountTab1::makeTTChoices(){
	QString types=transtypes->text().simplifyWhiteSpace();
	QStringList listo=QStringList::split( " ", types, false );
	QString type=dttype->text();

	dttype->clear();
	dttype->insertItem( AccountDlg::INCRSTR );
	dttype->insertStringList( listo );
	dttype->setText( type );
}


void AccountTab1::modify( Account& acct ) const {
	acct.set( QHacc::AOBAL, TableCol( bal->text() ) );
	acct.set( QHacc::ANAME, TableCol( name->text() ) );
	acct.set( QHacc::ANUM,  TableCol( number->text() ) );
	acct.set( QHacc::ATRANSNUMS,  TableCol( transtypes->text() ) );
	acct.set( QHacc::ADEFAULTNUM, TableCol( dttype->text() ) );
	acct.set( QHacc::ABUDGET, TableCol( budget->text() ) );
	acct.set( QHacc::ATYPE,
						 TableCol( ( uint )( typeBox->id( typeBox->selected() ) ) ) );

	Account pacc=engine->getA( parent->text() );
	if( pacc.isNull() ) acct.set( QHacc::APID, TableCol( ( uint )0 ) );
	else acct.set( QHacc::APID, TableCol( pacc.getu( QHacc::AID ) ) );

	acct.set( QHacc::ATAXED, TableCol( taxed->isChecked() ) );
	acct.set( QHacc::ACATEGORY, TableCol( category->isChecked() ) );
}

/* * * * * * * * * * * * * * * * */
/* * * * TAB 2 INFO* * * * * * * */
/* * * * * * * * * * * * * * * * */

AccountTab2::AccountTab2 ( QHacc *, const Account& a,
													 QWidget * p, const char * n ) : QWidget( p, n ){
	
  QGridLayout * layout=new QGridLayout( this, 14, 9, 5 );

	// FIXME: is there a better way than this?
	int limit=11, offset=9;
	instinfo=new QFrame * [limit];

	for( int i=0; i<limit; i++ ){
		int field=i+offset;
		QString data=a.gets( field );

		if( field==QHacc::AINSTNOTES ){
			data=data.replace( QRegExp( "qhacclinestop" ), "\n" );
			instinfo[i]=new QMultiLineEdit( this );
			( ( QMultiLineEdit * )instinfo[i] )->setText( data );
			( ( QMultiLineEdit * )instinfo[i] )->setWordWrap( QMultiLineEdit::WidgetWidth );
		}
		else instinfo[i]=new QLineEdit( data, this );
	}
	
  int row=0;
	const char * names[]={ QT_TR_NOOP( "Name" ), QT_TR_NOOP( "Address" ), "",
												 QT_TR_NOOP( "City" ), QT_TR_NOOP( "State" ),
												 QT_TR_NOOP( "Zip" ), QT_TR_NOOP( "Contact" ),
												 QT_TR_NOOP( "Phone" ), QT_TR_NOOP( "Fax" ),
												 QT_TR_NOOP( "Email" ), QT_TR_NOOP( "Notes" ) };
	for( int i=0; i<limit; i++ ){
		layout->addMultiCellWidget( new QLabel( tr( names[i] ), this ),
																row, row, 0, 3 );

		if( ( i+offset )==( int )QHacc::AINSTSTATE ){
			// put the zipcode on the same line as the state
			layout->addMultiCellWidget( instinfo[i], row, row, 4, 5 ); // state field

			// zip label
      layout->addMultiCellWidget( new QLabel( tr( names[++i] ), this ), 
																	row, row, 7, 8 );
      layout->addMultiCellWidget( instinfo[i], row, row, 9, 13 ); // zip field
		}

		else layout->addMultiCellWidget( instinfo[i], row, row, 4, 13 );

		row++;
	}
}

AccountTab2::~AccountTab2(){ delete [] instinfo; }
void AccountTab2::modify( Account& acct ) const {
	int offset=9, limit=11;

	for( int i=0; i<limit; i++ ) {
		int field=i+offset;
		if( field==( int )QHacc::AINSTNOTES ){
			QString txt=( ( QMultiLineEdit * )instinfo[i] )->text();
			txt=txt.replace( QRegExp( "\n" ), "qhacclinestop" );
			acct.set( field, TableCol( txt ) );								 
		}
		else acct.set( field, TableCol( ( ( QLineEdit * )instinfo[i] )->text() ) );
	}
}



/* * * * * * * * * * * * * * * * */
/* * * * * PREFS DIALOG  * * * * */
/* * * * * * * * * * * * * * * * */

PrefsDlg::PrefsDlg( QHacc * eng, int tab, QWidget * w, const char * n )
	: QTabDialog( w, n, false, WDestructiveClose ){
	
	engine=eng;

	setFont( eng->getWP( "FONT" ) );
	
	tab1=new QHaccPrefsWidget( eng, this );
  addTab( tab1, tr( "General Preferences" ) );

	tab2=new QHaccGraphPrefsWidget( eng, this );
  addTab( tab2, tr( "Graph and Reporting" ) );

	tab3=new QHaccOtherPrefsWidget( eng, this );
  addTab( tab3, tr( "Miscellaneous" ) );


	if( tab==1 ) showPage( tab2 );
	else if( tab==2 ) showPage( tab3 );

  setCancelButton( tr( "Dismiss" ) );
	setOKButton( tr( "Accept" ) );

	setCaption( tr( "QHacc Preferences" ) );
	//connect( this, SIGNAL( applyButtonPressed() ), SLOT( accept() ) );
}

PrefsDlg::~PrefsDlg(){}

void PrefsDlg::accept(){
	tab1->accept();
	tab2->accept();
	tab3->accept();
	QTabDialog::accept();
}

QHaccOtherPrefsWidget::QHaccOtherPrefsWidget( QHacc * eng, QWidget * w,
																							const char * n )
  : QWidget( w, n ){

	engine=eng;

  QGridLayout * layout=new QGridLayout( this, 21, 11, 5 );

	int row=1;
	simpleFD=new QCheckBox( this );
	simpleFD->setChecked( engine->getBP( "SIMPLEFILEDIALOG" ) );
	layout->addMultiCellWidget( new QLabel( tr( "Use simple QHACC_HOME dialog" ),
																					this ), row, row, 1, 3 );
	layout->addMultiCellWidget( simpleFD, row, row, 4, 9 );
	row+=2;

	QHaccTable * temp=0;
	engine->getATs( temp );

	uint trows=temp->rows();
  defAcct=new QButtonGroup( 2, Vertical, tr( "Default Account Type" ),
														this );
	for( uint i=0; i<trows; i++ ){
		new QRadioButton( temp->at( i ).gets( QHacc::YNAME ), defAcct );
	}
	defAcct->setButton( engine->getIP( "DEFAULTACCOUNTTYPE" ) );

	int add=trows/2;
	layout->addMultiCellWidget( defAcct, row, row+add, 1, 9 );
	row+=add;

}


QHaccOtherPrefsWidget::~QHaccOtherPrefsWidget(){}
void QHaccOtherPrefsWidget::accept(){
	engine->setBP( "SIMPLEFILEDIALOG", simpleFD->isChecked() );
	engine->setIP( "DEFAULTACCOUNTTYPE", defAcct->id( defAcct->selected() ) );
}


QHaccGraphPrefsWidget::QHaccGraphPrefsWidget( QHacc * eng, QWidget * w,
																							const char * n )
  : QWidget( w, n ){

	engine=eng;

  //QGridLayout * layout=new QGridLayout( this, 10, 10, 5 );
  QGridLayout * layout=new QGridLayout( this, 21, 11, 5 );

  grapher=new QButtonGroup( 3, Horizontal, tr( "Default Graph" ), this );
	const char * gs[]={ QT_TR_NOOP( "Line" ), QT_TR_NOOP( "Bar" ),
											QT_TR_NOOP( "Two Line" ), QT_TR_NOOP( "Two Bar" ),
											QT_TR_NOOP( "Pie" ) };
	for( int i=0; i<5; i++ ){ new QRadioButton( tr( gs[i] ), grapher ); }
	grapher->setButton( engine->getIP( "GRAPHTYPE" ) );

  reporter=new QButtonGroup( 3, Horizontal, tr( "Default Report" ), this );
	new QRadioButton( tr( "Totals by Payee" ), reporter );
	new QRadioButton( tr( "Monthly Balances" ), reporter );
	new QRadioButton( tr( "Monthly Deltas" ), reporter );
	new QRadioButton( tr( "Transaction Averages" ), reporter );
	new QRadioButton( tr( "Journal" ), reporter );
	new QRadioButton( tr( "Budget" ), reporter );
	reporter->setButton( engine->getIP( "REPORTTYPE" ) );

	grtime=new QLineEdit( QString().setNum( engine->getIP( "GRAPHTIME" ) ),
												this );
	QIntValidator * v=new QIntValidator( grtime );
	v->setBottom( 1 );
	grtime->setValidator( v );
	grtime->setAlignment( AlignCenter );

	mergepct=new QLineEdit( QString().setNum( engine->getFP( "MERGEPCT" )*100 ),
													this );
	mergepct->setAlignment( AlignCenter );


	heads=new QCheckBox( this );
	lines=new QCheckBox( this );
	totals=new QCheckBox( this );
	deltas=new QCheckBox( this );
	piepayees=new QCheckBox( this );


	int row=1;
  layout->addMultiCellWidget( grapher, row, row, 1, 9 );
  row+=2;

  layout->addMultiCellWidget( reporter, row, row, 1, 9 );
  row+=2;

	layout->addMultiCellWidget( new QLabel( tr( "Months to Display" ), this ),
															row, row, 1, 4 );
	layout->addMultiCellWidget( grtime, row, row, 5, 6 );
	row++;

	layout->addMultiCellWidget( new QLabel( tr( "Pie Graph Merge %" ), this ),
															row, row, 1, 4 );
	layout->addMultiCellWidget( mergepct, row, row, 5, 6 );
	row++;

	QCheckBox * bs[]={ heads, lines, totals, deltas, piepayees };
	const char * hs[]={ QT_TR_NOOP( "Show Names in Multi-Account Graphs" ),
											QT_TR_NOOP( "Draw Guidelines" ), 
											QT_TR_NOOP( "Show Totals Above Graph Values" ), 
											QT_TR_NOOP( "Bar and Line Graphs Display Deltas, Not Totals" ),
											QT_TR_NOOP( "Sort Double-Entry Pie Graphs on Payee" ) };
	const char * cs[]={ "GRAPHHEADS", "GRAPHGUIDES", "GRAPHSHOWTOTALS",
											"GRAPHDELTAS", "GRAPHPIEPAYEES" };

	for( int i=0; i<5; i++ ) {
		
		bs[i]->setChecked( engine->getBP( cs[i] ) );
		layout->addMultiCellWidget( new QLabel( tr( hs[i] ), this ),
																row, row, 1, 4 );
		layout->addMultiCellWidget( bs[i], row, row, 5, 6 );
		row++;
	}
}

QHaccGraphPrefsWidget::~QHaccGraphPrefsWidget(){}
void QHaccGraphPrefsWidget::accept(){
	int ints[]={ grapher->id( grapher->selected() ),
							 reporter->id( reporter->selected() ),
							 grtime->text().toInt() };
	QCheckBox * cs[]={ lines, heads, deltas, totals, piepayees };
	const char * ps[]={ "GRAPHTYPE", "REPORTTYPE", "GRAPHTIME", "GRAPHGUIDES",
											"GRAPHHEADS", "GRAPHDELTAS", "GRAPHSHOWTOTALS",
											"GRAPHPIEPAYEES" };

	for( int i=0; i<3; i++ ) engine->setIP( ps[i], ints[i] );
	for( int i=0; i<5; i++ ) engine->setBP( ps[i+3], cs[i]->isChecked() );
	
	float f=mergepct->text().toFloat();
	engine->setFP( "MERGEPCT", f/100 );
}


QHaccPrefsWidget::QHaccPrefsWidget( QHacc * eng, QWidget * w,	const char * n )
  : QWidget( w, n ){
	engine=eng;

  QGridLayout * layout=new QGridLayout( this, 21, 11, 5 );

  mbtn=new QPushButton( tr( "Main Color" ), this );
  mbtn->setPalette( engine->getCP( "MAINCOLOR" ) );
  connect( mbtn, SIGNAL( clicked() ), SLOT( mColor() ) );
  
  abtn=new QPushButton( tr( "Alt Color" ), this );
  abtn->setPalette( engine->getCP( "ALTCOLOR" ) );
  connect( abtn, SIGNAL( clicked() ), SLOT( aColor( ) ) );

  title=new QLineEdit( engine->getSP( "TITLEBARNAME" ), this );

  cats=new QCheckBox( this );
  cats->setChecked( engine->getBP( "HIDECATEGORIES" ) );

  hideleds=new QCheckBox( this );
  hideleds->setChecked( engine->getBP( "HIDELEDGERS" ) );

  showrec=new QCheckBox( this );
  showrec->setChecked( engine->getBP( "SHOWRECBAL" ) );

  fbtn=new QPushButton( "Change this Font", this );
  connect( fbtn, SIGNAL( clicked() ), SLOT( cFont( ) ) );

  symbol=new QLineEdit( engine->getSP( "CSYMBOL" ), this );
  symbol->setAlignment( AlignCenter );

  sep=new QLineEdit( engine->getSP( "DATESEPARATOR" ), this );
  sep->setAlignment( AlignCenter );
  sep->setMaxLength( 1 );

  liner=new QButtonGroup( 2, Horizontal, tr( "Bookkeeping Method" ), this );
  new QRadioButton( tr( "Single Entry" ), liner );
  new QRadioButton( tr( "Double Entry" ), liner );
  liner->setButton( engine->getBP( "DOUBLEENTRY" ) ? 1 : 0 );

  dater=new QButtonGroup( 2, Horizontal, tr( "Date Format" ), this );
  new QRadioButton( tr( "American (MM/DD/YYYY)" ), dater );
  new QRadioButton( tr( "European (DD/MM/YYYY)" ), dater );
  new QRadioButton( tr( "Year First (YYYY/MM/DD)" ), dater );
	dater->setButton( engine->getIP( "DATEFORMAT" ) );

  int row=0;
  layout->addMultiCellWidget( mbtn, row, row+1, 1, 4 );
  layout->addMultiCellWidget( abtn, row, row+1, 6, 9 );

  row+=2;
  layout->addMultiCellWidget( liner, row, row+2, 1, 9 );
  row+=3;
  layout->addMultiCellWidget( dater, row, row+2, 1, 9 );
  row+=3;

  QWidget * wa[]={ cats, hideleds, showrec, sep, symbol };
	const char * ns[]={ QT_TR_NOOP( "Hide Categories in Chooser" ),
											QT_TR_NOOP( "Hide Ledgers" ),
											QT_TR_NOOP( "Show Reconciled Balance in Status" ),
											QT_TR_NOOP( "Date Separator" ),
											QT_TR_NOOP( "Currency Symbol" ) };	
  
  for ( int i=0; i<5; i++ ){
    layout->addMultiCellWidget( new QLabel( tr( ns[i] ), this ),
																row, row, 1, 5 );
    layout->addMultiCellWidget( wa[i], row, row, 6, 7 );
    row++;
  }

  layout->addMultiCellWidget( fbtn, row, row, 1, 9 );
  row+=2;

  transTypes=new QLineEdit( engine->getSP( "TRANSTYPES" ), this );  

  autocomps=new QComboBox( false, this );
  autocomps->insertItem( tr( "No Transactions" ) );
  autocomps->insertItem( tr( "New Transactions Only" ) );
  autocomps->insertItem( tr( "All Transactions" ) );
  autocomps->setCurrentItem( engine->getIP( "AUTOCOMPLETION" ) );

	const char * wsn[]={ QT_TR_NOOP( "Title in Titlebar" ), 
											 QT_TR_NOOP( "Transaction Types" ),
											 QT_TR_NOOP( "Auto-Complete on " ) };
	QWidget * ws[]={ title, transTypes, autocomps };
  for ( int i=0; i<3; i++ ){
    layout->addMultiCellWidget( new QLabel( tr( wsn[i] ), this ),
																row, row, 1, 4 );
    layout->addMultiCellWidget( ws[i], row, row, 5, 9 );
    row++;
  }

	igcase=new QCheckBox( this );
  igcase->setChecked( engine->getBP( "AUTOCOMPNOCASE" ) );
	layout->addMultiCellWidget( new QLabel( tr( "Auto-Complete Ignores Case" ),
																					this ),	row, row, 1, 4 );
	layout->addMultiCellWidget( igcase, row, row, 5, 9 );

	row++;
	gtypeahead=new QCheckBox( this );
  gtypeahead->setChecked( engine->getBP( "GLOBALTYPEAHEAD" ) );
	layout->addMultiCellWidget( new QLabel( tr( "Global Auto-Complete" ),
																					this ),	row, row, 1, 4 );
	layout->addMultiCellWidget( gtypeahead, row, row, 5, 9 );
}

QHaccPrefsWidget::~QHaccPrefsWidget(){}

void QHaccPrefsWidget::accept(){
	const char * sps[]={ "TITLEBARNAME", "CSYMBOL", "DATESEPARATOR",
											 "TRANSTYPES" };
	QLineEdit * spps[]={ title, symbol, sep, transTypes };
	for( int i=0; i<4; i++ ) engine->setSP( sps[i], spps[i]->text() );
	QHaccDateEdit::setDateSeparator( sep->text() );

  engine->setCP( "MAINCOLOR", mbtn->backgroundColor() );
  engine->setCP( "ALTCOLOR", abtn->backgroundColor() );

  engine->setWP( "FONT", fonter );

	engine->setBP( "DOUBLEENTRY", liner->id( liner->selected() )==1 );
	const char * sps2[]={ "HIDECATEGORIES", "HIDELEDGERS", "AUTOCOMPNOCASE",
												"SHOWRECBAL",	"GLOBALTYPEAHEAD" };
	QCheckBox * spps2[]={ cats, hideleds, igcase, showrec, gtypeahead };
	for( int i=0; i<5; i++ ) engine->setBP( sps2[i], spps2[i]->isChecked() );


	int dfi=dater->id( dater->selected() );
	engine->setIP( "DATEFORMAT", dfi );
	QHaccDateEdit::setDateFormat( ( QHaccDateEdit::DateFormat )dfi );
	engine->setIP( "AUTOCOMPLETION", autocomps->currentItem() );

}

void QHaccPrefsWidget::mColor(){
  QColor c=QColorDialog::getColor( mbtn->backgroundColor() );
  if ( c.isValid() ) mbtn->setPalette( c );
}
void QHaccPrefsWidget::aColor(){
  QColor c=QColorDialog::getColor( abtn->backgroundColor() );
  if ( c.isValid() ) abtn->setPalette( c );
}
void QHaccPrefsWidget::cFont(){
  bool ok;
  QFont f=QFontDialog::getFont( &ok, font(), 0 );
  if ( ok ){
    setFont( f );
    fonter=f;
  }
}

/* * * * * * * * * * * * * * * * */
/* * * * * LEDGER DIALOG * * * * */
/* * * * * * * * * * * * * * * * */

LedgerDlg::LedgerDlg( QHacc * eng, const Ledger& l, QWidget * p,
											const char * n ) : QDialog( p, n, false ){
	engine=eng;
	if( l.isNull() ){
		edit=false;
		ledger=engine->getBlankL();
		setCaption( tr( "Create Ledger" ) );
	}
	else{
		edit=true;
		ledger=l;
		setCaption( tr( "Edit Ledger" ) );
	}
	
	QGridLayout * layout=new QGridLayout( this, 5, 6 );

	lname=new QLineEdit( ledger.gets( QHacc::LNAME ), this );
	QLabel * lbl=new QLabel( tr( "Name" ), this );
	layout->addMultiCellWidget( lbl, 0, 1, 0, 2 );
	layout->addMultiCellWidget( lname, 0, 1, 3, 6 );
	

	ldesc=new QLineEdit( ledger.gets( QHacc::LDESC ), this );
	lbl=new QLabel( tr( "Description" ), this );
	layout->addMultiCellWidget( lbl, 2, 3, 0, 2 );
	layout->addMultiCellWidget( ldesc, 2, 3, 3, 6 );

	QButton * but=new QPushButton( tr( "Accept" ), this );
	layout->addMultiCellWidget( but, 4, 5, 0, 2 );
	connect( but, SIGNAL( clicked() ), SLOT( accept() ) );

	but=new QPushButton( tr( "Dismiss" ), this );
	layout->addMultiCellWidget( but, 4, 5, 3, 6 );
	connect( but, SIGNAL( clicked() ), SLOT( close() ) );
}

LedgerDlg::~LedgerDlg(){}

void LedgerDlg::qremove( QHacc * eng, const Ledger& l, QWidget * p ){
	if( !l.isNull() ){
		if ( QMessageBox::warning( p, tr( "Delete " )+l.gets( QHacc::LNAME ),
															 tr( "Are you sure?" ), tr( "Delete Ledger" ),
															 tr( "Reconsider" ) ) == 0 ) eng->removeL( l );
	}
}

void LedgerDlg::accept(){
	Ledger newy( ledger );
	newy.set( QHacc::LNAME, lname->text() );
	newy.set( QHacc::LDESC, ldesc->text() );
	
	if( edit ) engine->updateL( ledger, newy ); 
	else engine->addL( newy );

	QDialog::accept();
}

