/************************** * * * * * * * * * * * * ***************************
    Copyright (c) 1999-2003 Ryan Bobko
                       ryan@ostrich-emulators.com

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.     
*************************** * * * * * * * * * * * * **************************/

#ifndef _QHACC_DIALOGS_H
#define _QHACC_DIALOGS_H

#include "qhacc.h"

#include <qcolor.h>
#include <qdialog.h>
#include <qdatetime.h>
#include <qtabdialog.h>

class QLabel;
class QFrame;
class QString;
class QComboBox;
class QCheckBox;
class QLineEdit;
class QCheckBox;
class QGridLayout;
class QPushButton;
class QButtonGroup;
class QRadioButton;
class QHaccLineEdit;
class QHaccDateEdit;
class QMultiLineEdit;
class QHaccChoiceEdit;


/********************************************/
/********* AccountEdit1 definitions *********/
/********************************************/

class AccountTab1 : public QWidget {
  Q_OBJECT
    public:
  AccountTab1( QHacc * engine, const Account&, QWidget * =0,
							 const char * =0 );
  ~AccountTab1();

	void modify( Account& ) const;

 protected:
	QLineEdit * bal, * name, * number, * transtypes, * budget;
	QHaccChoiceEdit * parent, * dttype;
	QButtonGroup * typeBox;
	QCheckBox * taxed, * category;
	QHacc * engine;

	public slots:
		void makeTTChoices();
};

/********************************************/
/********* AccountEdit2 definitions *********/
/********************************************/

class AccountTab2 : public QWidget {
  Q_OBJECT
    public:
  AccountTab2( QHacc *, const Account&, QWidget * =0, const char * =0 );
  ~AccountTab2();
	
	void modify( Account& ) const;

 protected:
	QFrame ** instinfo; // text widgets only
};

/********************************************************/
/*********  Account Create/Edit/Removal Dialog  *********/
/********************************************************/
class AccountDlg : public QTabDialog {
	Q_OBJECT
		public:
	AccountDlg( QHacc *, const Account& , QWidget * =0,
							const char * =0, bool =false );
	~AccountDlg();

	static void qremove( QHacc *, const Account&, QWidget * =0 );
	static const char * INCRSTR;

	Account getAccount() const;
 protected:
	bool edit; 
	QHacc * engine;
	Account account;
	AccountTab1 * tab1;
	AccountTab2 * tab2;

	void close();
	void accept();
};

/********************************************/
/*********** PrefsDlg definitions ***********/
/********************************************/
class QHaccPrefsWidget : public QWidget {
  Q_OBJECT
    public:
  QHaccPrefsWidget ( QHacc * engine, QWidget * =0, const char * =0 );
  ~QHaccPrefsWidget();
	void accept();
	
  protected slots:
		void mColor();
  void aColor();
  void cFont();
	
 private:
	QHacc * engine;
  QPushButton * mbtn, * abtn, * fbtn;
  QLineEdit * symbol, * sep, * transTypes, * title;
  QButtonGroup * liner, * dater;
  QComboBox * autocomps;
  QCheckBox * cats, * igcase, * showrec, * gtypeahead, * hideleds;
  QFont fonter;
};

class QHaccGraphPrefsWidget : public QWidget {
	Q_OBJECT
		public:
	QHaccGraphPrefsWidget( QHacc * engine, QWidget * =0, const char * =0 );
	~QHaccGraphPrefsWidget();
	void accept();
	
 private:
	QHacc * engine;
	QButtonGroup * grapher, * reporter;
	QLineEdit * grtime, * mergepct;
	QCheckBox * heads, * lines, * totals, * deltas, * piepayees;
};

class QHaccOtherPrefsWidget : public QWidget {
  Q_OBJECT
    public:
  QHaccOtherPrefsWidget ( QHacc * engine, QWidget * =0, const char * =0 );
  ~QHaccOtherPrefsWidget();
	void accept();
	
 private:
	QHacc * engine;
  QButtonGroup * defAcct;
	QCheckBox * simpleFD;

};

class PrefsDlg : public QTabDialog {
	Q_OBJECT
		public:
	PrefsDlg( QHacc * engine, int, QWidget * =0, const char * =0 );
	~PrefsDlg();
	
	protected slots:
		void accept();
	
 private:
	QHacc * engine;
	QHaccPrefsWidget * tab1;
	QHaccGraphPrefsWidget * tab2;
	QHaccOtherPrefsWidget * tab3;
};


/*********************************************/
/*********** LedgerDlg definitions ***********/
/*********************************************/
class LedgerDlg : public QDialog {
	Q_OBJECT
		public:
	LedgerDlg( QHacc *, const Ledger& , QWidget * =0, const char * =0 );
	~LedgerDlg();

	static void qremove( QHacc *, const Ledger&, QWidget * =0 );

 protected:
	bool edit; 
	Ledger ledger;
	QHacc * engine;
	QLineEdit * lname, * ldesc;

	private slots:
		void accept();
};

#endif





