/************************** * * * * * * * * * * * * ***************************
    Copyright (c) 1999-2003 Ryan Bobko
                       ryan@ostrich-emulators.com

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.     
************************** * * * * * * * * * * * * ***************************/

#ifndef _QHACCGRAPHER_H
#define _QHACCGRAPHER_H

#include <qimage.h>
#include <qpixmap.h>
#include <qdatetime.h>

class QHacc;
class QPainter;
class TableRow;
class QHaccTable;
class QHaccTableIndex;

class QHaccGraphPainter {
 public:
	virtual ~QHaccGraphPainter();
	
	static void getPainter( QHacc *, int type, QHaccGraphPainter *& );

	void paint( QPainter *, const QRect& );
	void setData( uint lid, const QHaccTable& accts, const QDate& s,
								const QDate& e );
	void describeMouse( const QPoint&, bool& inBase, bool& overZero,
											Account &, QDate& start, QDate& end ) const;
	
 protected:
	uint ledgerid;
	QHacc * engine;
	QDate start, end;
	QHaccTable * accounts;
	int baseline, zeroline;
  QColor * colors;
	QHaccTable * data;
	static const QString EMPTYSTR;

	QHaccGraphPainter( QHacc * );
	virtual void paintBase( QPainter *, const QRect& ) =0;
	virtual void paintMain( QPainter *, const QRect& ) =0;
	virtual void painted( const QPixmap& );
	virtual void mouseAccount( const QPoint&, Account& ) const =0;
	virtual void mouseDates( const QPoint&, QDate& s, QDate& e ) const;
	virtual void isetData() =0;
};


class QHaccPiePainter : public QHaccGraphPainter {
 public:
	QHaccPiePainter( QHacc * );
	~QHaccPiePainter();

 protected:
	void paintBase( QPainter *, const QRect& );
	void paintMain( QPainter *, const QRect& );
	void painted( const QPixmap& );
	void mouseAccount( const QPoint&, Account& ) const;
	void isetData();

 private:
	static const uint TITLE, TOTAL, MERGED, MERGEROWS;
	
	QHaccTableIndex * summer;
	QImage image;
};

class QHaccMonthlyPainter : public QHaccGraphPainter {
 public:
	static const uint AID, NUMBER, INCREASES, DECREASES;

	QHaccMonthlyPainter( QHacc *, int type );
	~QHaccMonthlyPainter();
	
 protected:
	void paintBase( QPainter *, const QRect& );
	void paintMain( QPainter *, const QRect& );
	void mouseAccount( const QPoint&, Account& ) const;
	void mouseDates( const QPoint&, QDate& s, QDate& e ) const;
	void isetData();
	void drawSum( QPainter *, float, int, int, int, int ) const;
	void drawTitle( QPainter *, const QString&, int wstart, int wspot,
									int hspot ) const;

	virtual void paintRow( QPainter *, const TableRow& last, const TableRow&,
												 int wstart, int wspot, const QColor&, bool,
												 const QString& ) const =0;

	bool line, ivse;
	QString * monthstrs;
	uint months, wpm, wpa; //widthpermonth, widthperaccount
	float hiVal, loVal, factor;
};


class QHaccSingleValPainter : public QHaccMonthlyPainter {
 public:
	QHaccSingleValPainter( QHacc *, int type );
	~QHaccSingleValPainter();
	
 protected:
	void paintRow( QPainter *, const TableRow&, const TableRow&, int wstart,
								 int wspot, const QColor&, bool, const QString& ) const;
};

class QHaccIvsEPainter : public QHaccMonthlyPainter {
 public:
	QHaccIvsEPainter( QHacc *, int type );
	~QHaccIvsEPainter();
	
 protected:
	void paintRow( QPainter *, const TableRow&, const TableRow&, int wstart,
								 int wspot, const QColor&, bool, const QString& ) const;
	void paintRow( QPainter *, const TableRow&, const TableRow&, int wstart,
								 int wspot, const QColor&, bool,
								 const QString&, uint column ) const;
};

#endif
