/************************* * * * * * * * * * * * * ***************************
    Copyright (c) 1999-2003 Ryan Bobko
                       ryan@ostrich-emulators.com

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.     
************************* * * * * * * * * * * * * ***************************/

#include "qhaccutils.h"
#include "qhaccgrwin.h"
#include "qhaccdialogs.h"
#include "qhacclineedits.h"
#include "qhaccsubsetwin.h"
#include "qhaccreportwriter.h"
#include "qhaccgraphpainters.h"
#include "qhaccgrwin.moc"

#include <time.h>
#include <stdlib.h>

#include <qlabel.h>
#include <qpixmap.h>
#include <qlayout.h>
#include <qprinter.h>
#include <qpainter.h>
#include <qcombobox.h>
#include <qtextedit.h>
#include <qpushbutton.h>
#include <qpaintdevicemetrics.h>

/********************/
/**  GRAPHCONTROL  **/
/********************/

QHaccDocControl::QHaccDocControl( QHacc * e, uint lid, QWidget * w,
																	const char * n ) : QFrame( w, n ){
	
	engine=e;

  QDate de=QDate::currentDate();
  de=de.addDays( 1+de.daysInMonth()-de.day() ); //...through the current month

  start=new QHaccDateEdit( de, this );
  end=new QHaccDateEdit( de, this );
  start->setAlignment( AlignCenter );
  end->setAlignment( AlignCenter );

  QGridLayout * layout=new QGridLayout( this, 10, 1 );
  QLabel * lbl=new QLabel( tr( "start date" ), this );
	int row=0;
  layout->addMultiCellWidget( lbl, row, row, 0, 0, AlignCenter );
	row++;
  layout->addMultiCellWidget( start, row, row, 0, 0 );
	row++;

  lbl=new QLabel( tr( "end date" ), this );
  layout->addMultiCellWidget( lbl, row, row, 0, 0, AlignCenter );
	row++;
  layout->addMultiCellWidget( end, row, row, 0, 0 );
	row++;
	
  lbl=new QLabel( tr( "type" ), this );
  typer=new QComboBox( false, this );	
  layout->addMultiCellWidget( lbl, row, row, 0, 0, AlignCenter );
	row++;
  layout->addMultiCellWidget( typer, row, row, 0, 0 );
	row++;

  llbl=new QLabel( tr( "ledger" ), this );
  ledgerer=new QComboBox( false, this );	
	QHaccTable * led=0;
	engine->getLs( led );
	ledgerer->insertItem( tr( "All Ledgers" ) );
	for( uint i=0; i<led->rows(); i++ ){
		Ledger l=led->at( i );
		ledgerer->insertItem( l.gets( QHacc::LNAME ), l.getu( QHacc::LID ) );
	}
	ledgerer->setCurrentItem( lid );

  layout->addMultiCellWidget( llbl, row, row, 0, 0, AlignCenter );
	row++;
  layout->addMultiCellWidget( ledgerer, row, row, 0, 0 );
	row++;

  lbl=new QLabel( tr( "accounts" ), this );
	pick=new QListBox( this );
	pick->setSelectionMode( QListBox::Extended );
  layout->addMultiCellWidget( lbl, row, row, 0, 0, AlignCenter );
	row++;
  layout->addMultiCellWidget( pick, row, row, 0, 0 );
	row++;

	QPushButton * prefsb=new QPushButton( tr( "Set Preferences" ), this );
	layout->addMultiCellWidget( prefsb, row, row, 0, 0, 0 );
	row++;

	QPushButton * printb=new QPushButton( tr( "Print" ), this );
	layout->addMultiCellWidget( printb, row, row, 0, 0, 0 );

	connect( prefsb, SIGNAL( clicked() ), SLOT( getPrefs() ) );
	connect( printb, SIGNAL( clicked() ), SLOT( print() ) );
}

QHaccDocControl::~QHaccDocControl(){}
void QHaccDocControl::readPrefs( bool initial ){
	//cout<<"qdc readprefs()"<<endl;
	if( initial ){
		connect( engine, SIGNAL( changedP( const QString&, bool ) ),
						 this, SLOT( changeP( const QString&, bool ) ) );

		QDate de=end->getDate();
		start->setDate( de.addMonths( 0-engine->getIP( "GRAPHTIME" ) ) );

		const QString str="HIDELEDGERS";
		changeP( str, engine->getBP( str ) );

		connect( start, SIGNAL( dateChanged( QDate ) ), SLOT( newValues() ) );
		connect( end, SIGNAL( dateChanged( QDate ) ), SLOT( newValues() ) );
		connect( pick, SIGNAL( selectionChanged() ), SLOT( newValues() ) );
		connect( ledgerer, SIGNAL( activated( int ) ), SLOT( newValues() ) );
		connect( typer, SIGNAL( activated( int ) ), SLOT( newType( int ) ) );
	}
}

void QHaccDocControl::setPickMode( QListBox::SelectionMode m ){
	pick->setSelectionMode( m );
}
void QHaccDocControl::getPrefs(){
	// pop the prefs dialog, opened to graphing tab
  ( new PrefsDlg( engine, 1, this ) )->show();
}

void QHaccDocControl::newType( int t ) { emit typeChanged( t ); }
void QHaccDocControl::newValues(){
	QStringList alist;
	for( uint i=0; i<pick->count(); i++ ){
		if( pick->isSelected( i ) )	alist.append( pick->text( i ) );
	}
  emit dataChanged( ledgerer->currentItem(), alist,
										start->getDate(), end->getDate() );
}

void QHaccDocControl::changeP( const QString& str, bool b ){
	if( str=="GRAPHDELTAS" ) newValues();
	else if( str=="HIDELEDGERS" ){
		if( b ){
			ledgerer->hide();
			llbl->hide();
		}
		else{
			ledgerer->show();
			llbl->show();
		}
		newValues();
	}
}

void QHaccDocControl::changeP( const QString& str, QString  ){
	if( str=="DATEFORMAT" || str=="DATESEPARATOR" ) {
		start->refresh();
		end->refresh();
	}
}
void QHaccDocControl::setPicks( QStringList s, int i ){
	pick->clear();
  pick->insertStringList( s );
  pick->setSelected( i, true );
	pick->setCurrentItem( i );
}

void QHaccDocControl::setTypes( QStringList s, int i ){
  typer->clear();
  typer->insertStringList( s );
  typer->setCurrentItem( i );
}

void QHaccDocControl::print(){ emit printCalled(); }

/* * * * * * * * * * * * * * * * * * * */
/* * Base for Reporters and Graphers * */
/* * * * * * * * * * * * * * * * * * * */

QHaccDocDisplay::QHaccDocDisplay( QHacc * e, QWidget * w,
																	const char * n ) : QFrame( w, n ) {
	engine=e;
	engine->makeAT( accounts, QHacc::AID );
	ledgerid=0;
}

QHaccDocDisplay::~QHaccDocDisplay() {}
void QHaccDocDisplay::setData( uint lid, const QHaccTable& accts,
															 const QDate& s, const QDate& e ){
	accounts->clear();
	accounts->load( accts );
	ledgerid=lid;
	start=s;
	end=e;
	refresh();
}
	
void QHaccDocDisplay::readPrefs( bool initial ){
	if( initial ){
		connect( engine, SIGNAL( changedP( const QString&, bool ) ),
						 this, SLOT( refresh() ) );
		connect( engine, SIGNAL( updatedA( const Account&, const Account& ) ),
						 this, SLOT( updA( const Account&, const Account& ) ) );
	}
}

void QHaccDocDisplay::isetType(){}
void QHaccDocDisplay::setType( int t ){
	type=t;
	isetType();
	refresh();
}

void QHaccDocDisplay::irefresh(){}
void QHaccDocDisplay::refresh(){ irefresh(); }
void QHaccDocDisplay::refreshA( const Account& a ){
	uint idx=0;
	if( accounts->contains( a, idx ) ) refresh();
}

void QHaccDocDisplay::updA( const Account&, const Account& newy ){
	refreshA( newy );
}

void QHaccDocDisplay::print( QPrinter * p ) const { iprint( p ); }
void QHaccDocDisplay::iprint( QPrinter * ) const {}

/* * * * * * * * * * * */
/* Base for Reporters  */
/* * * * * * * * * * * */

QHaccReporter::QHaccReporter( QHacc * e, QWidget * w,	const char * n )
	: QHaccDocDisplay( e, w, n ){

	viewer=new QTextEdit( this );
	viewer->setFont( QFont( "Courier" ) );
	viewer->setReadOnly( true );

	QBoxLayout * layout=new QBoxLayout( this, QBoxLayout::TopToBottom );
	layout->add( viewer );
	viewer->setTextFormat( PlainText );
}

QHaccReporter::~QHaccReporter(){ delete accounts; }

void QHaccReporter::irefresh(){
	if( type==QHacc::JOURNAL )
		viewer->setText( QHaccReportWriter::journal( ledgerid, engine,
																								 start, end ) );
	else
		viewer->setText( QHaccReportWriter::report( type, engine, ledgerid,
																								accounts,	start, end ) );
}

void QHaccReporter::iprint( QPrinter * printer ) const {
	QPainter p;
	if( !p.begin( printer ) )	return;

	QPaintDeviceMetrics metrics( printer );
	QFontMetrics fm=p.fontMetrics();

	p.setFont( viewer->font() );
	int page=1, yPos=0;
	const int MARG=60, FMLS=fm.lineSpacing(), LIMIT=metrics.height()-MARG-FMLS;
	const int MW=metrics.width(), MH=metrics.height();

	QString today=QHaccDateEdit::getDateString( QDate::currentDate() );
	const int LINES=viewer->lines();
	const bool FORJ=( type==QHacc::JOURNAL );

	for( int i=0; i<LINES; i++ ) {
		// try not to break a splitgroup across pages
		int more=0;
		if( FORJ ){
			bool keepGoing=true;
			while( keepGoing && i+more<LINES ){
				if( viewer->text( i+more )==" " ) keepGoing=false;
				else more++;
			}
		}
		else more=1;
		
		//cout<<"new paragraph is "<<endl;
		//for( int j=0; j<more; j++ ) cout<<j<<" "<<viewer->text( i+j )<<endl;
		//cout<<"end"<<endl;
		
		// FIXME: this will bomb if a splitgroup won't fit on a single page
		if( MARG+yPos+( more-1 )*FMLS>LIMIT ) {
			yPos=metrics.height();
			p.drawText( MARG, MH-MARG, MW, FMLS, ExpandTabs | DontClip, 
									tr( "page" )+" "+QString::number( page++ )+" - "+today );
			
			printer->newPage();             // no more room on this page
			yPos=0;                         // back to top of page
		}
		for( int j=0; j<more; j++ ){
			p.drawText( MARG, MARG+yPos, MW, FMLS, ExpandTabs | DontClip,
									viewer->text( i ) );
			yPos+=FMLS;
			if( FORJ ) i++;
		}
		if( FORJ ){
			// journals should print a blank line after every splitgroup
			p.drawText( MARG, MARG+yPos, MW, FMLS, ExpandTabs | DontClip, "" );
			yPos+=FMLS;
		}
	}
	if( MARG+yPos<LIMIT ){
		// we've stopped printing somewhere in the middle of the page, 
		// so print a footer at the bottom
		yPos=metrics.height();
		p.drawText( MARG, MH-MARG, MW, FMLS, ExpandTabs | DontClip, 
								tr( "page" )+" "+QString::number( page++ )+" - "+today );
	}
	p.end();
}

/* * * * * * * * * * */
/* Base for Graphers */
/* * * * * * * * * * */

QHaccGrapher::QHaccGrapher( QHacc * e, QWidget * w, const char * n )
	: QHaccDocDisplay( e, w, n ){
	
	QHaccGraphPainter::getPainter( e, e->getIP( "GRAPHTYPE" ), painter );
}

QHaccGrapher::~QHaccGrapher(){ delete painter; }
void QHaccGrapher::irefresh(){
	painter->setData( ledgerid, *accounts, start, end );
	repaint( false );
}

void QHaccGrapher::drawContents( QPainter * p ){ painter->paint( p, rect() ); }
void QHaccGrapher::isetType(){
	QHaccGraphPainter * np;
	QHaccGraphPainter::getPainter( engine, type, np );
	delete painter;
	painter=np;
}

void QHaccGrapher::mouseReleaseEvent( QMouseEvent * qme ){
	bool baser=false;
	bool abzero=false;
	Account acct;
	QDate start, end;
	painter->describeMouse( qme->pos(), baser, abzero, acct, start, end );
	
	if( !acct.isNull() ){
		QHaccSubsetWin  * win=new QHaccSubsetWin( engine );
		win->show();
		win->setAll( ledgerid, acct, start, end );
	}


	/*
	cout<<"inBase="<<baser<<"\tabzero="<<abzero<<"\t";
	if( acct ){
		cout<<"acct="<<acct->toString();
		delete acct;
	}
	else cout<<"no account";
	cout<<"\t"<<TableCol( start ).toString()<<"-"<<TableCol( end ).toString()<<endl;
	*/
}

void QHaccGrapher::iprint( QPrinter * printer ) const {
	QPainter p;
	if( !p.begin( printer ) ) return;

	const int MARG=60, MARG2=120;
	QPaintDeviceMetrics metrics( printer );

	//const QSize sizer( metrics.height(), metrics.width() );
	painter->paint( &p, QRect( MARG, MARG,
														 metrics.width()-MARG2,
														 metrics.height()-MARG2 ) );

	// label the graph
	QFontMetrics fm=p.fontMetrics();
	QString text;
	if( accounts->rows()==1 ) text=accounts->at( 0 ).gets( QHacc::ANAME )+" ";

	text.append( QHaccDateEdit::getDateString( start )+" - "+
							 QHaccDateEdit::getDateString( end ) );

	p.drawText( MARG2, MARG+fm.height(), text );

	p.end();
}

/*********************/
/**  QHACCGRDIALOG  **/
/*********************/

QHaccGRDialog::QHaccGRDialog( QHacc * e, DisplayType type, const Account& a,
															const Ledger& l, QWidget * w, const char * n )
	: QDialog( w, n, false, WDestructiveClose ){

	engine=e;
	printer=0;
		
  QPushButton * dismiss=new QPushButton( "Dismiss", this );
  control=new QHaccDocControl( engine, l.getu( QHacc::LID ), this );
	QString pref;
	QStringList types;

	if ( type==REPORT ){
		display=new QHaccReporter( engine, this );
		setCaption( tr( "QHacc Report" ) );
		pref="REPORTTYPE";

		types+=tr( "Totals by Payee" );
		types+=tr( "Monthly Balances" );
		types+=tr( "Monthly Deltas" );
		types+=tr( "Transaction Averages" );
		types+=tr( "Journal" );
		types+=tr( "Budget" );
		
	}
	else{
		display=new QHaccGrapher( engine, this );
		setCaption( tr( "QHacc Graph" ) );
		pref="GRAPHTYPE";

		types+=tr( "Line" );
		types+=tr( "Bar" );
		types+=tr( "Two Line" );
		types+=tr( "Two Bar" );
		types+=tr( "Pie" );

	}

	QStringList str; // this is our list of things we can display
	uint naccts=0, idx=0;
	QHaccTable * table=0;
	engine->getAs( false, table );
	naccts=table->rows();
	
	// first, cycle through and make sure we're
	// going to use the full name of the account.
	// we need to update the table, so the index
	// we're about to make will sort correctly
	for( uint i=0; i<naccts; i++ ){
		Account acct=table->at( i );
		table->updateWhere( QHacc::AID, acct.getu( QHacc::AID ),
												QHacc::ANAME,
												TableCol( engine->getFNameOfA( acct ) ) );
	}
	
	// now, cycle through again add all the accounts to the stringlist
	// for display. Also, figure out the index of the account we want
	// to display first, so it can be highlighted in the picklist
	QHaccTableIndex * nidx=0;
	table->addIndexOn( QHacc::ANAME, nidx );
	for( uint i=0; i<naccts; i++ ){
		Account acct=nidx->at( i );
		str.append( acct.gets( QHacc::ANAME ) );
		if( acct.compareTo( a, QHacc::AID )==0 ) idx=i;
	}
	delete table;

	const int PICK=engine->getIP( pref );
	control->setTypes( types, PICK );
  control->setPicks( str, idx );
	if ( type==REPORT ) display->setType( PICK );

  QGridLayout * layout=new QGridLayout( this, 10, 10, 2, 2 );
	layout->addMultiCellWidget( control, 0, 9, 0, 1 );
	layout->addMultiCellWidget( display, 0, 8, 2, 9 );
	layout->addMultiCellWidget( dismiss, 9, 9, 2, 9 );

  connect( dismiss, SIGNAL( clicked() ), SLOT( close() ) );
  connect( control, SIGNAL( dataChanged( uint, QStringList, QDate, QDate ) ),
					 SLOT( changeData( uint, QStringList, QDate, QDate ) ) );
	connect( control, SIGNAL( typeChanged( int ) ), SLOT( changeType( int ) ) );
	connect( control, SIGNAL( printCalled() ), SLOT( print() ) );

  resize( 550, 450 );
  control->newValues(); // force a refresh
}


QHaccGRDialog::~QHaccGRDialog(){ if( printer ) delete printer; }

void QHaccGRDialog::print(){
	if( !printer ){
		printer=new QPrinter();
		printer->setPageSize( QPrinter::Letter );
	}

	if( printer->setup( this ) ) display->print( printer );
}

void QHaccGRDialog::changeType( int t ){ display->setType( t ); }
void QHaccGRDialog::changeData( uint lid, QStringList picks,
																QDate start, QDate end ){
	QHaccTable data( QHacc::ACOLTYPES, QHacc::ACOLNAMES, QHacc::ACOLS );

	for ( QStringList::Iterator it=picks.begin(); it!=picks.end(); ++it )
		data.add( engine->getA( *it ) );

	display->setData( lid, data, start, end );
}

void QHaccGRDialog::save(){
	QString str;
	QSize s=size();
	engine->setSP( "GRDWINSIZE", str.setNum( s.width() )+
								 " "+str.setNum( s.height() ) );
}

void QHaccGRDialog::readPrefs( bool initial ){
	QString str=engine->getSP( "GRDWINSIZE" );
	if( !str.isEmpty() ){
		QString vals[2];
		Utils::parser( str, " ", 0, vals, 2 );
		resize( vals[0].toInt(), vals[1].toInt() );
	}
	
	control->readPrefs( initial );
	display->readPrefs( initial );
	
}

void QHaccGRDialog::done( int r ){
	save();
	QDialog::done( r );
}
