/************************* * * * * * * * * * * * * ***************************
    Copyright (c) 1999-2003 Ryan Bobko
                       ryan@ostrich-emulators.com

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.     
************************** * * * * * * * * * * * * **************************/

#include "qhacc.h"
#include "qhaccledchsr.h"
#include "qhaccledchsr.moc"

QHaccLedgerChooser::QHaccLedgerChooser( QHacc * eng, QWidget * p,
																				const char * c ) : QTabBar( p, c ){
	ids=new int[0];
	engine=eng;

  QHaccTable * ledgs=0;
	engine->getLs( ledgs );
	for( uint i=0; i<ledgs->rows(); i++ ) addL( ledgs->at( i ) );
}

QHaccLedgerChooser::~QHaccLedgerChooser(){}

void QHaccLedgerChooser::save() const {
	engine->setIP( "LEDGERINDEX", currentTab() );
}

void QHaccLedgerChooser::readPrefs( bool initial ) {
	if( initial ){
		connect( engine, SIGNAL( addedL( const Ledger& ) ),
						 SLOT( addL( const Ledger& ) ) );
		connect( engine, SIGNAL( removedL( const Ledger& ) ),
						 SLOT( remL( const Ledger& ) ) );
		connect( engine, SIGNAL( updatedL( const Ledger&, const Ledger& ) ),
						 this, SLOT( updL( const Ledger&, const Ledger& ) ) );
		connect( this, SIGNAL( selected( int ) ), SLOT( changeLedger( int ) ) );
	}

	int li=engine->getIP( "LEDGERINDEX" );
	if( li>=count() ) li=0; // sanity check of (possibly) user-edited files
	setCurrentTab( ids[li] );
	changeLedger( ids[li] );

}

void QHaccLedgerChooser::changeLedger( int i ){
	QTab * t=tab( i );
	emit changedLedger( engine->getL( t->text() ) );
	//cout<<"changed ledger to "<<engine->getL( t->text() ).gets( QHacc::LNAME )<<endl;
}

void QHaccLedgerChooser::addL( const Ledger& l ){
	int lim=count();
	int * newids=new int[lim+1];
	for( int i=0; i<lim; i++ ) newids[i]=ids[i];
	delete [] ids;
	ids=newids;

	QTab * t=new QTab( "" );
	ids[lim]=addTab( t );
	// tabbar doesn't refresh automatically (?), so force a refresh
	t->setText( l.gets( QHacc::LNAME ) );
	
	if( engine->getBP( "HIDELEDGERS" ) ) show();
}

void QHaccLedgerChooser::remL( const Ledger& l ){
	QTab * t=getLTab( l ); // tab to remove
	int id=t->identifier();
	int lim=count(), cur=0;
	int * newids=new int[lim-1];
	removeTab( t );

	for( int i=0; i<lim; i++ )	if( ids[i]!=id ) newids[cur++]=ids[i];
	delete [] ids;
	ids=newids;
	setCurrentTab( ids[0] );
	changeLedger( ids[0] );
}

void QHaccLedgerChooser::updL( const Ledger& older, const Ledger& newer ){
	if( older.gets( QHacc::LNAME )!=newer.gets( QHacc::LNAME ) ){
		QTab * t=getLTab( older );
		t->setText( newer.gets( QHacc::LNAME ) );
	}
}

QTab * QHaccLedgerChooser::getLTab( const Ledger& l ) const {
	int lim=count();
	for( int i=0; i<lim; i++ ){
		QTab * t=tab( ids[i] );
		if( l.gets( QHacc::LNAME )==t->text() ) return t;
	}
	return 0;
}

Ledger QHaccLedgerChooser::getSelected() const {
	return engine->getL( tab( currentTab() )->text() );
}
