/************************** * * * * * * * * * * * * ***************************
    Copyright (c) 1999-2003 Ryan Bobko
                       ryan@ostrich-emulators.com

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.     
************************** * * * * * * * * * * * * ***************************/

#include "qhaccutils.h"
#include "qhacclineedits.h"
#include "qhacclineedits.moc"

#include <qevent.h>
#include <qregexp.h>


/* * * * * * * * * * * * * */
/* * * *  LINEEDIT * * * * */
/* * * * * * * * * * * * * */

//needed to make this because I need a focusout event
QHaccLineEdit::QHaccLineEdit( QWidget * p, const char * n ) : QLineEdit(p,n){}
QHaccLineEdit::QHaccLineEdit( const QString& c, QWidget * p, const char * n )
  : QLineEdit( c, p, n ){}
QHaccLineEdit::~QHaccLineEdit(){}

void QHaccLineEdit::focusInEvent( QFocusEvent * ){
  selectAll();
  emit gainedFocus();
}
void QHaccLineEdit::focusOutEvent( QFocusEvent * ){
  deselect();
  emit lostFocus();
}

void QHaccLineEdit::setText( const QString& s ){
	QLineEdit::setText( ( s.isEmpty() ? empty : s ) );
	selectAll();
}

void QHaccLineEdit::setEmptyText( const QString& s ){
	empty=QString( s );
	QLineEdit::setText( s );
}

QString QHaccLineEdit::text() const {
	QString ret=QLineEdit::text();
	if( ret==empty ) return QString();
	else return ret;
}

/* * * * * * * * * * * * * */
/* * * * CHOICEEDIT  * * * */
/* * * * * * * * * * * * * */

//similar to a dropbox, but without the drop part
QHaccChoiceEdit::QHaccChoiceEdit( bool allow, QWidget * w, const char * n )
  : QHaccLineEdit( w, n ){
  allowInserts=allow;
  index=0;
	nocase=false;
}

QHaccChoiceEdit::~QHaccChoiceEdit(){}
void QHaccChoiceEdit::ignoreCase( bool b ){ nocase=b; }
void QHaccChoiceEdit::insertItem( const QString& newOne ){
  //if ( strings.isEmpty() ) setText( newOne );
  //else if ( !strings.contains( newOne ) ) strings.inSort( newOne );
  if ( !strings.contains( newOne ) ) strings.inSort( newOne );
}

void QHaccChoiceEdit::insertStringList( QStringList newOnes ){
	for( QStringList::iterator it=newOnes.begin(); it!=newOnes.end(); ++it ){
		if( !strings.contains( *it ) ) strings.inSort( *it );
	}
}

void QHaccChoiceEdit::remove( QString s ){
  strings.remove( s );
}

void QHaccChoiceEdit::setText( const QString& stringy ){
	//cout<<"qhaccchoiceedit settext: "<<stringy<<endl;
  QHaccLineEdit::setText( stringy );

	// setting text DOES NOT add the text to the strings list
  // if ( !strings.contains( stringy ) ) strings.inSort( stringy );
	
  for ( unsigned int i=0; i<strings.count(); i++ ){
    if ( strings.at( i )==stringy ){
      index=i;
      break;
    }
  }
	setCursorPosition( 0 );
	
}

void QHaccChoiceEdit::clear(){ strings.clear(); }
void QHaccChoiceEdit::keyPressEvent( QKeyEvent * qke ){
	//cout<<"qce got kpe"<<endl;
  /* CAN YOU SAY HACK? */
	int selstart, sellen; // start and length of selection

	if( hasSelectedText() ){
		int end=0;
		getSelection( &selstart, &end );
		sellen=end-selstart;
	}
	else {
		selstart=text().length();
		sellen=0;
	}

	int key=qke->key();
	if( key==Key_Escape || key==Key_Shift ){
		QHaccLineEdit::keyPressEvent( qke );
		return;
	}
	
	qke->accept();

	switch( key ){
  case Key_Down:
    if ( !strings.isEmpty() ){
      index=++index%strings.count();
      setText( strings.at( index ) );
			setEdited( false );
			selectAll();
		}
		emit arrowPressed( false );
    break;
  case Key_Up:
    if ( !strings.isEmpty() ){
      index=(--index+strings.count() )%strings.count();
      setText( strings.at( index ) );
			setEdited( false );
      selectAll();
		}
		emit arrowPressed( true );
    break;
  case Key_Return:
  case Key_Left:
    QHaccLineEdit::keyPressEvent( qke );
		selstart--;
		setSelection( selstart, sellen+1 );
		break;
  case Key_Right:
    QHaccLineEdit::keyPressEvent( qke );
		selstart++;
		setSelection( selstart, sellen-1 );
    break;
  case Key_Backspace:
  case Key_Delete:
    if( allowInserts ) QHaccLineEdit::keyPressEvent( qke );
		else QHaccLineEdit::setText( "" );
    break;
  default:
		//cout<<"key press: ->"<<qke->text()<<"<-"<<endl;
		QString iniUnselText=( selstart ? text().left( selstart ) : QString() );
    QString txt=iniUnselText+qke->text();
		QString nctxt=txt.upper();
		QString filltxt;
		
		index=0;
		int txtlen=txt.length();
		int lim=strings.count();

		/*
		cout<<"iniUnselText is "<<iniUnselText<<"; txt is "<<txt<<"; txtlen is "<<txtlen<<endl;
		cout<<lim<<" strings are:"<<endl;
		while( index<lim ){
			cout<<index<<": "<<strings.at( index )<<endl;
			index++;
		}
		index=0;
		*/

		// go through all our strings to see if we have a match		
		while( index<lim ){
      QString s=strings.at( index );
			int comp;
			if( nocase ) comp=s.left( txtlen ).upper().compare( nctxt );
			else comp=s.left( txtlen ).compare( txt );

			//if( nocase ) cout<<"comparing "<<s.left( txtlen ).upper()
			//                 <<" to "<<nctxt<<"="<<comp<<endl;
			//else cout<<"comparing "<<s.left( txtlen )
			//         <<" to "<<txt<<"="<<comp<<endl;


			if( comp==0 ){
				filltxt=s;
				break;
      } 
			else if( !nocase && comp>0 ) break; // too far: we'll never get a match

			index++;
    }

		//if( filltxt.isNull() && allowInserts ) filltxt=txt;
		//cout<<"\ttxt is ->"<<txt<<"<-\n\tfilltxt is ->"<<filltxt<<"<-"<<endl;
		if( filltxt.isNull() ){
			if( allowInserts ){
				QHaccLineEdit::setText( txt );
				setSelection( selstart+1, text().length()-selstart-1 );
			}
		}
		else{
			QHaccLineEdit::setText( filltxt );
			setSelection( selstart+1, text().length()-selstart-1 );
		}
		
		setEdited( true );
  } // switch
}

/* * * * * * * * * * * * * */
/* * * * * DATEEDIT  * * * */
/* * * * * * * * * * * * * */

QHaccDateEdit::DateFormat QHaccDateEdit::form=QHaccDateEdit::AMERICAN;
QString QHaccDateEdit::sep="/";
QString QHaccDateEdit::AmForm="MM/dd/yyyy";
QString QHaccDateEdit::EuForm="dd/MM/yyyy";
QString QHaccDateEdit::YfForm="yyyy/MM/dd";

// a text box that is specially designed to store dates
QHaccDateEdit::QHaccDateEdit( QWidget * p, const char * n ) 
  : QHaccLineEdit( p, n ){

  setDate( QDate::currentDate() );	
	setValidator( new QHaccDateValidator( this ) );
}

QHaccDateEdit::QHaccDateEdit( QDate d, QWidget * p,
															const char * n ) : QHaccLineEdit( p, n ){
  setDate( d );
	setValidator( new QHaccDateValidator( this ) );
}

QHaccDateEdit::~QHaccDateEdit(){}

QString QHaccDateEdit::getShortDisplayString( QDate d ){
  QString s=getDateString( d );
  return ( form==YEARFIRST ? s.right( 5 ) : s.left( 5 ) );
}

void QHaccDateEdit::refresh(){ setText( getDateString( day ) ); }
QString QHaccDateEdit::getDateString( QDate d ){
  QString format;
  if ( form==AMERICAN ) format=AmForm;
	else if ( form==EUROPEAN ) format=EuForm;
	else format=YfForm;
	return d.toString( format );
}

QDate QHaccDateEdit::getDateFromString( const QString& dstring ){
  int start=0, end=0;
  end=dstring.find( sep );
  int m=dstring.left( end ).toInt();
  start=end+1;
  end=dstring.find( sep, start );  
  int d=dstring.mid( start, end-start ).toInt();
  end=dstring.find( sep, start );
  start=end+1;
  int y=dstring.mid( start, end-start ).toInt();

  if ( form==EUROPEAN ){
    //switch day and month for european formats
    int t=m;
    m=d;
    d=t;
  }
  else if ( form==YEARFIRST ){
    //switch everything for yearfirst
    int t=y;
    y=m;
    m=d;
    d=t;
  }

  return ( QDate::isValid( y, m, d ) ?
	   QDate( y, m, d ) : QDate::currentDate() );
}

void QHaccDateEdit::setDateFormat( DateFormat d ){ form=d; }
void QHaccDateEdit::setDateSeparator( const QString& s ){
	sep=s;
	AmForm="MM"+s+"dd"+s+"yyyy";
	EuForm="dd"+s+"MM"+s+"yyyy";
	YfForm="yyyy"+s+"MM"+s+"dd";
}

void QHaccDateEdit::setDate( QDate d ){
  QDate before=day;
  if( d.isValid() ) day=d;
  else day=QDate::currentDate();
  setText( getDateString( day ) );
  if ( before!=day ) emit dateChanged( day );
}

void QHaccDateEdit::focusOutEvent( QFocusEvent * ){	
  deselect();
  setDate( getDateFromString( text() ) );
}

void QHaccDateEdit::focusInEvent( QFocusEvent * ){
	setSelection( 0, text().find( sep ) );
}

bool QHaccDateEdit::event( QEvent * qe ){
  if ( qe->type()==QEvent::KeyPress ){
		int sep1=text().find( sep );
		int sep2=text().find( sep, sep1+1 )-1;
		int pos=cursorPosition();

		int elempos=0;
		if( pos>sep2+1 ) elempos=2;
		else if ( pos>sep1 ) elempos=1;

    QKeyEvent * k=( QKeyEvent * )qe;
    if ( k->key()==Key_Tab ){

			if( elempos==0 ){ // we're on the first date element
				setCursorPosition( sep1+1 );
				setSelection( sep1+1, sep2-sep1 ); // highlight the next element
			}
			else if( elempos==1 ){ // we're on element #2
				setCursorPosition( sep2+1 );
				setSelection( sep2+2, text().length()-( sep2+2 ) ); // last element
			}
			else if( elempos==2 )	return QWidget::event( qe );
			return true;
    }
    else if( k->key()==Key_Up || k->key()==Key_Down ){
			QDate me=getDateFromString( text() );
			QDate month, date, year;

			// figure out what might get changed 
			int change=-1;
			if ( k->key()==Key_Up ) change=1;
			
			year=me.addYears( change );
			month=me.addMonths( change );
			date=me.addDays( change );

			// apply the correct change to the date
			if ( elempos==0 ){
				if( form==AMERICAN ) setDate( month );
				else if ( form==EUROPEAN ) setDate( date );
				else setDate( year );
			}
			else if ( elempos==1 ){
				if( form==AMERICAN ) setDate( date );
				else setDate( month );
			}
			else if ( elempos==2 ){
				if( form==YEARFIRST ) setDate( date );
				else setDate( year );
			}

			// calling setDate will position the cursor at the end of the
			// lineedit. we want the cursor right where the user left it
			setCursorPosition( pos );
			return true;
		}
		else return QWidget::event( qe );
  }
  else return QWidget::event( qe );
}

QDate QHaccDateEdit::getDate() const { return day; }

QHaccDateEdit::DateFormat QHaccDateEdit::getDateFormat() { return form; }
QString QHaccDateEdit::getDateSeparator() { return sep; }

/* * * * * * * * * * * * * */
/* * * * DATEVALIDATOR * * */
/* * * * * * * * * * * * * */

QHaccDateValidator::QHaccDateValidator( QWidget * p, const char * n )
	: QValidator( p, n ) {}

QHaccDateValidator::~QHaccDateValidator(){}

QValidator::State QHaccDateValidator::validate( QString& text, int& ) const {
	// only accept numbers, or date separators in the correct form
	// don't worry about valid dates, just the right form
	

	QString sep="\\"+QHaccDateEdit::getDateSeparator();
	QString year="[12][09]\\d\\d";
	QString mon="[01]?\\d";
	QString day="[0123]?\\d";
	QString pattern="^";
	
	QHaccDateEdit::DateFormat form=QHaccDateEdit::getDateFormat();
	if( form==QHaccDateEdit::AMERICAN ) pattern+=mon+sep+day+sep+year;
	else if ( form==QHaccDateEdit::EUROPEAN ) pattern+=day+sep+mon+sep+year;
	else pattern+=year+sep+mon+sep+day;
	pattern+="$";
		

	QRegExp exp( pattern );
	if( exp.search( text, 0 )>-1 ) return Acceptable;
	exp.setPattern( "^\\d*"+sep+"\\d*"+sep+"\\d*$" );
	if( exp.search( text, 0 )>-1 ) return Intermediate;
	return Invalid;
}

