/************************** * * * * * * * * * * * * ***************************
    Copyright (c) 1999-2003 Ryan Bobko
                       ryan@ostrich-emulators.com

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.     
************************** * * * * * * * * * * * * ***************************/
#ifndef _QHACCLINEEDITS_H
#define _QHACCLINEEDITS_H

#include <qlabel.h>
#include <qevent.h>
#include <qstrlist.h>
#include <qdatetime.h>
#include <qlineedit.h>
#include <qvalidator.h>

class QHaccLineEdit : public QLineEdit {
  Q_OBJECT
    public:
  QHaccLineEdit( QWidget *, const char * =0 );
  QHaccLineEdit( const QString&, QWidget *, const char * =0 );
  virtual ~QHaccLineEdit();

	void setEmptyText( const QString& );
	void setText( const QString& );
	QString text() const;

 protected:
  virtual void focusInEvent( QFocusEvent * );
  virtual void focusOutEvent( QFocusEvent * );
  
 private:
	QString empty;

 signals:
  void lostFocus();
  void gainedFocus();
};


class QHaccChoiceEdit : public QHaccLineEdit {
  Q_OBJECT
    public:
  QHaccChoiceEdit( bool, QWidget * =0, const char * =0 );
  virtual ~QHaccChoiceEdit();
  
  void setText( const QString& );
  void insertItem( const QString& );
  void insertStringList( QStringList );
	void ignoreCase( bool );
  void remove( QString );
  void clear();

 protected:
  void keyPressEvent( QKeyEvent * );

 private:
  int index;
  QStrList strings;
  bool allowInserts, nocase;

 signals:
  void activated ( int index ) ;
  void activated ( const QString & );
	void arrowPressed( bool );
};

class QHaccDateEdit : public QHaccLineEdit {
  Q_OBJECT
    public:
  QHaccDateEdit( QWidget *, const char * =0 );
  QHaccDateEdit( QDate, QWidget *, const char * =0 );
  virtual ~QHaccDateEdit();

  enum DateFormat { AMERICAN, EUROPEAN, YEARFIRST };
 
  QDate getDate() const;
  void setDate( QDate );
  bool event( QEvent * );

  static QString getDateString( QDate );
  static QString getShortDisplayString( QDate );
  static QDate getDateFromString( const QString& );
  
  static DateFormat getDateFormat();
  static QString getDateSeparator();

 protected:
  virtual void focusInEvent( QFocusEvent * );
  virtual void focusOutEvent( QFocusEvent * );

 private:
  static DateFormat form;
	static QString sep, AmForm, EuForm, YfForm;

  QDate day;

  public slots:
    static void setDateFormat( DateFormat );
  static void setDateSeparator( const QString& );
  void refresh();

 signals:
  void dateChanged( QDate );
};

class QHaccDateValidator : public QValidator {
	Q_OBJECT
		public:
	QHaccDateValidator( QWidget * =0, const char * =0 );
	~QHaccDateValidator();

	QValidator::State validate( QString &, int & ) const;
};
#endif
