/************************* * * * * * * * * * * * * ***************************
    Copyright (c) 1999-2003 Ryan Bobko
                       ryan@ostrich-emulators.com

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.     
************************* * * * * * * * * * * * * ***************************/

#include "qhacc.h"
#include "qhaccutils.h"
#include "qhaccsegmenter.h"
#include "qhacclineedits.h"
#include "qhaccreportwriter.h"

const uint QHaccReportWriter::AVENTRANS=  0;
const uint QHaccReportWriter::AVEADELTA=  1;
const uint QHaccReportWriter::AVELTRANS=  2;
const uint QHaccReportWriter::AVESTRANS=  3;
const uint QHaccReportWriter::AVEATRANS=  4;
const uint QHaccReportWriter::AVEDDELTA=  5;
const uint QHaccReportWriter::AVESPAN=    6;
const uint QHaccReportWriter::AVENDEBITS= 7;
const uint QHaccReportWriter::AVEADEBIT=  8;
const uint QHaccReportWriter::AVENCREDITS=9;
const uint QHaccReportWriter::AVEACREDIT= 10;


QString QHaccReportWriter::makereport( int type, QHacc * eng, uint ledgerid,
																			 QHaccTable * accts, const QDate& start,
																			 const QDate& end, QHaccTable *& ret ){
	// return the title of the report, plus the data in ret
	// ret should only contain columns of type CTSTRING

	// create the return table and row structure
	if( type==QHacc::MONTHLYD ){
		ColType ct[]={ CTSTRING, CTSTRING, CTSTRING, CTSTRING };
		const char * ns[]={ "MONTH", "CREDITS", "DEBITS", "TOTAL" };
		ret=new QHaccTable( ct, ns, 4 );
	}
	else if( type==QHacc::BUDGET ){
		ColType ct[]={ CTSTRING, CTSTRING, CTSTRING };
		const char * ns[]={ "MONTH", "ACCOUNT", "BUDGET" };
		ret=new QHaccTable( ct, ns, 3 );
	}
	else{
		ColType ct[]={ CTSTRING, CTSTRING };
		const char * ns[]={ "MONTH", "VALUE" };
		ret=new QHaccTable( ct, ns, 2 );
	}


	QHaccTable * trans=0;
	eng->makeTT( trans, QHacc::TID );

	QHaccTableIndex * idx=0;
	QString title, temp;
	uint * pos=0, sz=0;

	TableSelect ge( QHacc::TDATE, TableCol( start ), TableSelect::GE );
	TableSelect le( QHacc::TDATE, TableCol( end ), TableSelect::LE ); 
	TableSelect led;
	if( ledgerid!=0 ) led=TableSelect( QHacc::TLID, TableCol( ledgerid ) );

	// figure out the title and the affected transactions
	uint arows=accts->rows();
	for( uint i=0; i<arows; i++ ){
		Account acct=accts->at( i );

		TableSelect act( QHacc::TACCTID, acct.get( QHacc::AID ) );
		TableSelect criteria[]={ act, ge, le, led };

		QHaccTable * temptrans=0;
		eng->getTWhere( criteria, 4, temptrans );

		if( title.isNull() ){
			title=eng->getFNameOfA( acct );
			title.append( ": " );
		}
		else title="<Multiple>: ";

		if( type==QHacc::BUDGET ){
			// the budget report needs account and date info,
			// so do everything here
			temptrans->addIndexOn( QHacc::TDATE, idx );
			QHaccSegmenter::segment( eng, idx, start, end, pos, sz );
			
			QDate ndate=start;
			for( uint i=0; i<sz-1; i++ ){
				QString head=QHaccDateEdit::getShortDisplayString( ndate );
				head.append( "-" );
				
				ndate=ndate.addMonths( 1 );
				ndate=ndate.addDays( -1 );
				if( ndate>end ) ndate=end;
				head.append( QHaccDateEdit::getShortDisplayString( ndate ) );
				ndate=ndate.addDays( 1 );
				TableCol month( head );

				int sum=0, budget=eng->convMoney( acct.gets( QHacc::ABUDGET ) );
				for( uint j=pos[i]; j<pos[i+1]; j++ ){
					int tsum=eng->convMoney( idx->at( j ).gets( QHacc::TSUM ) );
					sum+=tsum;
				}
				
				TableCol tca( eng->getFNameOfA( acct ) );
				if( budget!=0 ){
					QString val;
					int obBal=sum-budget;
					if( ( budget<0 && sum<budget ) || ( budget>0 && sum>budget ) ){
						val+="+ ";
					}
					else{
						val+="- ";
						obBal=0-obBal;
					}

					val+=QString( "%1 (%2/%3)" ).arg( eng->convMoney( obBal ) ).arg( eng->convMoney( sum ) ).arg( eng->convMoney( budget ) );
					TableCol cols[]={ month, tca, TableCol( val ) };
					ret->add( TableRow( cols, 3 ) );
				}
				else{
					TableCol cols[]={ month, tca, TableCol( "no budget set" ) };
					ret->add( TableRow( cols, 3 ) );
				}
			}
		}
		else trans->load( *temptrans );

		delete temptrans;
	}

	// now, make the return data rows
	if( type==QHacc::BUDGET )	title.append( "Monthly Budget Status" );
	else if( type==QHacc::TOTALS ){
		title.append( "Totals by Payee" );
		trans->addIndexOn( QHacc::TPAYEE, idx );
		
		QHaccSegmenter::segment( eng, idx, pos, sz );
		
		for( uint i=0; i<sz-1; i++ ){
			Transaction t=idx->at( pos[i] );

			int sum=0;
			for( uint j=pos[i]; j<pos[i+1]; j++ )
				sum+=eng->convMoney( idx->at( j ).gets( QHacc::TSUM ) );
			
			TableCol cols[]={ TableCol( t.gets( QHacc::TPAYEE )+
																	"("+temp.setNum( pos[i+1]-pos[i] )+")" ),
												TableCol( eng->convMoney( sum ) ) };
			ret->add( TableRow( cols, 2 ) );
		}
	}
	else if( type==QHacc::MONTHLYB || type==QHacc::MONTHLYD ){
		bool bals=( type==QHacc::MONTHLYB );
		title.append( bals ? "Monthly Balances" : "Increases vs. Decreases" );
		trans->addIndexOn( QHacc::TDATE, idx );

		QHaccSegmenter::segment( eng, idx, start, end, pos, sz );
		
		QDate ndate=start;
		for( uint i=0; i<sz-1; i++ ){
			QString head=QHaccDateEdit::getShortDisplayString( ndate );
			head.append( "-" );

			ndate=ndate.addMonths( 1 );
			ndate=ndate.addDays( -1 );
			if( ndate>end ) ndate=end;
			head.append( QHaccDateEdit::getShortDisplayString( ndate ) );
			ndate=ndate.addDays( 1 );
			TableCol month( head );
			int sum=0, credits=0, debits=0;
			if( bals ){
				for( uint i=0; i<arows; i++ ){
					// opening balance will be the combined balance of all accounts
					Account acct=accts->at( i );
					sum+=eng->getABalOn( acct, start );
				}
			}

			for( uint j=pos[i]; j<pos[i+1]; j++ ){
				int tsum=eng->convMoney( idx->at( j ).gets( QHacc::TSUM ) );
				sum+=tsum;
				if( !bals ){
					if( tsum>0 ) credits+=tsum;
					else debits+=tsum;
				}
			}
			
			if( !bals ){
				TableCol cols[]={ month, TableCol( eng->convMoney( credits ) ),
													TableCol( eng->convMoney( debits ) ),
													TableCol( eng->convMoney( sum ) ) };
				ret->add( TableRow( cols, 4 ) );
			}
			else{
				TableCol cols[]={ month, TableCol( eng->convMoney( sum ) ) };
				ret->add( TableRow( cols, 2 ) );
			}
		}
	}
	else{
		title.append( "Averages" );
		uint rows=trans->rows();
		float daysTo=0;
		int total, big, small, cds, dbs, ncds, ndbs;
		total=big=small=cds=dbs=ncds=ndbs=0;

		QHaccTableIndex * idx=0;
		trans->addIndexOn( QHacc::TDATE, idx );

		for( uint i=0; i<rows; i++ ){
			Transaction t=idx->at( i );
			int tsum=eng->convMoney( t.gets( QHacc::TSUM ) );
			
			total+=tsum;
			
			if( i==0 ){
				big=tsum;
				small=tsum;
			}
			else{
				if( tsum>big ) big=tsum;
				if( tsum<small ) small=tsum;
				
				Transaction ot=idx->at( i-1 );
				daysTo+=ot.getd( QHacc::TDATE ).daysTo( t.getd( QHacc::TDATE ) );
			}
			
			if( tsum>0 ){
				cds+=tsum;
				ncds++;
			}
			else{
				dbs+=tsum;
				ndbs++;
			}
		}
		
		float ftot=eng->convMoney( total ).toFloat();
		const uint count=11;
		float sums[count];
		sums[AVENTRANS]=rows;
		sums[AVEADELTA]=eng->convMoney( total ).toFloat();
		sums[AVELTRANS]=eng->convMoney( big ).toFloat();
		sums[AVESTRANS]=eng->convMoney( small ).toFloat();
		sums[AVEATRANS]=ftot/( rows==0 ? 1 : rows );
		sums[AVEDDELTA]=ftot/( start>=end ? 1 : start.daysTo( end ) );
		sums[AVESPAN]=daysTo/( trans->rows()<2 ? 1 : trans->rows()-1 );
		sums[AVENDEBITS]=ncds;
		sums[AVEADEBIT]=eng->convMoney( cds ).toFloat()/( ncds==0 ? 1 : ncds );
		sums[AVENCREDITS]=ndbs;
		sums[AVEACREDIT]=eng->convMoney( dbs ).toFloat()/( ndbs==0 ? 1 : ndbs );
		
		const char * strs[]={ "Number of Transactions:",
													"Account Delta:",
													"Largest Transaction:",
													"Smallest Transaction:",
													"Average Transaction:",
													"Average Daily Change:",
													"Average Days Between Transactions:",
													"Number of Credits:",
													"Average Credit:",
													"Number of Debits:",
													"Average Debit:" };

		for( uint i=0; i<count; i++ ){
			int x=2;
			if( i==AVENTRANS || i==AVENCREDITS || i==AVENDEBITS ) x=0;

			TableCol cols[]={ TableCol( strs[i] ),
												TableCol( temp.setNum( sums[i], 'f', x ) ) };
			ret->add( TableRow( cols, 2 ) );
		}

		//cout<<"result is: "<<endl;for( uint i=0; i<ret->rows(); i++ ) cout<<ret->at( i ).toString()<<endl; cout<<"END"<<endl;

	}

	delete trans;
	if( pos ) delete [] pos;

	if( ledgerid!=0 )
		title.prepend( eng->getL( ledgerid ).gets( QHacc::LNAME )+"/" );
	
	title.append( " (" );
	title.append( QHaccDateEdit::getDateString( start ) );
	title.append( "-" );
	title.append( QHaccDateEdit::getDateString( end ) );
	title.append( ")" );
	return title;
}

QString QHaccReportWriter::writereport( const QString& title,
																				QHaccTable * data ){
	QString report=title;
	if( data->isEmpty() )	report.append( "\n\tNo Transactions" );
	else{
		uint cols=data->columns(), rows=data->rows();
		QString format="\n";

		// figure out each column's longest string
		int bigColLen[cols];
		//cout<<" colsxrows="<<cols<<"x"<<rows<<endl;
		for( uint col=0; col<cols; col++ ){
			format.append( " %"+QString::number( col+1 ) );
			bigColLen[col]=0;

			//cout<<"col is "<<col<<endl;
			for( uint row=0; row<rows; row++ ){
				//cout<<"  row is "<<row<<"  "<<data->at( row ).toString()<<endl;
				QString str=data->at( row ).gets( col );
				if( str.length()>( uint )bigColLen[col] ) bigColLen[col]=str.length();
			}
		}

		bigColLen[0]=0-bigColLen[0]; // left-justify the first col

		// create a string that contains the entire report
		for( uint row=0; row<rows; row++ ){
			QString str=format;
			for( uint col=0; col<cols; col++ )
				str=str.arg( data->at( row ).gets( col ), bigColLen[col] );
			report.append( str );
		}
	}
	
	return report;
}

QString QHaccReportWriter::report( int type, QHacc * eng, uint lid, 
																	 QHaccTable * accts, const QDate& start,
																	 const QDate& end ){
	// an all-in-one report writer

	QHaccTable * table=0;
	QString report=writereport( makereport( type, eng, lid, accts, start,
																					end, table ), table );
	delete table;
	return report;
}

QString QHaccReportWriter::journal( int ledgerid, QHacc * engine,
																		const QDate& start, const QDate& end ){
	// all-in-one journal creater/writer
	Ledger led;
	if( ledgerid==0 ){ 
		TableCol tcs[]={ TableCol( ( uint )0 ), TableCol( "All Ledgers" ),
										 TableCol( "" ) };
		led=Ledger( tcs, 3 );
	}
	else led=engine->getL( ledgerid );
	
	QString journ( led.gets( QHacc::LNAME )+" Journal" );
	journ.append( " ("+QHaccDateEdit::getDateString( start )+"-"+
								QHaccDateEdit::getDateString( end )+")\n" );
	journ.append( QString( "%1 %2 R %3%4" ).arg( "Date",-10 ).arg( "Payee/Memo",-33 ).arg( "Credit", 10 ).arg( "Debit", 10 ) );
								

	QHaccTable * trans=0;
	TableSelect criteria[]={
		( ledgerid==0 ? TableSelect() : TableSelect( QHacc::TLID,
																								 led.get( QHacc::LID ) ) ), 
		TableSelect( QHacc::TDATE, TableCol( start ), TableSelect::GE ),
		TableSelect( QHacc::TDATE, TableCol( end ), TableSelect::LT )
	};
	engine->getTWhere( criteria, 3, trans );
	
	QHaccTableIndex * dsgidx=0;
	trans->addIndexOn( QHacc::TDATE, QHacc::TSPLITGROUP, dsgidx );
	
	//for( uint i=0; i<trans->rows(); i++ ) cout<<i<<": "<<dsgidx->at( i ).toString()<<endl;

	uint lastsg=0;
	uint lim=trans->rows();
	//cout<<" journaling "<<lim<<" transactions"<<endl;

	for( uint i=0; i<lim; i++ ){
		Transaction t=dsgidx->at( i );
		uint sg=t.getu( QHacc::TSPLITGROUP );
		
		if( sg!=lastsg ){ // write the header info
			lastsg=sg;
			journ.append( "\n\n" );
			journ.append( QHaccDateEdit::getDateString( t.getd( QHacc::TDATE ) ) );
			journ.append( " "+t.gets( QHacc::TPAYEE ) );
			if( !t.gets( QHacc::TMEMO ).isEmpty() )
				journ.append( "\n            "+t.gets( QHacc::TMEMO ) );
		}


		// write the adjustments info
		int offset=0;
		Account acct=engine->getA( t.getu( QHacc::TACCTID ) );

		QString sum=t.gets( QHacc::TSUM );
		if( sum.startsWith( "-" ) ){
			sum=sum.mid( 1 );
			offset=20;
		}
		else offset=10;

		//make sure we have the proper format
		sum=engine->convMoney( engine->convMoney( sum ) );
		journ.append( QString( "\n              %1 %2 %3" ).arg( engine->getFNameOfA( acct ), -30 ).arg( t.getu( QHacc::TRECO )==QHacc::YREC ? "R" : "-" ).arg( sum, offset ) );
	}

	delete trans;
	return journ;
}
