/************************* * * * * * * * * * * * * ***************************
    Copyright (c) 1999-2003 Ryan Bobko
                       ryan@ostrich-emulators.com

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.     
************************** * * * * * * * * * * * * **************************/

#include "qhacc.h"
#include "qhaccscrollview.h"
#include "qhaccscrollview.moc"

#include <qevent.h>
#include <qcursor.h>
#include <qheader.h>
#include <qpainter.h>
#include <qpopupmenu.h>

const int QHaccScrollView::HEADERH=20;

QHaccScrollView::QHaccScrollView( QHacc * eng, QWidget * p, const char * n ) 
	: QScrollView( p, n ){
	
	engine=eng;
	
	index=0;
	popup=0;
	sortcol=0;	
	rowsize=1;
	linesize=1;
	linesperrow=1;
	ium=chosen=-1;
	sortasc=paintOverlap=true;
	pattern=HIGHLIGHTCHOSEN;
	
	head=new QHeader( this );
	head->resize( visibleWidth(), HEADERH );
  head->setClickEnabled( true );

	setMargins( 0, HEADERH, 0, 0 );
  setMidLineWidth( 0 );
  setLineWidth( 1 );

  viewport()->setFocusPolicy( StrongFocus );
  viewport()->setMouseTracking( true );  

	connect( head, SIGNAL( clicked( int ) ), SLOT( clickHeader( int ) ) );
	connect( head, SIGNAL( sizeChange( int, int, int ) ), SLOT( resizer() ) );

	const ColType cts[]={ CTNULL };
	const char * nms[]={ "NULL" };
	data=new QHaccTable( cts, nms, 1, -1, 0, 1, 0, 0 );
}

QHaccScrollView::~QHaccScrollView(){ delete data; }

void QHaccScrollView::setPaintPattern( PaintPattern pp ){	pattern=pp; }
QHeader * QHaccScrollView::header() const { return head; }

int QHaccScrollView::getIUM() const { return ium; }
int QHaccScrollView::getChosen() const { return chosen; }

TableRow QHaccScrollView::getData( int i ) const {
	TableRow ret;
	if( i>=0 && ( uint )i<data->rows() ){
		uint idx=i;
		if( !sortasc ) idx=data->rows()-( i+1 );
		ret=index->at( idx );
	}
	return ret;
}

TableRow QHaccScrollView::getMoused() const {
	Transaction t=getData( ium );
	if( !t.isNull() )	return uconv( t );
	return t;
}

TableRow QHaccScrollView::getSelected() const {
	Transaction t=getData( chosen );
	if( !t.isNull() ) return uconv( t );
	return t;
}
void QHaccScrollView::refresh(){ repaintContents( false ); }
void QHaccScrollView::grabData( QHaccTable *& table ){ table=0; }
void QHaccScrollView::makeData(){
	TableRow sel;;
	// if we have an index, try to maintain the
	// chosen row after resetting the data table
	if( data->idcol()>-1 ) sel=getData( getChosen() );

	delete data;
	index=0;
	
	grabData( data );
	makeIndexFor( sortcol, index );

	// here's the chosen row maintenance
	setOldChosen( sel );

	resizer();
}

void QHaccScrollView::setOldChosen( const TableRow& old ){
	if( old.isNull() ) isetC( -1 );
	else{
		// the idea is to find whatever row used to be highlighted and
		// set chosen to that position. this avoids unnecessary repains
		if( data->idcol()>-1 && index ){
			uint idx=0;
			//if( data->contains( old, idx ) ){
			//index->contains( old, idx );
			if( index->contains( old, idx ) ){
				if( !sortasc ) idx=data->rows()-( idx+1 );
				isetC( ( int )idx );
			}
			else setChosen( chosen );
		}
		else isetC( -1 );
	}
}

void QHaccScrollView::setIUM( int i ){ 
	if( i!=ium ){
		int oldi=ium;
		ium=i;
		repaintIndex( oldi );
		repaintIndex( ium );
	}
}

void QHaccScrollView::isetC( int i ) {
	int oldc=chosen;
	chosen=i;
	repaintIndex( oldc );
	repaintIndex( chosen );	
}
void QHaccScrollView::doSetChosen( int ) {}
void QHaccScrollView::setChosen( int i ){
	int oldc=chosen;
	chosen=i;
	repaintIndex( oldc );
	doSetChosen( i );
	repaintIndex( chosen );
}

void QHaccScrollView::setPopup( QPopupMenu * pop ){
	if( popup ) delete popup;
	popup=pop;
}

void QHaccScrollView::setLinesPerRow( int i ){
	linesperrow=i;
	rowsize=linesperrow*linesize;
	resizer();
}
int QHaccScrollView::getRowSize() const { return rowsize;}

void QHaccScrollView::resizer(){
	head->resize( visibleWidth(), HEADERH );
	// the +3 is to give a little buffer before the last row ends
	// we need a buffer or the transeditor will look smooshed
	resizeContents( visibleWidth(), getRowSize()*( data->rows()+2 )+3 );
	refresh();
}

void QHaccScrollView::resizeEvent( QResizeEvent * ){ resizer(); }

void QHaccScrollView::keyPressEvent( QKeyEvent * qke ){
	int temp=0;
	int rows=data->rows()+1;
	int key=qke->key();
  switch( key ){
  case Key_Space:
    setChosen( getIUM() );
		qke->accept();
    break;
  case Key_Up:
    if ( getIUM()>0 ) setIUM( getIUM()-1 );
		qke->accept();
    break;
  case Key_Down:
    if ( getIUM()<=rows ) setIUM( getIUM()+1 );
		qke->accept();
    break;
	case Key_PageUp:
		temp=( visibleHeight()/rows*getRowSize() ); // rows/screen
		if( getIUM()-temp<0 ) setIUM( 0 );
		else setIUM( getIUM()-temp );
		qke->accept();
		break;
	case Key_PageDown:
		temp=( visibleHeight()/rows*getRowSize() ); // rows/screen
		if( getIUM()+temp>=rows ) setIUM( rows );
		else setIUM( getIUM()+temp );
		qke->accept();
		break;
	case Key_Delete:
		deleteRow( getIUM() );
		qke->accept();
		break;
	case Key_End:
		setIUM( rows-2 );
		qke->accept();
		break;
	case Key_Home:
		setIUM( 0 );
		qke->accept();
		break;
  default:
    qke->ignore();
  }
	ensureVisible( 0, ium*getRowSize() );
}

void QHaccScrollView::viewportMouseMoveEvent( QMouseEvent * qme ) {
	QPoint p=viewportToContents( qme->pos() );
	setIUM( p.y()/getRowSize() );
}
void QHaccScrollView::viewportMousePressEvent( QMouseEvent * qme ) {
	if( popup && qme->button()==QMouseEvent::RightButton )
		popup->popup( QCursor::pos() );
}

void QHaccScrollView::viewportMouseReleaseEvent( QMouseEvent * qme ) {
  QPoint p=viewportToContents( qme->pos() );
	setChosen( p.y()/getRowSize() );
}

void QHaccScrollView::updateRow( const TableRow& old, const TableRow& newy ){
	//cout<<"qsv old="<<old.toString()<<"\tnew="<<newy.toString()<<endl;
	int idcol=data->idcol();
	if( idcol>=0 ){
		data->updateWhere( idcol, old.get( idcol ), newy );
		if( index ){
			// if the data change will affect the index ordering, refresh everything
			if( old.compareTo( newy, index->sorts() )!=0 ) makeData();
			else{ // else just repaint the row
				uint idx=0;
				if( index->contains( newy, idx ) ){
					if( !sortasc ) idx=data->rows()-( idx+1 );
					//cout<<"updating "<<old.toString()<<" to "<<newy.toString()<<" at index "<<idx<<endl;
					repaintIndex( idx );
				}
			}
		}
	}
	else makeData();
}

void QHaccScrollView::repaintIndex( int i ){
	paintOverlap=false;
  repaintContents( 0, i*getRowSize(), visibleWidth(), getRowSize(), false );
	paintOverlap=true;
}

void QHaccScrollView::isave() const {}
void QHaccScrollView::ireadPrefs( bool ){}

void QHaccScrollView::save() const { isave(); }
void QHaccScrollView::readPrefs( bool initial ){
	if( initial ){
		connect( engine, SIGNAL( changedP( const QString&, QFont ) ),
						 SLOT( changeP( const QString&, QFont ) ) );
		connect( engine, SIGNAL( changedP( const QString&, QColor ) ),
						 SLOT( changeP( const QString&, QColor ) ) );
	}

	maincol=engine->getCP( "MAINCOLOR" );
	altcol=engine->getCP( "ALTCOLOR" );

	ireadPrefs( initial );
	changeP( "FONT", engine->getWP( "FONT" ) );
}

void QHaccScrollView::changeP( const QString& pref, QFont f ){
	if( pref=="FONT" ){
		linesize=QFontMetrics( f ).lineSpacing();
		rowsize=linesize*linesperrow;
		resizer();
	}
}

void QHaccScrollView::changeP( const QString& pref, QColor c ){
	if( pref=="ALTCOLOR" ) altcol=c;
	else maincol=c;
	refresh();
}
int QHaccScrollView::getLinesPerRow() const { return linesperrow; }
bool QHaccScrollView::sortAsc() const { return sortasc; }
int QHaccScrollView::sortCol() const { return sortcol; }
void QHaccScrollView::setSorter( int i, bool b ){
	sortasc=b;
	sortcol=i;
	data->dropIndex( index );
	//if( index ) delete index;
	makeIndexFor( sortcol, index );
}

TableRow QHaccScrollView::conv( const TableRow& old ) const {	return old; }
TableRow QHaccScrollView::uconv( const TableRow& old ) const { return old; }

void QHaccScrollView::makeIndexFor( int, QHaccTableIndex *& idx ) const {
	idx=0;
}

void QHaccScrollView::clickHeader( int i ){
	TableRow oldc=getData( getChosen() );

	if( i==sortcol ) sortasc=!sortasc;		
	else{
		sortcol=i;
		if( index ) data->dropIndex( index );
		makeIndexFor( i, index );
	}
	
	setOldChosen( oldc );
	refresh();
}

void QHaccScrollView::deleteRow( int ){}
// this is the function that draws one row of the dataset
void QHaccScrollView::drawRow( QPainter *, int, int, TableRow& ){}
void QHaccScrollView::drawContents( QPainter * p, int x, int y, int w, int h ){
	// start with a basic color
	p->fillRect( x, y, w, h, QBrush( altcol ) );

	int vw=visibleWidth();
	int rowsize=getRowSize();
	QFont f=font();

	// figure out where to start, and how many rows to draw
	uint start=( y/rowsize );
	uint stop=( ( y+h )/rowsize );
	uint rows=data->rows();

	if( paintOverlap ) stop++; // this is to make scrolling look better
	uint tstop=stop; // last transaction

	uint yLine=start*rowsize;
	if( tstop>rows ) tstop=rows;

	// paint a background color
	if( pattern==ALTROWS ){
		// paint alternating rows of color
		QBrush qb( SolidPattern );
		uint ender=( ( y+h )/rowsize );
		for( uint i=start; i<ender+1; i++ ){ // the +1 is for scrolling
			if( i%2 )	qb.setColor( maincol );
			else qb.setColor( altcol );
			p->fillRect( 0, yLine, vw, rowsize, qb );
			yLine+=rowsize;
		}
	}
	else{
		// paint a highlighted strip under the chosen index
		uint idx=start;
		while( idx<tstop ){
			if( ( int )idx==chosen ){
				p->fillRect( 0, yLine, vw, rowsize, QBrush( maincol ) );
				break;
			}
			idx++;
			yLine+=rowsize;
		}
	}
	

	// still need to paint the appropriate data to the screen
	yLine=start*rowsize;
	bool unbold=false;
	uint i=start;
	for( ; i<=tstop; i++ ){
		if( ( int )i==ium ){
			QFont f2=font();
			f2.setBold( true );
			p->setFont( f2 );
			unbold=true;
		}
		else if( unbold ){
			p->setFont( f );
			unbold=false;
		}

		TableRow row;
		// this if stmt will give us a null row immediately after
		// the last row of data, which we can use as a "total" row
		if( i<rows ){
			if( sortasc ) row=index->at( i );
			else row=index->at( data->rows()-( i+1 ) );
		}
		
		drawRow( p, yLine, linesize, row );
		yLine+=rowsize;
	}

	if( viewport()->hasFocus() )
		p->drawRect( 0, 0, contentsWidth(), visibleHeight()>contentsHeight() ?
								 visibleHeight()-1 : contentsHeight()-1 );
}
