/************************* * * * * * * * * * * * * ***************************
    Copyright (c) 1999-2003 Ryan Bobko
                       ryan@ostrich-emulators.com

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.     
************************** * * * * * * * * * * * * **************************/

#ifndef _QHACCSCROLLVIEW_H
#define _QHACCSCROLLVIEW_H

#include <qfont.h>
#include <qcolor.h>
#include <qscrollview.h>

class QHacc;
class TableRow;
class QHaccTable;
class QHaccTableIndex;

class QHeader;
class QPainter;
class QKeyEvent;
class QPopupMenu;
class QMouseEvent;
class QResizeEvent;

class QHaccScrollView : public QScrollView {
	Q_OBJECT
		public:
	QHaccScrollView( QHacc *, QWidget * =0, const char * =0 );
	virtual ~QHaccScrollView();

	QHeader * header() const;

	TableRow getMoused() const;
	TableRow getSelected() const;
	TableRow getData( int ) const;

	int getIUM() const;
	int getChosen() const;

	void save() const;
	void readPrefs( bool );

	virtual void grabData( QHaccTable *& );
	void drawContents ( QPainter * p, int x, int y, int w, int h );
	virtual void drawRow( QPainter *, int y, int h, TableRow& );

 protected:
	QPopupMenu * popup;
	QHaccTable * data;
	QHacc * engine;

	enum PaintPattern { ALTROWS, HIGHLIGHTCHOSEN };

	void resizeEvent( QResizeEvent * );
	virtual void keyPressEvent( QKeyEvent * );
	void viewportMouseMoveEvent( QMouseEvent * );
	void viewportMousePressEvent( QMouseEvent * );
	void viewportMouseReleaseEvent( QMouseEvent * );
	void repaintIndex( int );
	void updateRow( const TableRow& older, const TableRow& newer );

	void setIUM( int );
	void setChosen( int );
	virtual void doSetChosen( int );
	void setPaintPattern( PaintPattern );

	int sortCol() const;
	bool sortAsc() const;
	void setLinesPerRow( int );
	void setSorter( int, bool );
	void setPopup( QPopupMenu * );
	void refresh();
	int getLinesPerRow() const;
	int getRowSize() const;

	void setOldChosen( const TableRow& );

	virtual void deleteRow( int );
	virtual TableRow conv( const TableRow& ) const;
	virtual TableRow uconv( const TableRow& ) const;
	virtual void makeIndexFor( int, QHaccTableIndex *& ) const;

	virtual void isave() const;
	virtual void ireadPrefs( bool );

 private:
	static const int HEADERH;

	QHeader * head;
	int ium, chosen, sortcol, linesperrow, linesize, rowsize;
	bool paintOverlap, sortasc;
	PaintPattern pattern;
	QColor altcol, maincol;
	QHaccTableIndex * index;

	void isetC( int i );
	
	private slots:
	void changeP( const QString&, QFont );
	void changeP( const QString&, QColor );
	void clickHeader( int );

	protected slots:
		void makeData();
	void resizer();
};

#endif
