/************************* * * * * * * * * * * * * ***************************
    Copyright (c) 1999-2003 Ryan Bobko
                       ryan@ostrich-emulators.com

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.     
************************* * * * * * * * * * * * * ***************************/

#include "qhaccview.h"
#include "qhaccutils.h"
#include "qhacclineedits.h"
#include "qhaccsubsetwin.h"
#include "qhaccreportwriter.h"
#include "qhaccsubsetwin.moc"

#include <qlabel.h>
#include <qlayout.h>
#include <qpushbutton.h>

const int QHaccSubsetWin::NTRANS=5;
const int QHaccSubsetWin::ADELTA=6;
const int QHaccSubsetWin::LTRANS=7;
const int QHaccSubsetWin::STRANS=8;
const int QHaccSubsetWin::ATRANS=9;

QHaccSubsetWin::QHaccSubsetWin( QHacc * e, QWidget * p, const char * n )
	: QWidget( p, n, WDestructiveClose ) {
  
	engine=e;
	ledgerid=0;
	setFont( e->getWP( "FONT" ) );
  view=new QHaccView( e, this );

  startdate=new QHaccDateEdit( QDate(), this );
  startdate->setAlignment( AlignCenter );

  enddate=new QHaccDateEdit( QDate(), this );
  enddate->setAlignment( AlignCenter );
  
  QPushButton * ok=new QPushButton( tr( "Dismiss" ), this );

  connect ( ok, SIGNAL( clicked() ), SLOT( close() ) );
  
  connect( enddate, SIGNAL( dateChanged( QDate ) ), SLOT( changeDates() ) );
  connect( startdate, SIGNAL( dateChanged( QDate ) ), SLOT( changeDates() ) );
  
  QGridLayout * layout=new QGridLayout( this, 15, 20, 5 );
  QWidget * widgs[]={ new QLabel( tr( "Start Date" ), this ), startdate,
											new QLabel( tr( "End Date" ), this ), enddate, ok,
											new QLabel( this ) };
											
	int align=AlignCenter;
	int row=0;
	for( int i=0; i<6; i++ ){
		// fields and buttons should span the whole area
		if( i==0 || i==2 ) align=AlignCenter;
		else align=0;

		layout->addMultiCellWidget( widgs[i], row, row, 0, 2, align );
		row++;
	}

	infosize=5;
	const char * infos[]={ QT_TR_NOOP( "Transactions:" ),
												 QT_TR_NOOP( "Delta:" ),
												 QT_TR_NOOP( "Largest:" ),
												 QT_TR_NOOP( "Smallest:" ),
												 QT_TR_NOOP( "Average:" ) };

	info=new QLabel * [infosize*2];
	for( uint i=0; i<infosize; i++ ){
		info[i]=new QLabel( infos[i], this );
		info[i]->setAlignment( AlignLeft );
		info[i+infosize]=new QLabel( this );
		info[i+infosize]->setAlignment( AlignRight );
		layout->addMultiCellWidget( info[i], row, row, 0, 1, AlignLeft );
		layout->addMultiCellWidget( info[i+infosize], row, row, 1, 2, AlignRight );
		row++;
	}

  layout->addMultiCellWidget( view, 0, 14, 3, 19 );

	readPrefs( true );
}

QHaccSubsetWin::~QHaccSubsetWin(){ delete [] info; }

void QHaccSubsetWin::setAL( const Account& a, const Ledger& l ){
	view->setLedger( l );
	view->setAccount( a );
	if( !a.isNull() && !l.isNull() ){
		setCaption( tr( "Subset View of" )+" "+l.gets( QHacc::LNAME )+"/"+a.gets( QHacc::ANAME ) );
		account=a;
		ledgerid=l.getu( QHacc::LID );
		setInfo();
	}
}

void QHaccSubsetWin::readPrefs( bool initial ){
	view->readPrefs( initial );
	if( initial ){
		connect( engine, SIGNAL( changedP( const QString&, QString ) ),
						 this, SLOT( changeP( const QString&, QString ) ) );
		connect( engine, SIGNAL( changedP( const QString&, int ) ),
						 this, SLOT( changeP( const QString&, int ) ) );
		connect( engine, SIGNAL( removedA( const Account& ) ),
						 this, SLOT( removeA( const Account& ) ) );
	}

	QString str=engine->getSP( "SUBSETWINSIZE" );
	if( !str.isEmpty() ){
		QString vals[2];
		Utils::parser( str, " ", 0, vals, 2 );
		resize( vals[0].toInt(), vals[1].toInt() );
	}

	// set the starting date equal to the graphtime limit
	QDate de=enddate->getDate();
	startdate->setDate( de.addMonths( 0-engine->getIP( "GRAPHTIME" ) ) );
}

void QHaccSubsetWin::save(){
	QString str;
	QSize s=size();
	engine->setSP( "SUBSETWINSIZE",	str.setNum( s.width() )+
								 " "+str.setNum( s.height() ) );
	view->save();
}

void QHaccSubsetWin::changeP( const QString& str, QString ){
	if( str=="DATESEPARATOR" ){
		enddate->refresh();
		startdate->refresh();
	}
}

void QHaccSubsetWin::changeP( const QString& str, int ){
	if( str=="DATEFORMAT" ){
		enddate->refresh();
		startdate->refresh();
	}
}

void QHaccSubsetWin::changeDates() {
	QDate start=startdate->getDate();
	QDate end=enddate->getDate();

	TableSelect criteria[]={
		TableSelect( QHacc::TDATE, TableCol( start ), TableSelect::GE ),
		TableSelect( QHacc::TDATE, TableCol( end ), TableSelect::LE )
	};

	view->setSelectors( criteria, 2 );
	setInfo();
}

void QHaccSubsetWin::setAll( uint lid, const Account& acct,
														 const QDate& s, const QDate& e ){
  disconnect( enddate, SIGNAL( dateChanged( QDate ) ),
							this, SLOT( changeDates() ) );
  disconnect( startdate, SIGNAL( dateChanged( QDate ) ),
							this, SLOT( changeDates() ) );

	startdate->setDate( s );
	enddate->setDate( e );
	
	connect( enddate, SIGNAL( dateChanged( QDate ) ), SLOT( changeDates() ) );
  connect( startdate, SIGNAL( dateChanged( QDate ) ), SLOT( changeDates() ) );

	setAL( acct, engine->getL( lid ) );
	changeDates();
}


void QHaccSubsetWin::setInfo(){
	if( !( account.isNull() || ledgerid==0 ) ){
		QHaccTable * accts=0, * aves=0;
		engine->makeAT( accts );
		accts->add( account );

		QHaccReportWriter::makereport( QHacc::AVES, engine, ledgerid, accts,
																	 startdate->getDate(), enddate->getDate(),
																	 aves );

		// we only want to display some of the rows from the averages report
		uint rows[]={ QHaccReportWriter::AVENTRANS,
									QHaccReportWriter::AVEADELTA,
									QHaccReportWriter::AVELTRANS,
									QHaccReportWriter::AVESTRANS,
									QHaccReportWriter::AVEATRANS };
									
		uint lkp[]={ NTRANS, ADELTA, LTRANS, STRANS, ATRANS };
		for( uint i=0; i<infosize; i++ ){
			TableRow data=aves->at( rows[i] );
			info[lkp[i]]->setText( data.gets( 1 ) );
		}

		delete accts;
		delete aves;
	}
	else for( uint i=infosize; i<2*infosize; i++ ) info[i]->setText( "" );
}

void QHaccSubsetWin::close(){
	save();
	QWidget::close( true );
}

void QHaccSubsetWin::removeA( const Account& a ){
	if( a.compareTo( account, QHacc::AID ) ) close();
}
