/************************** * * * * * * * * * * * * ***************************
    Copyright (c) 1999-2003 Ryan Bobko
                       ryan@ostrich-emulators.com

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.     
*************************** * * * * * * * * * * * * **************************/

#ifndef _QHACCTABLE_H
#define _QHACCTABLE_H

#include "qhaccutils.h"
#include "qhacctablerows.h"

#include <qlist.h>
#include <qstring.h>
#include <qdatetime.h>

class QFile;

class ostream;

class QHaccDB;
class QHaccTableIndex;

class TableSelect {
 public:
	static const int EQ, NE, GT, GE, LT, LE, UQ, NO, ALL;

	TableSelect( int col, TableCol model, int checktype =TableSelect::EQ );
	TableSelect();
	TableSelect( const TableSelect& );
	const TableSelect& operator=( const TableSelect& );
	~TableSelect();

	void getAll( int& pos, TableCol& equal, int& type ) const;
	bool check( const TableRow& ) const;
	QString toString() const;
	QString sqlValue() const;

 protected:
	int column, checktype;
	TableCol model;
};

class QHaccTable {
 public:
	friend class QHaccTableIndex;
	//friend class QHaccDB loadRow( const QString& );
	friend class QHaccDB;

	QHaccTable( const ColType *, const char **, int, int pkcol =-1,
							const char * name =0, uint =5, uint =5, uint =8);
	QHaccTable( const QHaccTable& );
	QHaccTable& operator=( const QHaccTable& );
	virtual ~QHaccTable();

	bool load( const QHaccTable& );
	bool load( QFile&, QString& );
	bool save( QFile&, QString& ) const;
	void startLoad( uint =0 );
	void stopLoad();
	void clear();

	TableRow at( uint idx ) const;
	int add( const TableRow& );


	//friend std::ostream& operator<<( std::ostream&, const QHaccTable& );
	bool getWhere( TableSelect *, uint size, QHaccTable *& results );
	bool getWhere( const TableSelect&, QHaccTable *& results );

	int idcol() const;

	uint rows() const;
	uint columns() const;
	TableCol maxID() const;
	TableCol minID() const;
	bool isEmpty() const;
	
	uint deleteWhere( const TableSelect& );
	uint deleteWhere( int position, const TableCol& equals );
	uint deleteWhere( QHaccTable * );

	int updateWhere( const TableSelect&, int, const TableCol& replace );
	int updateWhere( int position, const TableCol& equals, 
									 const TableRow& replace );
	int updateWhere( int checkpos, const TableCol& equals, int changepos,
									 const TableCol& replace );

	void dropIndex( QHaccTableIndex * );
	void addIndexOn( int col, QHaccTableIndex *& );
	void addIndexOn( int col, int subcol, QHaccTableIndex *& );
	bool contains( const TableCol&, uint& ) const;
	bool contains( const TableRow&, uint& ) const;
	
	void setName( QString );
	QString colname( int ) const;

	static const int VALID;
	static const int CHANGESIZE;
	static const int COLCOUNT;
	static const int COLTYPE;
	static const int NULLROW;
	static const int DUPLKEY;
	static const int NOTFOUND;
	static const char * error( int code );

 protected:
	virtual int verifyRow( const TableRow * ) const;
	virtual void loadRow( const QString& );

 private:
	int cols, pkcol;
	uint size, maxFree, arrSize, growBy;
	QString * names, name;
	ColType * types;
	TableRow ** data;
	QList<QHaccTableIndex> indexes;
	bool loading, deleter;
	QHaccTableIndex * pki;

	int  add( TableRow * );
	void at( uint idx, TableRow *& );
	void update( const TableRow *, const TableRow& );
	void remove( uint idx );
	void deleteWhere( uint * idxarr, uint size );

	void setAutoDelete( bool );
	void igetWhere( const TableSelect&, QHaccTable * rows, QHaccTable *& ret );
	bool getIndexOn( int pos, QHaccTableIndex *& );
	bool getIndexOn( int pos, int spos, QHaccTableIndex *& );
	void removeIndex( QHaccTableIndex * );
	void resizeTo( uint );
	void reindex();
	void remake();

	bool debug( int, std::ostream *& ) const;
	bool error( int, std::ostream *& ) const;
};




class QHaccTableIndex {
 public:
	QHaccTableIndex( const QHaccTableIndex& );
	QHaccTableIndex( QHaccTable *, int );
	QHaccTableIndex( QHaccTable *, int, int );
	QHaccTableIndex& operator=( const QHaccTableIndex& );
	virtual ~QHaccTableIndex();
	
	TableRow at( uint ) const;
	uint rows() const;

	void reindex();
	void remake();

	uint starts( const TableCol& );
	uint ends( const TableCol& );
	bool contains( const TableCol& );
	bool contains( const TableCol&, uint& idx );
	bool contains( const TableRow&, uint& idx );
	void unique( QHaccTable *& );
	bool sorts( int ) const;
	bool sorts( int, int ) const;
	int sorts() const;
	uint loc( uint idx ) const;
	TableCol maxID();
	TableCol minID();

 private:
	uint * lookup;
	QHaccTable * data;
	int field, subfield;
	
	void init( QHaccTable *, int, int );
	void at( uint idx, TableRow *& );

	// here are our quicksort functions
	void qsort( uint, uint );
	uint findQ( uint, uint );
	void swap( uint, uint );
	uint ifind( const TableCol&, uint start, uint end, bool& exactFind );
};

#endif
