/************************* * * * * * * * * * * * * ***************************
    Copyright (c) 1999-2003 Ryan Bobko
                       ryan@ostrich-emulators.com

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.     
************************** * * * * * * * * * * * * **************************/

#include "qhaccview.h"
#include "qhaccutils.h"
#include "transeditor.h"
#include "qhaccdialogs.h"
#include "qhacclineedits.h"
#include "qhaccview.moc"

#include <qheader.h>
#include <qcursor.h>
#include <qaction.h>
#include <qpixmap.h>
#include <qlayout.h>
#include <qpainter.h>
#include <qcheckbox.h>
#include <qpopupmenu.h>
#include <qmessagebox.h>
#include <qcolordialog.h>
#include <qfontmetrics.h>

const int QHaccView::NHEADS=7;
const char * QHaccView::HEADS[]={ QT_TR_NOOP( "Num" ), QT_TR_NOOP( "Date" ),
																	QT_TR_NOOP( "Payee" ), QT_TR_NOOP( "Memo" ), 
																	QT_TR_NOOP( "Deposit" ),
																	QT_TR_NOOP( "Withdrawal" ),
																	QT_TR_NOOP( "Rec" ) };

const int QHaccView::NUM=   0;
const int QHaccView::DATE=  1;
const int QHaccView::PAYEE= 2;
const int QHaccView::MEMO=  3;
const int QHaccView::SUM=   4;
const int QHaccView::RECO=  5;
const int QHaccView::PNAME= 6;
const int QHaccView::SGRP=  7;
const int QHaccView::ID=    8;
const int QHaccView::AID=   9;
const int QHaccView::PFNAME=10;
const int QHaccView::FSUM=  11;
const int QHaccView::INUM=  12;
const int QHaccView::COLS=  13;

const char * QHaccView::COLNAMES[]={ "NUM", "DATE", "PAYEE", "MEMO", "SUM",
																		 "RECO", "PAIRNAME", "SGRP", "ID", "AID",
																		 "PAIRFNAME", "ISUM", "INTNUM" };
const ColType QHaccView::COLTYPES[]={ CTSTRING, CTDATE, CTSTRING, CTSTRING,
																			CTSTRING, CTUINT, CTSTRING, CTUINT,
																			CTUINT, CTUINT, CTSTRING, CTINT,
																			CTINT };


QHaccTable * QHaccView::copytrans=0;

QHaccView::QHaccView( QHacc * eng, QWidget * p, const char * n )
  : QHaccScrollView( eng, p, n ){
	
  QHeader * head=header();
	for( int i=0; i<NHEADS; i++ ) head->addLabel( tr( HEADS[i] ) );
	head->setMovingEnabled( false );

	setHScrollBarMode( AlwaysOff );
	setVScrollBarMode( AlwaysOn );

	viewport()->setMouseTracking( true );
	setPaintPattern( ALTROWS );
	editor=0;
	editorLPT=0;
	drec=QHacc::YREC;
	pushEditorRecButton=false;
	memtrans=0;
	selectSize=0;
	selectors=new TableSelect[0];

  QPopupMenu * popup=new QPopupMenu( this );
  popup->insertItem( tr( "insert" ), this, SLOT( newTrans() ) );
  popup->insertItem( tr( "edit" ), this, SLOT( editTrans() ) );
  popup->insertItem( tr( "toggle reco" ), this, SLOT( recTrans() ) );
  popup->insertItem( tr( "delete" ), this, SLOT( remTrans() ) );  

	popup->insertSeparator();
	
	popup->insertItem( tr( "copy" ), this, SLOT( copyTrans() ), CTRL+Key_C );
	popup->insertItem( tr( "cut" ), this, SLOT( cutTrans() ), CTRL+Key_X );
	paster=popup->insertItem( tr( "paste" ), this, SLOT( pasteTrans() ), CTRL+Key_P );
	popup->setItemEnabled( paster, copytrans!=0 );

	popup->insertSeparator();

	mempop=new QPopupMenu( this );
	unmempop=new QPopupMenu( this );
	connect( mempop, SIGNAL( activated( int ) ), SLOT( addMTrans( int ) ) );
	connect( unmempop, SIGNAL( activated( int ) ), SLOT( remMTrans( int ) ) );
	popup->insertItem( tr( "memorize" ), this, SLOT( memTrans() ) );
	popup->insertItem( tr( "memorized" ), mempop );
	popup->insertItem( tr( "unmemorize" ), unmempop );
	popup->insertItem( tr( "write to file" ), this,
										 SLOT( writeTrans() ), CTRL+Key_W );
	//popup->insertItem( "test", this, SLOT( test() ) );

	setPopup( popup );
}

QHaccView::~QHaccView(){
	if( editor ) delete editor;
	if( memtrans ) delete memtrans;
	delete [] selectors;
}

void QHaccView::setSelectors( TableSelect * arr, uint sz ){
	// Note: these selectors are run against the pre-converted
	// dataset, so their criteria should reference QHaccc:T*,
	// not, QHaccView::* constants

	delete [] selectors;
	selectSize=sz;
	selectors=new TableSelect[selectSize];
	for( uint i=0; i<sz; i++ ) selectors[i]=arr[i];
	makeData();
}

void QHaccView::isave() const {
	QString num, str;
  QHeader * head=header();	

	for( int i=0; i<NHEADS; i++ )
		str.append( num.setNum( head->sectionSize( i ) )+" " );
	engine->setSP( "HEADINGSIZES", str );

	engine->setIP( "VIEWSORTCOL", sortCol() );
	engine->setBP( "VIEWSORTASC", sortAsc() );
}

void QHaccView::ireadPrefs( bool initial ){
	data->clear();

	if( initial ){
		connect( engine, SIGNAL( changedP( const QString&, QString ) ),
						 this, SLOT( changeP( const QString&, QString ) ) );
		connect( engine, SIGNAL( changedP( const QString&, int ) ),
						 this, SLOT( changeP( const QString&, int ) ) );
		connect( engine, SIGNAL( changedP( const QString&, bool ) ),
						 this, SLOT( changeP( const QString&, bool ) ) );
		
		connect( engine, SIGNAL( addedT( const Transaction & ) ),
						 this, SLOT( addTrans( const Transaction & ) ) );
		connect( engine, SIGNAL( removedT( const Transaction & ) ),
						 this, SLOT( removeTrans( const Transaction & ) ) );
		connect( engine, SIGNAL( updatedT( const Transaction & ) ),
						 this, SLOT( updateTrans( const Transaction & ) ) );
		
		connect( engine, SIGNAL( removedA( const Account & ) ),
						 this, SLOT( removeA( const Account & ) ) );
		connect( engine, SIGNAL( updatedA( const Account &, const Account& ) ),
						 this, SLOT( updateA( const Account &, const Account& ) ) );
	}

	setLinesPerRow( engine->getBP( "DOUBLEENTRY" ) ? 2 : 1 );
	editorLPT=getLinesPerRow();

  QHeader * head=header();	
	QString ss=engine->getSP( "HEADINGSIZES" );
	if( !ss.isEmpty() ){
		QString vals[NHEADS];
		Utils::parser( ss, " ", 0, vals, NHEADS );
		for( int i=0; i<NHEADS; i++ )	head->resizeSection( i, vals[i].toInt() );
	}

	int indexer=engine->getIP( "VIEWSORTCOL" );
	bool asc=engine->getBP( "VIEWSORTASC" );
	setSorter( indexer, asc );
}

QDate QHaccView::getIUMDate() const {
	Transaction t=getData( getIUM() );
	if( t.isNull() ) return QDate::currentDate();
	else return t.getd( DATE );
}

void QHaccView::grabData( QHaccTable *& ret ){
	std::ostream * str=0;
	if( Utils::debug( 6, str ) ) *str<<"viewer grabbing data!"<<endl;

	ret=new QHaccTable( COLTYPES, COLNAMES, COLS, ID, "viewer table" );

	if( account.isNull() || ledger.isNull() ) return;

	bool findHighNum=( account.gets( QHacc::ADEFAULTNUM ) ==
										 AccountDlg::INCRSTR );
	highNum=( findHighNum ? 1 : 0 );

	
	//TableSelect crit[]={ TableSelect( QHacc::TACCTID,	a.get( QHacc::AID ) ),
	//									 TableSelect( QHacc::TLID, l.get( QHacc::LID ) ) };
	//engine->getTWhere( crit, 2, table1 );


	// we can't do the above getWhere, because we might need to know what the 
	// highNum is, and that should cross ledger boundaries.
	QHaccTable * table1=0;
	engine->getTForA( account, table1 );

	if( findHighNum ){ // auto-incrementing check numbers
		for( uint i=0; i<table1->rows(); i++ ){
			int tnum=table1->at( i ).gets( QHacc::TNUM ).toInt();
			if( tnum>=highNum ) highNum=tnum+1;
		}
	}

	// get transactions just from the current ledger
	QHaccTable * table2=0;
	table1->getWhere( TableSelect( QHacc::TLID, ledger.get( QHacc::LID ) ), 
										table2 );
	delete table1;

	//table1->setName( "ryan" );
	QHaccTable * table=0;
	table2->getWhere( selectors, selectSize, table );


	//cout<<"qv selectors results are:"<<endl;for( uint i=0; i<table->rows(); i++ ){cout<<"  "<<table->at( i ).toString()<<endl;}

	// load the remaining transactions into our view table
	ret->startLoad( table->rows() );
	for( uint i=0; i<table->rows(); i++ )	ret->add( conv( table->at( i ) ) );
	ret->stopLoad();
	
	delete table;

	grabMemData();
}

void QHaccView::grabMemData(){
	// memorized transactions
	if( memtrans ) delete memtrans;
	engine->getMSubsetForA( account, memtrans );
	mempop->clear();
	unmempop->clear();

	uint rows=memtrans->rows();
	for( uint i=0; i<rows; i++ ){
		Transaction t=memtrans->at( i );
		QString str=t.gets( QHacc::TPAYEE )+ " ("+t.gets( QHacc::TSUM )+")";
		mempop->insertItem( str, i );
		unmempop->insertItem( str, i );
	}
}

void QHaccView::setAccount( const Account& a ){
	dropEditor();
	account=a;
	makeData();

	if( !data->isEmpty() ) ensureVisible( 0, getRowSize()*data->rows() );
}

void QHaccView::setLedger( const Ledger& l ){
	//cout<<"view setting ledger to "<<l.gets( QHacc::LNAME )<<endl;
	//dropEditor();
	ledger=l;
	setAccount( account );
	//makeData();
}

void QHaccView::makeIndexFor( int col, QHaccTableIndex *& idx ) const {
	int mycol=col;
	if( col==0 && ( !account.isNull() && account.gets( QHacc::ADEFAULTNUM )==AccountDlg::INCRSTR ) )
		mycol=INUM;
	if( col==4 || col==5 ) mycol=FSUM;
	else if( col==6 ) mycol=RECO;
	data->addIndexOn( mycol, ID, idx );
}

TableRow QHaccView::uconv( const TableRow& old ) const {
	return engine->getT( old.getu( ID ) );
}

TableRow QHaccView::conv( const TableRow& trans ) const { return sconv( trans ); }
TableRow QHaccView::sconv( const TableRow& trans ){
	TableCol tsum=trans.get( QHacc::TSUM );
	TableCol tcs[]={ trans.get( QHacc::TNUM ),
									 trans.get( QHacc::TDATE ),
									 trans.get( QHacc::TPAYEE ),
									 trans.get( QHacc::TMEMO ),
									 tsum,
									 trans.get( QHacc::TRECO ),
									 TableCol( QString() ),
									 trans.get( QHacc::TSPLITGROUP ),
									 trans.get( QHacc::TID ),
									 trans.get( QHacc::TACCTID ),
									 TableCol( QString() ),
									 TableCol( QHacc::convMoney( tsum.gets() ) ),
									 TableCol( trans.gets( QHacc::TNUM ).toInt() ) };
	return Transaction( tcs, COLS );
}

void QHaccView::getPNamesFor( QHacc * eng, uint id,
															QString& pname, QString& pfname ){
	// if split trans, find the pair's parent's name and full name
	if( id==0 ) return;

	Transaction model=eng->getT( id );

	//if( !model ) return; // we might get transactions that haven't be entered yet
	QHaccTable * splits=0;
	eng->getTSplits( model, splits );
	
	uint srows=splits->rows();

	if( srows==1 ){ 
		Account acct=eng->getA( splits->at( 0 ).getu( QHacc::TACCTID ) );
		pname=acct.gets( QHacc::ANAME );
		pfname=eng->getFNameOfA( acct );
	}
	else if( srows>1 ){
		pname=tr( TransactionEditor::SPLIT );
		pfname=pname;
	}
	
	delete splits;
}

void QHaccView::drawRow( QPainter * p, int y, int h, Transaction& trans ){
	QHeader * heads=header();
	bool two=( getLinesPerRow()==2 );
	
	if( trans.isNull() ){
		for( int c=0; c<NHEADS; c++ ){
			int x=heads->sectionPos( c )+1;
			int w=heads->sectionSize( c )-3;
			
			if( c==1 ) p->drawText( x, y, w, h, AlignLeft, "<new>" );
			
			bool draw=true;
			int myh=h;
			if( two ){
				if( c==3 ) draw=false;
				else myh+=h;
			}
			if( draw ) p->drawLine( x-3, y, x-3, y+myh );
		}
		return;
	}

	bool credit=true;
	QString sum=trans.gets( SUM );
	if( sum.startsWith( "-" ) ){
		sum=sum.mid( 1 );
		credit=false;
	}

	QString acctstr=trans.gets( PNAME );
	QString pfname;
	// we really don't care about the splitgroup id, what we really
	// want is the name to print. if we figure out the name to print, 
	// update the record so we don't have to re-figure it out again
	if( getLinesPerRow()>1 && acctstr.isNull() ){
		TableCol id=trans.get( ID );
		getPNamesFor( engine, id.getu(), acctstr, pfname );
		data->updateWhere( ID, id, PNAME, TableCol( acctstr ) );
		data->updateWhere( ID, id, PFNAME, TableCol( pfname ) );
		trans.set( PNAME, acctstr );
		trans.set( PFNAME, pfname );
	}

	for( int c=0; c<NHEADS; c++ ){
		QString str;
		int x=heads->sectionPos( c )+1;
		int w=heads->sectionSize( c )-3;
		int align=AlignLeft;
		int my=y;
		int mw=w;

		switch( c ){
		case 1:
			str=QHaccDateEdit::getDateString( trans.getd( c ) );
			break;
		case 6:
			str=( trans.getu( RECO )>=getDRec() ? "R" : "" );
			align=AlignCenter;
			break;
		case 3:
			if( two ) my+=h;
			str=trans.gets( c );
			break;
		case 4:
			if( credit ) str=sum;
			align=AlignRight;
			break;
		case 5:
			if( !credit ) str=sum;
			align=AlignRight;
			break;
		case 2:
			if( two ){
				p->drawText( x, my+h, mw, h, align, acctstr );
				mw+=heads->sectionSize( c+1 );
			}
			
		default:
			str=trans.gets( c );
			break;
		}
		p->drawText( x, my, mw, h, align, str );
		
		bool draw=true;
		int myh=h;
		if( two ){
			if( c==3 ) draw=false;
			else myh+=h;
		}
		if( draw ) p->drawLine( x-3, y, x-3, y+myh );
	}
}

void QHaccView::changeP( const QString& pref, int ){
	if( pref=="DATEFORMAT" ) refresh();
}

void QHaccView::changeP( const QString& pref, bool b ){
	if( pref=="DOUBLEENTRY" ){
		dropEditor();
		int i=( b ? 2 : 1 );
		editorLPT=i;
		setLinesPerRow( i );
	}
}

void QHaccView::changeP( const QString& pref, QString ){
	if( pref=="DATESEPARATOR" ) refresh();
}

void QHaccView::doSetChosen( int ){ if( !account.isNull() ) editTrans(); }

void QHaccView::keyPressEvent( QKeyEvent * qke ){
	int key=qke->key();
	//cout<<"qv got kre"<<endl;
	if( key==Key_I ) {
		newTrans();
		qke->accept();
	}
	else if( key==Key_E ){
		editTrans();
		qke->accept();
	}
	else if ( key==Key_R ){
		recTrans();
		qke->accept();
	}
	else QHaccScrollView::keyPressEvent( qke );
}

void QHaccView::dropEditor(){
	if( editor ){
		closeEditor();
		delete editor;
		editor=0;
	}
}

void QHaccView::closeEditor(){
	editor->hide();
	setFocus();
}

void QHaccView::openEditor( int y, const Transaction& tr, const QDate date ){
	if( editor==0 ){
		editor=new TransactionEditor( engine, account, ledger, editorLPT>1, this );
		connect( editor, SIGNAL( closed() ), SLOT( closeEditor() ) );
	}

	bool edit=( !tr.isNull() );

	Transaction t;
	if( edit ) t=uconv( tr );
	else { // new transaction
		t=engine->getBlankTForAL( account, ledger );
		
		
		t.set( QHacc::TDATE, TableCol( date ) );
		if( highNum>0 )	t.set( QHacc::TNUM,
													 TableCol( QString::number( highNum ) ) );
	}

	int rs=getRowSize();
	editor->reLayout( header(), visibleWidth(),
										editorLPT>getLinesPerRow() ? rs*editorLPT : rs );
	editor->prepare( t, edit, getDRec(), getEPRec() );
	addChild( editor, 0, y*rs );
	editor->show();
}

void QHaccView::newTrans(){
	Transaction t;
	//cout<<"new trans"<<endl;
	openEditor( getIUM(), t, getIUMDate() );
}

void QHaccView::deleteRow( int i ){
	Transaction mytr=getData( i );
	if( !mytr.isNull() ) engine->removeTSG( mytr.get( SGRP ) );
}

void QHaccView::remTrans(){ deleteRow( getIUM() ); }
//void QHaccView::editTrans(){ cout<<"edit trans"<<endl;openEditor( getIUM(), getData( getIUM() ) ); }
void QHaccView::editTrans(){ openEditor( getIUM(), getData( getIUM() ) ); }
void QHaccView::recTrans(){
	Transaction t=getMoused();
	if( t.isNull() ) newTrans();
	else{
		if( t.getu( QHacc::TRECO )==QHacc::NREC ){
			engine->setTRec( t, getDRec() );
			setIUM( getIUM()+1 );
		}
		else engine->setTRec( t, QHacc::NREC );
	}
}

void QHaccView::addTrans( const Transaction& t ){
	bool add=( t.getu( QHacc::TACCTID )==account.getu( QHacc::AID ) );
	for( uint i=0; i<selectSize; i++ ) add=( add && selectors[i].check( t ) );

	if( add ){
		TableRow newy=conv( t );
		if( highNum>0 ){
			int hn=t.gets( QHacc::TNUM ).toInt();
			if( hn>=highNum ) highNum=hn+1;			
		}
		
		data->add( newy );
		resizer();
	}
}

void QHaccView::removeA( const Account& ){ makeData(); }
//void QHaccView::updateA( const Account& ){ makeData(); }

void QHaccView::updateA( const Account& oldy, const Account& newy ){
	if( account.isNull() ) return;
	if( newy.getu( QHacc::AID )==account.getu( QHacc::AID ) ){
		// if we're dealing with the open account, the default transaction
		// type might have changed, so worry about that
		account=newy;
		highNum=0;
		if( account.gets( QHacc::ADEFAULTNUM )==AccountDlg::INCRSTR ){
			// auto-incrementing check numbers?
			for( uint i=0; i<data->rows(); i++ ){
				int tnum=data->at( i ).gets( NUM ).toInt();
				if( tnum>=highNum ) highNum=tnum+1;
			}
		}
	}
	else{
		// not the open account, but if the fname changed the
		// display must reflect that change--remake the table
		if( oldy.gets( QHacc::ANAME )!=newy.gets( QHacc::ANAME ) ) makeData();
	}
}

void QHaccView::updateTrans( const Transaction& t ){
	// the given trans has been updated, so
	// determine if we need to repaint anything

	// if the trans isn't in our account, there's no way we'll update it
	if( t.getu( QHacc::TACCTID )!=account.getu( QHacc::AID ) ||
			t.getu( QHacc::TLID ) !=ledger.getu( QHacc::LID ) ) return;

	bool upd=true;
	for( uint i=0; i<selectSize; i++ ) upd=( upd && selectors[i].check( t ) );

	QHaccTable * temp=0;
	data->getWhere( TableSelect( ID, t.get( QHacc::TID ) ), temp );

	// if we're supposed to update a row we don't have, remake data
	// to make sure we have the row. else if we have a row we're 
	// update, just update it. else remake data to get rid of a
	// row we're not supposed to have to update
	// (this logic is mainly for the recwin and subset views)
	
	if( temp->isEmpty() ){ if( upd ) makeData(); }
	else{
		Transaction old=temp->at( 0 );
		if( upd )	updateRow( old, conv( t ) );
		else makeData(); 
	}
	delete temp;
}

void QHaccView::removeTrans( const Transaction& t ){
	bool rem=( t.getu( QHacc::TACCTID )==account.getu( QHacc::AID ) );
	for( uint i=0; i<selectSize; i++ ) rem=( rem && selectors[i].check( t ) );
	
	if( rem ){
		data->deleteWhere( ID, t.get( QHacc::TID ) );
		resizer();
	}
}

uint QHaccView::getDRec() const { return drec; }
void QHaccView::setDRec( uint i ) { drec=i; }
bool QHaccView::getEPRec() const { return pushEditorRecButton; }
void QHaccView::setEPRec( bool b ) { pushEditorRecButton=b; }

void QHaccView::remMTrans( int i ){
	engine->memorize( memtrans->at( i ).getu( QHacc::TID ), false );
	grabMemData();
}

void QHaccView::addMTrans( int i ){
	engine->addMtoL( memtrans->at( i ).getu( QHacc::TID ), getIUMDate(),
									 ledger );
}

void QHaccView::memTrans(){
	Transaction t=getMoused();
	if( !t.isNull() ){
		engine->memorize( t.getu( QHacc::TID ), true );
		grabMemData();
	}
}

void QHaccView::cutTrans(){
	imakeCopy();
	remTrans();
}

void QHaccView::copyTrans(){
	imakeCopy();
	copytrans->updateWhere( TableSelect(), QHacc::TDATE, QHacc::XDATE );
}

void QHaccView::imakeCopy(){
	Transaction t=getMoused();
	
	if( copytrans ){ // get rid of the last copytrans
		delete copytrans;
		copytrans=0;
	}

	if( !t.isNull() ){ // keep the copied splitgroup handy
		engine->getTSplits( t, copytrans );
		copytrans->updateWhere( TableSelect(), QHacc::TID, TableCol( ( uint )0 ) );
		copytrans->add( t );
	}
	popup->setItemEnabled( paster, copytrans!=0 );
}

void QHaccView::pasteTrans(){
	if( copytrans ){
		copytrans->updateWhere( TableSelect( QHacc::TID, TableCol( ( uint )0 ),
																				 TableSelect::NE ),
														QHacc::TACCTID,	account.get( QHacc::AID ) );
		QHaccTable * temp=0;
		temp=new QHaccTable( *copytrans );
		if( temp->at( 0 ).getd( QHacc::TDATE )==QHacc::XDATE )
			temp->updateWhere( TableSelect(), QHacc::TDATE,
												 TableCol( getIUMDate() ) );
		temp->updateWhere( TableSelect(), QHacc::TRECO, TableCol( QHacc::NREC ) );
		temp->updateWhere( TableSelect(), QHacc::TLID, ledger.get( QHacc::LID ) );
		engine->addTSG( temp );
		
		delete temp;
	}
}

void QHaccView::writeTrans(){
	Transaction t=getMoused();
	if( !t.isNull() ) emit needTransWritten( t.getu( QHacc::TID ) );
}



/* * * * * * * * * * * * * * * */
/*    QHACCRECVIEW METHODS     */
/* * * * * * * * * * * * * * * */

QHaccRecView::QHaccRecView( QHacc * e, QWidget * p,	const char * n ) 
	: QHaccView( e, p, n ){

	setDRec( QHacc::MREC );
	setEPRec( true );
}

QHaccRecView::~QHaccRecView(){}

void QHaccRecView::getContents( QHaccTable *& ret ) const {
	engine->makeTT( ret );

	ret->startLoad( data->rows() );
	for( uint i=0; i<data->rows(); i++ ) ret->add( uconv( getData( i ) ) );
	ret->stopLoad();

	QHaccTableIndex * junk=0;
	// we'll pretty much always want an index on dates
	ret->addIndexOn( QHacc::TDATE, junk );
}

void QHaccRecView::save() const {
	QString num, str;
  QHeader * head=header();	
	
	for( int i=0; i<NHEADS; i++ )
		str.append( num.setNum( head->sectionSize( i ) )+" " );
	engine->setSP( "RECHEADINGSIZES", str );
	
	engine->setIP( "RECVIEWSORTCOL", sortCol() );
	engine->setBP( "RECVIEWSORTASC", sortAsc() );
}

void QHaccRecView::readPrefs( bool initial ){
	QHaccView::readPrefs( initial );

	setLinesPerRow( 1 );
	editorLPT=( engine->getBP( "DOUBLEENTRY" ) ? 2 : 1 );
  QHeader * head=header();	
	QString ss=engine->getSP( "RECHEADINGSIZES" );
	if( !ss.isEmpty() ){
		QString vals[NHEADS];
		Utils::parser( ss, " ", 0, vals, NHEADS );
		for( int i=0; i<NHEADS; i++ )	head->resizeSection( i, vals[i].toInt() );
	}
	
	int indexer=engine->getIP( "RECVIEWSORTCOL" );
	bool asc=engine->getBP( "RECVIEWSORTASC" );
	setSorter( indexer, asc );
}

void QHaccRecView::doSetChosen( int ){ recTrans(); }
void QHaccRecView::setLimit( const QDate& d ){ limit=d; }
QDate QHaccRecView::getIUMDate() const {
	QDate d=QHaccView::getIUMDate();
	if( d>limit ) return limit;
	else return d;
}
