/************************** * * * * * * * * * * * * ***************************
    Copyright (c) 1999-2003 Ryan Bobko
                       ryan@ostrich-emulators.com

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.     
*************************** * * * * * * * * * * * * **************************/

#ifndef _QHACCVIEW_H
#define _QHACCVIEW_H

#include "qhacc.h"
#include "qhaccscrollview.h"

#include <qcolor.h>
#include <qdatetime.h>

class QHacc;
class TransactionEditor;

class QHaccView : public QHaccScrollView {
  Q_OBJECT
    public:
	friend class TransactionEditor;

  QHaccView( QHacc *, QWidget * =0, const char * =0 );
  virtual ~QHaccView();

	void setSelectors( TableSelect *, uint size );

 protected:
	void drawRow( QPainter *, int y, int h, Transaction& row );
	void deleteRow( int );
	virtual void makeIndexFor( int, QHaccTableIndex *& ) const;
	virtual void  doSetChosen( int );
	static TableRow sconv( const TableRow& );
	TableRow conv( const TableRow& ) const;
	TableRow uconv( const TableRow& ) const;
	void keyPressEvent( QKeyEvent * );
	virtual void grabData( QHaccTable *& );
	uint getDRec() const;
	void setDRec( uint i );
	bool getEPRec() const;
	void setEPRec( bool );
	static void getPNamesFor( QHacc *, uint, QString& pn, QString& pfn );
	virtual QDate getIUMDate() const;
	void dropEditor();

	void imakeCopy();
	void isave() const;
	void ireadPrefs( bool );

	int editorLPT;
	Account account;
	Ledger ledger;
	TableSelect * selectors;
	uint selectSize;

	static const int ID, NUM, DATE, PAYEE, MEMO, SUM, RECO, PNAME,
		SGRP, AID, PFNAME, FSUM, INUM, COLS;
	static const char * COLNAMES[];
	static const ColType COLTYPES[];


 private:
	static const int NHEADS;
	static const char * HEADS[];

	void openEditor( int y, const Transaction&,
									 const QDate =QDate::currentDate() );
	void grabMemData();

	TransactionEditor * editor;
	int highNum, paster;
	uint drec;
	QPopupMenu * mempop, * unmempop;
	QHaccTable * memtrans;
	static QHaccTable * copytrans;
	bool pushEditorRecButton;

	public slots:
		void setAccount( const Account& );
		void setLedger( const Ledger& );
		
	protected slots:
		void newTrans();
	void remTrans();
	void recTrans();		
	
	private slots:
		void changeP( const QString&, int );
	void changeP( const QString&, QString );
	void changeP( const QString&, bool );
	
	//public slots:
	void closeEditor();
	virtual void editTrans();

	void addTrans( const Transaction& );
	void updateTrans( const Transaction& );
	void removeTrans( const Transaction& );

	void memTrans(); // memorize a transaction
	void addMTrans( int ); // add a previously-memorized transaction
	void remMTrans( int ); // remove a previously-memorized transaction

	void cutTrans();
	void copyTrans();
	void pasteTrans();
	void writeTrans();

	void removeA( const Account& );
	void updateA( const Account&, const Account& );

 signals:
  void needTransWritten( uint );
};

class QHaccRecView : public QHaccView {
  Q_OBJECT
    public:
  QHaccRecView( QHacc *, QWidget * =0, const char * =0 );
  virtual ~QHaccRecView();
  void getContents( QHaccTable *& ) const;
	
	void save() const;
	void readPrefs( bool );
	void setLimit( const QDate& );
	
 protected:
	void doSetChosen( int );
	QDate getIUMDate() const;
	
 private:
  QDate limit;
};

#endif
