/************************* * * * * * * * * * * * * ***************************
    Copyright (c) 1999-2003 Ryan Bobko
                       ryan@ostrich-emulators.com

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.     
************************** * * * * * * * * * * * * **************************/

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include "qhaccview.h"
#include "qhaccutils.h"
#include "qhaccgrwin.h"
#include "qhaccwidget.h"
#include "qhaccrecwin.h"
#include "qhaccdialogs.h"
#include "qhaccledchsr.h"
#include "qhaccacctchsr.h"
#include "qhaccsubsetwin.h"
#include "qhaccwidget.moc"

#include "pixmaps/graph.xpm"
#include "pixmaps/report.xpm"
#include "pixmaps/prefer.xpm"
#include "pixmaps/resort.xpm"
#include "pixmaps/subset.xpm"
#include "pixmaps/recacct.xpm"
#include "pixmaps/newacct.xpm"
#include "pixmaps/editacct.xpm"
#include "pixmaps/filesave.xpm"

#include <qfile.h>
#include <qlabel.h>
#include <qaction.h>
#include <qlayout.h>
#include <qtoolbar.h>
#include <qmenubar.h>
#include <qcheckbox.h>
#include <qstatusbar.h>
#include <qpopupmenu.h>
#include <qfiledialog.h>
#include <qtoolbutton.h>
#include <qmessagebox.h>
#include <qapplication.h>
#include <qinputdialog.h>

QHaccWidget::QHaccWidget( QHacc * qhacc, QWidget * p, const char * n )
  : QMainWindow( p, n, WDestructiveClose ){

  tools=0;
	engine=qhacc;
	
	// put something in the middle of this window
	widget=new QHaccCenter( engine, this );
	setCentralWidget( widget );

	connect( engine, SIGNAL( updatedA( const Account&, const Account& ) ),
					 this, SLOT( updateAccount( const Account&, const Account& ) ) );

	connect( engine, SIGNAL( changedP( const QString&, QFont ) ),
					 SLOT( changePref( const QString&, QFont ) ) );
	
	connect( engine, SIGNAL( changedP( const QString&, QString ) ),
					 SLOT( changePref( const QString&, QString ) ) );

	connect( engine, SIGNAL( changedP( const QString&, bool ) ),
					 SLOT( changePref( const QString&, bool ) ) );

	connect( engine, SIGNAL( needSave( bool ) ), SLOT( setSaveFlag( bool ) ) );
	
	connect( engine, SIGNAL( overBudget( const Account&, int ) ),
					 this, SLOT( OBAccount( const Account&, int ) ) );

	connect( widget, SIGNAL( needTransWritten( uint ) ),
					 SLOT( writeTrans(  uint ) ) );

	connect( widget, SIGNAL( changedAccount( const Account& ) ),
					 this, SLOT( setAccount( const Account& ) ) );

	connect( widget, SIGNAL( needReportOf( const Account& ) ), 
					 this, SLOT( report( const Account& ) ) );
	connect( widget, SIGNAL( needGraphOf( const Account& ) ), 
					 this, SLOT( chart( const Account& ) ) );
	connect( widget, SIGNAL( needSubsetOf( const Account& ) ), 
					 this, SLOT( subset( const Account& ) ) );
	connect( widget, SIGNAL( needRecWinOf( const Account& ) ), 
					 this, SLOT( reconcile( const Account& ) ) );

	QApplication::setFont( qhacc->getWP( "FONT" ), true );

	// build the status bar
  QStatusBar * status=statusBar();
  abalance=new QLabel( status );
  saveflag=new QLabel( status );
	saveflag->setAlignment( AlignCenter );
  tcount=new QLabel( status );
  msgs=new QLabel( status );
	
  abalance->setAlignment( AlignRight );

  status->addWidget( msgs, 4, true );
  status->addWidget( saveflag, 2, true );
  status->addWidget( tcount, 8, true );
  status->addWidget( abalance, 5, true );


	// build the toolbar
	tools=new QToolBar( this, "toolbar" );

	// build the menubar
	QPopupMenu * file=new QPopupMenu( this );
	QPopupMenu * help=new QPopupMenu( this );
	QPopupMenu * prefm=new QPopupMenu( this );
	QPopupMenu * acct=new QPopupMenu( this );
	QPopupMenu * ledg=new QPopupMenu( this );
	QPopupMenu * graphs=new QPopupMenu( this );
	
	QMenuBar * menu=menuBar();
	menu->insertItem( tr( "&File" ), file );
	menu->insertItem( tr( "&Account" ), acct );
	menu->insertItem( tr( "&Ledger" ), ledg );
	menu->insertItem( tr( "&Preferences" ), prefm );
	menu->insertItem( tr( "&Graphs" ), graphs );
	menu->insertSeparator();
	menu->insertItem( tr( "&Help" ), help );

	//QPixmap reconcile( recacct );
	//QPixmap create( newacct );
	//QPixmap edit( editacct );
	//QPixmap prefer( prefers );
	//QPixmap resorty( resorter );
	//QPixmap subset( subsetter );

	// these are the actions we'll perform
	QAction * save=new QAction( tr( "Save Now" ), QPixmap( filesave ),
															tr( "&Save Now" ), CTRL+Key_S, this );
	connect( save, SIGNAL( activated() ), this, SLOT( save() ) );

	QAction * home=new QAction( tr( "Set QHacc Home" ), QPixmap(),
															tr( "Q&Hacc Home..." ), CTRL+Key_H, this );
	connect( home, SIGNAL( activated() ), this, SLOT( reload() ) );

	QAction * ex=new QAction( tr( "Exit" ), QPixmap(),
														tr( "E&xit" ), CTRL+Key_T, this );
	connect( ex, SIGNAL( activated() ), this, SLOT( saveAndQuit() ) );
	
	QAction * quit=new QAction( tr( "Quit" ), QPixmap(),
															tr( "&Quit" ), CTRL+Key_Q, this );
	connect( quit, SIGNAL( activated() ), this, SLOT( close() ) );

	save->addTo( file );
	home->addTo( file );
	ex->addTo( file );
	quit->addTo( file );

	//file->insertItem( tr( "&Save Now" ), this, SLOT( save() ), CTRL+Key_S );
	//file->insertItem( tr( "Q&Hacc Home..." ), this,
	//								SLOT( reload() ), CTRL+Key_H );
	//file->insertItem( tr( "E&xit" ), this, SLOT( saveAndQuit() ), CTRL+Key_X );
	//file->insertItem( tr( "&Quit" ), this, SLOT( close() ), CTRL+Key_Q );

	help->insertItem( tr( "About QHacc" ), this, SLOT( about() ) );
	help->insertSeparator();
	help->insertItem( tr( "About QT" ), this, SLOT( aboutQt() ) );
	
	QAction * cp=new QAction( tr( "Change Prefs" ), QPixmap( prefers ),
														tr( "Change &Prefs" ), CTRL+Key_F, this );
	connect( cp, SIGNAL( activated() ), this, SLOT( changePrefs() ) );
	
	QAction * ss=new QAction( this );
	ss->setText( "Switch Sides" );
	ss->setMenuText( "&Switch Sides" );
	ss->setIconSet( QPixmap( resorter ) );
	connect( ss, SIGNAL( activated() ), widget, SLOT( switchSides() ) );

	QAction * rb=new QAction( tr( "Show Reconciled Balance" ), QPixmap(),
														tr( "Show Reconci&led Balance" ), CTRL+Key_L,
														this, 0, true );
	rb->setOn( engine->getBP( "SHOWRECBAL" ) );
	connect( rb, SIGNAL( activated() ), this, SLOT( toggleRecBal() ) );

	cp->addTo( prefm );
	ss->addTo( prefm );
	rb->addTo( prefm );
	//prefm->insertItem( tr( "Change &Prefs" ), this, SLOT( changePrefs() ) );
	//prefm->insertItem( tr( "&Switch Sides" ), widget, SLOT( switchSides() ) );
	//prefm->insertItem( tr( "Show Reconciled &Balance" ),
	//								 this, SLOT( toggleRecBal() ), CTRL+Key_L );
	
	QAction * rc=new QAction( tr( "Reconcile" ), QPixmap( recacct ),
														tr( "&Reconcile" ), CTRL+Key_R, this );
	connect( rc, SIGNAL( activated() ), this, SLOT( reconcile() ) );

	QAction * vs=new QAction( tr( "View Subset" ), QPixmap( subsetter ),
														tr( "View Su&bset" ), CTRL+Key_B, this );
	connect( vs, SIGNAL( activated() ), this, SLOT( subset() ) );
	
	QAction * cr=new QAction( tr( "Create Account" ), QPixmap( newacct ),
														tr( "&Create..." ), CTRL+Key_N, this );
	connect( cr, SIGNAL( activated() ), this, SLOT( newAcct() ) );

	QAction * dl=new QAction( tr( "Delete Account" ), QPixmap(),
														tr( "&Delete" ), CTRL+SHIFT+Key_X, this );
	connect( dl, SIGNAL( activated() ), widget, SLOT( removeOpened() ) );

	QAction * ed=new QAction( tr( "Edit Account" ), QPixmap( editacct ),
														tr( "&Edit..." ), CTRL+Key_E, this );
	connect( ed, SIGNAL( activated() ), this, SLOT( editAcct() ) );

	rc->addTo( acct );
	vs->addTo( acct );
	cr->addTo( acct );
	dl->addTo( acct );
	ed->addTo( acct );
	


	QAction * lc=new QAction( tr( "Create Ledger" ), QPixmap(),
														tr( "&Create..." ), CTRL+ALT+Key_N, this );
	connect( lc, SIGNAL( activated() ), this, SLOT( newLed() ) );

	QAction * ll=new QAction( tr( "Delete Ledger" ), QPixmap(),
														tr( "&Delete" ), CTRL+SHIFT+ALT+Key_X, this );
	connect( ll, SIGNAL( activated() ), this, SLOT( remLed() ) );

	QAction * ld=new QAction( tr( "Edit Ledger" ), QPixmap(),
														tr( "&Edit..." ), CTRL+ALT+Key_E, this );
	connect( ld, SIGNAL( activated() ), this, SLOT( editLed() ) );

	lc->addTo( ledg );
	ll->addTo( ledg );
	ld->addTo( ledg );

	//acct->insertItem( tr( "&Reconcile" ), this, SLOT( reconcile() ) );
	//acct->insertItem( tr( "View Su&bset" ), this, SLOT( subsetView() ) );
	//acct->insertItem( tr( "&Create" ), this, SLOT( newAcct() ) );
	//acct->insertItem( tr( "&Delete" ), widget, SLOT( removeOpened() ) );
	//acct->insertItem( tr( "&Edit" ), this, SLOT( editAcct() ) );

	QAction * gr=new QAction( tr( "Graph Account..." ), QPixmap( grapher ),
														tr( "&Graphs" ), CTRL+Key_G, this );
	connect( gr, SIGNAL( activated() ), this, SLOT( chart() ) );
 
	QAction * rp=new QAction( tr( "Report on Account..." ), QPixmap( reporter ),
														tr( "&Reports" ), CTRL+Key_Y, this );
	connect( rp, SIGNAL( activated() ), this, SLOT( report() ) );

	gr->addTo( graphs );
	rp->addTo( graphs );
	
	//graphs->insertItem( tr( "&Graphs" ), this, SLOT( chart() ) );
	//graphs->insertItem( tr( "&Reports" ), this, SLOT( report() ) );
	

	save->addTo( tools );
	tools->addSeparator();
	rc->addTo( tools );
	cr->addTo( tools );
	ed->addTo( tools );
	tools->addSeparator();
	gr->addTo( tools );
	rp->addTo( tools );
	tools->addSeparator();
	cp->addTo( tools );
	ss->addTo( tools );
	vs->addTo( tools );


	//new QToolButton( saveIcon, tr( "Save Accounts" ), QString::null,
	//							 this, SLOT( save() ), tools );
		
	//new QToolButton( reconcile, tr( "Reconcile Account" ), QString::null,
	//							 this, SLOT( reconcile() ), tools );
	//new QToolButton( create, tr( "Create Account" ), QString::null,
	//							 this, SLOT( newAcct() ), tools );
	//new QToolButton( edit, tr( "Edit Account" ), QString::null,
	//							 this, SLOT( editAcct() ), tools );
		
	//QPixmap graphy( grapher );
	//QPixmap reporty( reporter );
	
	//new QToolButton( graphy, tr( "Graph Account" ), QString::null,
		//							 this, SLOT( chart() ), tools );
		//new QToolButton( reporty, tr( "Account Report" ), QString::null,
		//						 this, SLOT( report() ), tools );
		//tools->addSeparator();
	
	//new QToolButton( prefer, tr( "Preferences" ), QString::null,
	//							 this, SLOT( changePrefs() ), tools );	
	//new QToolButton( resorty, tr( "Switch Sides" ), QString::null,
	//							 widget, SLOT( switchSides() ), tools );
	//new QToolButton( subset, tr( "View Subset" ), QString::null,
	//							 this, SLOT( subsetView() ), tools );

	// make changes based on preferences
	readPrefs( true );
}

QHaccWidget::~QHaccWidget(){}

void QHaccWidget::readPrefs( bool initialLoad ){
	// now that everything's built, customize based on prefs
	setCaption( tr( engine->getSP( "TITLEBARNAME" ) ) );
	setFont( engine->getWP( "FONT" ) );
	moveToolBar( tools, ( ToolBarDock )( engine->getIP( "TOOLBAR" ) ) );
	
	// if we don't have a pre-saved location, start with a reasonable size
	if( initialLoad ){
		QString pref=engine->getSP( "LOCATION" );	
		if( pref.isEmpty() ) resize( 800, 600 );
		else{
			QString locs[4];
			Utils::parser( pref, " ", 0, locs, 4 );
			setGeometry( locs[0].toInt(), locs[1].toInt(),
									 locs[2].toInt(), locs[3].toInt() );
		}
	}
	widget->readPrefs( initialLoad );
}

void QHaccWidget::setAccount( const Account& acct ){
	account=acct;
	updateAccount( acct, acct );
}

void QHaccWidget::updateAccount( const Account&, const Account& newy ){
	if( account.compareTo( newy, QHacc::AID )!=0 ) return;

	int bal=0;
	int count=0;
	QString type, name, temp;

	if( !newy.isNull() ){
		bal=engine->convMoney( newy.gets( QHacc::ACBAL ) );
		QHaccTable * temp=0;
		engine->getTForA( newy, temp );
    count=temp->rows(); 
		delete temp;

		type=tr( engine->getTypeNameOfA( newy ) );
		name=engine->getFNameOfA( newy );
  }
	else name=QT_TR_NOOP( "No Account" );
	
	QString s=QT_TR_NOOP( "Balance " );
	if ( bal<0 ) {
		s.append ( "-" );
		bal=0-bal;
	}

	s.append( engine->getSP( "CSYMBOL" ) );
	s.append( engine->convMoney( bal ) );

	if( engine->getBP( "SHOWRECBAL" ) ){
		int rbal=engine->convMoney( newy.gets( QHacc::ARBAL ) );
		s.append( " / " );
		
		if ( rbal<0 ) {
			s.append ( "-" );
			rbal=0-rbal;
		}
		
		s.append( engine->getSP( "CSYMBOL" ) );
		s.append( engine->convMoney( rbal ) );
	}

	abalance->setText( s );
	
		
	// update the number of transactions
	// every time you update the balance
	tcount->setText( tr( name+": " )+temp.setNum( count )+" "+tr( type )+
	 								 tr( ( count==1 ? " Transaction" : " Transactions" ) ) );
}

void QHaccWidget::OBAccount( const Account& acct, int obBal ){
	QMessageBox::information( this, tr( "Account Out of Budget" ),
														engine->getFNameOfA( acct )+
														" has exceeded its budget by "+
														engine->getSP( "CSYMBOL" )+
														engine->convMoney( obBal ) );
}

void QHaccWidget::aboutQt(){ QMessageBox::aboutQt( this, tr( "About QT" ) );}
void QHaccWidget::about(){
  QString vline;
  vline.sprintf( tr( "The Q Home Accountant v%s\nWritten by: Ryan Bobko\nryan@ostrich-emulators.com" ), VERSION );
  QMessageBox::about( this, tr( "About QHacc" ), vline );
}

void QHaccWidget::chart(){ chart( widget->getOpenAccount() ); }
void QHaccWidget::chart( const Account& acct ){
  if ( !acct.isNull() ){
		QHaccGRDialog * grd=new QHaccGRDialog( engine, QHaccGRDialog::GRAPH, 
																					 acct, widget->getOpenLedger(),
																					 this );
		grd->readPrefs( true );
		grd->show();
	}
}

void QHaccWidget::report(){ report( widget->getOpenAccount() ); }
void QHaccWidget::report( const Account& acct ){
	if( !acct.isNull() ){
		QHaccGRDialog * grd=new QHaccGRDialog( engine, QHaccGRDialog::REPORT,
																					 acct, widget->getOpenLedger(),
																					 this );
		grd->readPrefs( true );
		grd->show();
	}
}

void QHaccWidget::changePref( const QString& s, bool ){
	if( s=="SHOWRECBAL" ){
		Account a=widget->getOpenAccount();
		updateAccount( a, a );
	}
}

void QHaccWidget::changePref( const QString&, QFont f ){
	setFont( f );
	QApplication::setFont( f, true );
}

void QHaccWidget::changePref( const QString& s, QString v ){
	if( s=="TITLEBARNAME" ) setCaption( v );
	else if( s=="CSYMBOL" ){
		Account a=widget->getOpenAccount();
		updateAccount( a, a );
	}
}

void QHaccWidget::changePrefs(){
	( new PrefsDlg( engine, 0, this ) )->show();
}

void QHaccWidget::saveAndQuit(){
  hide();
  save();
  close();
}

void QHaccWidget::save(){
	QString temp;

	QRect recto=geometry();
	QString val=temp.setNum( recto.x() )+" "+temp.setNum( recto.y() )
		+" "+temp.setNum( recto.width() )+" "+temp.setNum( recto.height() );
	engine->setSP( "LOCATION", val );

	ToolBarDock tbd;
	int tbl;
	bool nl;
	int extra;
	getLocation( tools, tbd, tbl, nl, extra );
	engine->setIP( "TOOLBAR", ( int )tbd );
	
	widget->save();
	
	if( !engine->save( temp ) ) msgs->setText( temp );
	else msgs->setText( "" );
}

void QHaccWidget::setSaveFlag( bool b ){
	saveflag->setText( b ? "save" : "" );
}

void QHaccWidget::reload(){
	QString home;
	bool cont=true;
	bool localhome=engine->homeIsLocal();
	QString title=tr( "Select New QHacc Home" );

	if( !localhome || engine->getBP( "SIMPLEFILEDIALOG" ) ){
	  bool ok = FALSE;
		home=QInputDialog::getText( tr( "QHacc" ), title,	QLineEdit::Normal,
																engine->getHome(), &ok, this );
		if ( !ok || home.isEmpty() ) cont=false;
	}
	else{
		if( localhome )	home=engine->getHome();
		else home=QDir::homeDirPath();
		
		QFileDialog fd( home, 0, this, 0, true );
		fd.setCaption( title );
		fd.setMode( QFileDialog::DirectoryOnly );
		
		fd.setShowHiddenFiles( true );
		if ( fd.exec()==QDialog::Accepted )	home=fd.selectedFile();
		else cont=false;
	}
    
	if( cont ){
		std::ostream * str=0;
		if( Utils::debug( 3, str ) )
			*str<<tr( "Changing QHACC_HOME to " )<<home<<endl;
		
		engine->setHome( home );
		readPrefs( false );
	}
}

void QHaccWidget::reconcile(){ reconcile( widget->getOpenAccount() ); }
void QHaccWidget::reconcile( const Account& a ){
	if( !a.isNull() ){
		QHaccRecWin  * win=new QHaccRecWin( engine, widget->getOpenLedger() );
		win->show();
		win->setAccount( a );
	}
}

void QHaccWidget::writeTrans( uint tid ) {
  /* 
   * This function will take the transaction, and write it to
   * a file so it could later be added to cron. This way,
   * scheduled transactions don't have to be written by hand
   * each time
   */

  QFileDialog fd( engine->getHome(), 0, this, 0, true );
  fd.setCaption( tr( "Select Save File" ) );
  fd.setMode( QFileDialog::AnyFile );
  fd.setShowHiddenFiles( true );
  if ( fd.exec()==QDialog::Accepted ){
    QString path=fd.selectedFile();
		
    if ( !path.isEmpty() ){
			QString error;
			if( engine->writeTransToFile( tid, path, error ) )
				msgs->setText( tr( "transaction written" ) );
			else msgs->setText( tr( error ) );
		}
	}
}

void QHaccWidget::newAcct(){
	Account acct;
  ( new AccountDlg( engine, acct, this ) )->show();
}

void QHaccWidget::editAcct(){
	Account a=widget->getOpenAccount();
	if( a.isNull() ) newAcct();
	else ( new AccountDlg( engine, a, this ) )->show();
}

void QHaccWidget::subset(){ subset( widget->getOpenAccount() ); }
void QHaccWidget::subset( const Account& a ){
	if( !a.isNull() ){
		QHaccSubsetWin  * win=new QHaccSubsetWin( engine );
		win->show();
		win->setAL( a, widget->getOpenLedger() );
	}
}

void QHaccWidget::toggleRecBal(){ 
	engine->setBP( "SHOWRECBAL", !engine->getBP( "SHOWRECBAL" ) );
}

void QHaccWidget::newLed(){
	( new LedgerDlg( engine, Ledger(), this ) )->show();
}
void QHaccWidget::remLed(){
	LedgerDlg::qremove( engine, widget->getOpenLedger(), this );
}
void QHaccWidget::editLed(){
	( new LedgerDlg( engine, widget->getOpenLedger(), this ) )->show();
}

/*****************************************************************************/
/* QHaccCenter: a widget to hold the viewer and the chooser and nothing else */
/*****************************************************************************/
QHaccCenter::QHaccCenter( QHacc * eng, QWidget * p, const char * n ) 
	: QSplitter( p, n ){

	engine=eng;
	chooser=new QHaccAccountChooser( eng, this );
	
	viewerSplit=new QSplitter( Vertical, this );
	ledger=new QHaccLedgerChooser( eng, viewerSplit );
	viewer=new QHaccView( eng, viewerSplit );

	connect( chooser, SIGNAL( changedAccount( const Account& ) ), 
					 viewer, SLOT( setAccount( const Account& ) ) );
	connect( chooser, SIGNAL( changedAccount( const Account& ) ), 
					 this, SLOT( newAccount( const Account& ) ) );
	connect( chooser, SIGNAL( needReportOf( const Account& ) ), 
					 this, SLOT( report( const Account& ) ) );
	connect( chooser, SIGNAL( needGraphOf( const Account& ) ), 
					 this, SLOT( graph( const Account& ) ) );
	connect( chooser, SIGNAL( needSubsetOf( const Account& ) ), 
					 this, SLOT( subset( const Account& ) ) );
	connect( chooser, SIGNAL( needRecWinOf( const Account& ) ), 
					 this, SLOT( reconcile( const Account& ) ) );
	


	connect( ledger, SIGNAL( changedLedger( const Ledger& ) ), 
					 viewer, SLOT( setLedger( const Ledger& ) ) );

	connect( viewer, SIGNAL( needTransWritten( uint ) ),
					 SLOT( writeTrans( uint ) ) );
}

QHaccCenter::~QHaccCenter(){}

void QHaccCenter::writeTrans( uint i ){ emit needTransWritten( i ); }
void QHaccCenter::newAccount( const Account& a ) {	emit changedAccount( a ); }
Ledger QHaccCenter::getOpenLedger() const { return ledger->getSelected(); }
Account QHaccCenter::getOpenAccount() const { return chooser->getSelected(); }
Account QHaccCenter::getHighlightedAccount() const {
	return chooser->getMoused();
}

void QHaccCenter::save() const {
	engine->setBP( "RIGHTCHOOSER", rightChooser );

	QValueList <int> listo=sizes();
	QString num;
	int i=listo.first();
	int j=width()-i;
	engine->setSP( "SPLITTERSIZE", num.setNum( i )+" "+num.setNum( j ) );
	
	chooser->save();
	viewer->save();
	ledger->save();
}

void QHaccCenter::readPrefs( bool initial ){
	rightChooser=engine->getBP( "RIGHTCHOOSER" );
	if( rightChooser ) moveToFirst( viewer );

	if( initial ){
		QValueList <int> sizes;
		sizes.append( ledger->minimumSize().height() );
		//sizes.append( height()-ledger->minimumSize().height() );
		viewerSplit->setSizes( sizes );

		connect( engine, SIGNAL( changedP( const QString&, bool ) ),
						 this, SLOT( changeP( const QString&, bool ) ) );
	}
	changeP( "HIDELEDGERS", engine->getBP( "HIDELEDGERS" ) );

	QString ss=engine->getSP( "SPLITTERSIZE" );
	if( !ss.isEmpty() ){
		QString vals[2];
		Utils::parser( ss, " ", 0, vals, 2 );

		QValueList <int> sizes;
		sizes.append( vals[0].toInt() );
		sizes.append( vals[1].toInt() );
		setSizes( sizes );
	}

	viewer->readPrefs( initial );
	chooser->readPrefs( initial );
	ledger->readPrefs( initial );

	viewer->setFocus();
}

void QHaccCenter::switchSides(){
	if( rightChooser ) moveToFirst( chooser );
	else moveToFirst( viewerSplit );
	rightChooser=!rightChooser;
	refresh();
}

void QHaccCenter::removeOpened(){
	AccountDlg::qremove( engine, chooser->getSelected() );
}

void QHaccCenter::changeP( const QString& pref, bool b ){
	if( pref=="HIDELEDGERS" ){
		if( b ) ledger->hide();
		else ledger->show();
	}
}

void QHaccCenter::reconcile( const Account& a ){ emit needRecWinOf( a ); }
void QHaccCenter::subset( const Account& a ){ emit needSubsetOf( a ); }
void QHaccCenter::report( const Account& a ){ emit needReportOf( a ); }
void QHaccCenter::graph( const Account& a ){ emit needGraphOf( a ); }
