/************************* * * * * * * * * * * * * ***************************
    Copyright (c) 1999-2003 Ryan Bobko
                       ryan@ostrich-emulators.com

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.     
************************** * * * * * * * * * * * * **************************/

#ifndef _QHACCWIDGET_H
#define _QHACCWIDGET_H

#include "qhacc.h"

#include <qtabbar.h>
#include <qstrlist.h>
#include <qsplitter.h>
#include <qmainwindow.h>

class QHaccView;
class QHaccLedgerChooser;
class QHaccAccountChooser;

class QLabel;
class QToolBar;
class QCheckBox;
class QComboBox;
class QPopupMenu;


/************************************/
/***** QHaccCenter definitions ******/
/************************************/

class QHaccCenter : public QSplitter {
  Q_OBJECT
    public:
  QHaccCenter( QHacc *, QWidget * =0, const char * =0 );
  ~QHaccCenter();
	Account getOpenAccount() const;
	Account getHighlightedAccount() const ;
	Ledger getOpenLedger() const;

	void save() const;
	void readPrefs( bool );

 private:
	QHacc * engine;
	bool rightChooser;
  QHaccView * viewer;
  QHaccAccountChooser * chooser;  
	QHaccLedgerChooser * ledger;
	QSplitter * viewerSplit;

	public slots:
		void switchSides();
	void newAccount( const Account& );
	void removeOpened();
	void writeTrans( uint );
	void changeP( const QString&, bool );
  void report( const Account& );
  void graph( const Account& );
	void subset( const Account& );
	void reconcile( const Account& );

 signals:
	void changedAccount( const Account& );
	void needReportOf( const Account& );
	void needGraphOf( const Account& );
	void needSubsetOf( const Account& );
	void needRecWinOf( const Account& );
	void needTransWritten( uint );
};

/******************************************/
/******** QHaccWidget definitions *********/
/******************************************/

class QHaccWidget : public QMainWindow {
  Q_OBJECT
    public:
  QHaccWidget( QHacc *, QWidget * =0, const char * =0 );
  ~QHaccWidget();
	
	void save() const;

 private:
	QHacc * engine;
	Account account;
  QHaccCenter * widget;
  QToolBar * tools;
  QLabel * abalance, * tcount, * msgs, * saveflag;

	void readPrefs( bool firstTime );


  public slots:
    void saveAndQuit();
  void writeTrans( uint );
  void setAccount( const Account& );
  void updateAccount( const Account&, const Account& );
	void changePref( const QString&, QString );
	void changePref( const QString&, QFont );
	void changePref( const QString&, bool );
	void OBAccount( const Account&, int );
	void setSaveFlag( bool );
	void toggleRecBal();
	void changePrefs();
  void reconcile();
  void editAcct();
	void newAcct();
  void aboutQt();
  void reload();
	void subset();
  void report();
  void chart();
  void report( const Account& );
  void chart( const Account& );
	void subset( const Account& );
	void reconcile( const Account& );
  void about();
  void save();

	void newLed();
	void remLed();
	void editLed();
};

#endif
