/************************* * * * * * * * * * * * * ***************************
    Copyright (c) 1999-2003 Ryan Bobko
                       ryan@ostrich-emulators.com

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.     
************************** * * * * * * * * * * * * **************************/

#ifndef _TRANSEDITOR_H
#define _TRANSEDITOR_H

#include "qhacc.h"
#include "qhacclineedits.h"

#include <qlist.h>
#include <qframe.h>
#include <qdialog.h>
#include <qdatetime.h>
#include <qscrollview.h>
#include <qstringlist.h>

class AccountArea;
class QHaccLineEdit;
class QHaccChoiceEdit;
class QHaccDateEdit;


class QResizeEvent;
class QPushButton;
class QCloseEvent;
class QCheckBox;
class QPainter;
class QHeader;

class TransactionEditor : public QFrame {
  Q_OBJECT
    public:
	friend class AccountArea;
	static const int SACCT, SSUM, SCOLS;
	static const char * SCOLNAMES[], * SPLIT, * DEATEXT, * DEP, * WD;
	static const ColType SCOLTYPES[];

  TransactionEditor( QHacc *, const Account&, const Ledger&, bool, QWidget *,
										 const char * =0 );
  ~TransactionEditor();
	
  void prepare( const Transaction&, bool forEdit, uint recostate,
								bool recpushed );
  void closeEvent( QCloseEvent * );
  void reLayout( QHeader *, int w, int h );

 protected:
  QHaccChoiceEdit * num, * payee, * credit, * debit, * deAccount;
  QHaccLineEdit * memo;
  QHaccDateEdit * date;
  QPushButton * split, * okay;
  QCheckBox * reco;
	QHacc * engine;
  Transaction mytrans;
  Account account;
	Ledger ledger;
	bool doubleEntry, edit, connected;
	uint recstate;
	int modelpos, modelstart, modelend;
	QHaccTable * tsplits;
	QHaccTable * atrans;
	QHaccTableIndex * apidx;

	void makePayees();
	void setModel( const Transaction& );
	static bool verifyOrMakeA( QHacc *, QHaccLineEdit * );
	void keyPressEvent( QKeyEvent * );
	void accept();
	

  protected slots:
		void conditionallyAccept();
  void fillInForPayee( const QString& );
  void moveInSum( bool );
  void openSplitEditor();
	void makeDAccts();
	void setModel();

 signals:
  void closed();
};

class SplitEditor : public QDialog {
  Q_OBJECT
    public:
  SplitEditor( QHacc *, QHaccTable *, QWidget * =0, const char * =0 );
  ~SplitEditor();
  
  void getSplits( QHaccTable *& );
	void setSplits( QHaccTable * );

 private:
  AccountArea * area;

	private slots:
		void addSplitToArea();
};

class AccountArea : public QScrollView {
  Q_OBJECT
    public:
  AccountArea( QHacc *, QHaccTable *, QWidget * =0, const char * =0 );
  ~AccountArea();
  
  void getSplits( QHaccTable *& );
  void addSplit( const TableRow& );

 protected:
  int lineSize() const;
	uint getNumSplits() const;
  QList<QHaccChoiceEdit> accounts;
  QList<QHaccLineEdit> credits;
  QList<QHaccLineEdit> debits;
	QHacc * engine;
	QHeader * header;
};

#endif
